C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPBAND                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Plots bands along symmetry lines.                  *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 01.07.97/mbs                                       *
C *  Remarks....: Search for "VMS", "AIX" or "MS-Windows".           *
C *               Changed for use with PGPLOT graphics library.      *
C *                                                                  *
C *  Called from: BANDPL,BPGRPH,BPRDRW                               *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSCI,PGSLW,PGSWIN,PGSVP,PGLINE    *
C *                                                                  *
C ********************************************************************

      SUBROUTINE BPBAND(VPXOFF,VPYOFF,VPXSCT,VPYAXH,EMIN,EMAX,NSYM,
     .                  IFAIL)
C
C NAG/GKS version.
C
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)
      INCLUDE 'sizes'

      DIMENSION VPXSCT(MAXSYM+1)
      DIMENSION BNDENE(MAXBA,MAXPTS),XK(MAXPTS),YBNDEN(MAXPTS)
C
C NAG/GKS version.
C
C      REAL WLINE
C
      IFAIL=0
      IPEN=1
      MARGIN=0
      METHOD=1
      XOFSUM=VPXOFF
      VPYEND=VPYOFF+VPYAXH
C
C Select pen.
C
C NAG/GKS version.
C
C      CALL J06YMF(IPEN)
C
C PGPLOT version.
C
      CALL PGSCI(IPEN)
C
C Set line width scale factor.
C
C      WLINE=1.0
C      CALL GSLWSC(WLINE)
C
C PGPLOT version.
C
      IWLINE=1
      CALL PGSLW(IWLINE)
C
C Read UNIT 2: Outputfile of Bandfit, contains interpolated band data
C
C MS-Windows and Windows NT version: Add FILE='fort.2' in OPEN statement.
C VMS and AIX version: Remove FILE='fort.2'.
C
      OPEN(UNIT=2,STATUS='OLD',ERR=950,FILE='fort.2')
C      OPEN(UNIT=2,STATUS='OLD',ERR=950)
      READ(2,'(I2)') IDUMMY
      DO I=1,NSYM
        READ(2,'(2I3)') NPTS,NBAND
        DO J=1,NPTS
C
C NAG/GKS version.
C
C          XK(J)=DBLE(FLOAT(J))
C
C PGPLOT version.
C
           XK(J)=FLOAT(J)
        END DO
        DO J=1,NBAND
          READ(2,'(F10.5)') (BNDENE(J,K),K=1,NPTS)
        END DO
C
C Set viewport and data region for corresponding symmetry line.
C
C NAG/GKS version.
C
C        XNPTS=DBLE(FLOAT(NPTS))
C        CALL J06WBF(1.0,XNPTS,EMIN,EMAX,MARGIN)
C        VPXEND=VPXOFF+VPXSCT(I+1)
C        CALL J06WCF(XOFSUM,VPXEND,VPYOFF,VPYEND)
C
C PGPLOT version.
C
        XNPTS=FLOAT(NPTS)
        CALL PGSWIN(1.0,XNPTS,EMIN,EMAX)
        VPXEND=VPXOFF+VPXSCT(I+1)
        CALL PGSVP(XOFSUM,VPXEND,VPYOFF,VPYEND)
C
C Plot bands.

        DO J=1,NBAND
          DO K=1,NPTS
            YBNDEN(K)=BNDENE(J,K)
          END DO
C
C NAG/GKS version. Use piecewise monotonic interpolation method for
C drawing curves.
C
C          CALL J06CAF(XK,YBNDEN,NPTS,METHOD,IFAIL)
C
C PGPLOT version. Draw straight lines.
C
          CALL PGLINE(NPTS,XK,YBNDEN)
        END DO
        XOFSUM=VPXEND
      END DO    
      CLOSE(UNIT=2)
C
C Use a cheap trick for output of whole polyline buffer.
C NAG/GKS version.
C
C      CALL J06YAF(1.0D0,EMIN)
C
C Set line width scale factor.
C
C NAG/GKS version.
C
C      WLINE=3.0
C      CALL GSLWSC(WLINE)
C      CALL GUWK(1,1)
C
C PGPLOT version.
C
      IWLINE=3
      CALL PGSLW(IWLINE)
      RETURN

950   IFAIL=1
      CLOSE(UNIT=2)
      RETURN
      END