C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPRAW                                                           *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Plots raw bands along symmetry line.               *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 19.08.92                                           *
C *  Revision...: 28.04.93/mbs, 01.07.97/mbs                         *
C *  Remarks....: Changed for use with PGPLOT library.               *
C *                                                                  *
C *  Called from: BANDPL                                             *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSCI,PGSCH,PGSCF,PGSWIN,PGSVP,    *
C *               PGTEXT,PGLINE,PGPT                                 * 
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE BPRAW(X,Y,ERAW,EMIN,EMAX,XLEN,VPXOFF,VPYOFF,VPXAXW,
     .                 VPYAXH,NKLIN,NNOFIT,NBFIT,IDIREC)

C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)
      INCLUDE 'sizes'
 
      DIMENSION   X(NKLIN),ERAW(MAXBA,MAXK),Y(NKLIN)
      CHARACTER*4 TEXT
      CHARACTER*7 TEXT2
 
      XLO=0.0
      XHI=XLEN
      IPEN=1
      IMARK=2
      MARGIN=0
      METHOD=1
      VPXEND=VPXOFF+VPXAXW
      VPYEND=VPYOFF+VPYAXH
      ERANGE=EMAX-EMIN
      XTXTOF=0.0
      YTXTOF=0.01*ERANGE
C
C Select pen.
C
C NAG/GKS version.
C
C      CALL J06YMF(IPEN)
C
C PGPLOT version.
C
      CALL PGSCI(IPEN)

C
C Select high marker quality. Set marker size.
C
C NAG/GKS version.
C
C      IMARKQ=2
C      CALL J06XFF(IMARKQ)
C      CALL J06XHF(SZMARK)
C
C PGPLOT version.
C
      CALL PGSCH(SZMARK)
C
C Select font.
C
C NAG/GKS version.
C
      IFONT=1
C      CALL J06YSF(IFONT)
C
C PGPLOT version.
C
      CALL PGSCF(IFONT)
C
C Set viewport and data region for annotating energy axis.
C
C NAG/GKS version.
C
C      CALL J06WBF(0.0,1.0,0.0,1.0,MARGIN)
C      CALL J06WCF(0.0,1.0,0.0,1.0)
C
C PGPLOT version.
C
      CALL PGSWIN(0.0,1.0,0.0,1.0)
      CALL PGSVP(0.0,1.0,0.0,1.0)
C
C Set character size and spacing for annotating energy axis.
C
C NAG/GKS version.
C
C      CWIDTH=0.01
C      CHEIGT=0.01
C      CALL J06YKF(CWIDTH,CHEIGT)
C      CALL J06YLF(CWIDTH*1.01,0.0)
C
C Annotate energy axis (as simple as possible).
C
      WRITE(TEXT2(1:7),'(F7.2)') EMIN
C
C NAG/GKS version.
C
C      CALL J06YAF(0.01,VPYOFF-CHEIGT/2.0)
C      CALL J06YHF(TEXT2,7)
C
C PGPLOT version.
C
      CALL PGTEXT(0.01,VPYOFF-0.01,TEXT2)
      WRITE(TEXT2(1:7),'(F7.2)') EMAX
C
C NAG/GKS version.
C
C      CALL J06YAF(0.01,VPYEND-CHEIGT/2.0)
C      CALL J06YHF(TEXT2,7)
C
C PGPLOT version.
C
      CALL PGTEXT(0.01,VPYEND-0.01,TEXT2)
C
C Set viewport and data region for corresponding symmetry line.
C
C NAG/GKS version.
C
C      CALL J06WBF(XLO,XHI,EMIN,EMAX,MARGIN)
C      CALL J06WCF(VPXOFF,VPXEND,VPYOFF,VPYEND)
C 
C PGPLOT version.
C
      CALL PGSWIN(XLO,XHI,EMIN,EMAX)
      CALL PGSVP(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C Set character size and spacing for band numbers.
C
C NAG/GKS version.
C 
C      CWIDTH=0.02*XLEN
C      CHEIGT=0.02*ERANGE
C      CALL J06YKF(CWIDTH,CHEIGT)
C      CALL J06YLF(CWIDTH*1.01,0.0)
C
C Because graphic clipping does not always work correctly, clipping is
C done "by hand".
C
      ILO=0
      IHI=0
      DO I=NNOFIT+1,NNOFIT+NBFIT
        DO  K=1,NKLIN
          IF (ILO .EQ. 0) THEN
            IF (ERAW(I,K) .GE. EMIN) THEN
              ILO=I
              GOTO 10
            ENDIF
          ENDIF
        END DO
      END DO
10    DO I=NNOFIT+NBFIT,ILO,-1
        DO K=1,NKLIN
          IF (IHI .EQ. 0) THEN
            IF (ERAW(I,K) .LE. EMAX) THEN
              IHI=I
              GOTO 20
            ENDIF
          ENDIF
        END DO
      END DO
C
C Plot bands, use piecewise monotonic interpolation method for drawing
C curves (NAG/GKS version).
C
20    DO I=ILO,IHI
        DO K=1,NKLIN
          Y(K)=ERAW(I,K)
        END DO
        IFAIL=0
C
C NAG/GKS version.
C
C        CALL J06CAF(X,Y,NKLIN,METHOD,IFAIL)
C
C PGPLOT version.
C
        CALL PGLINE(NKLIN,X,Y)
        IFAIL=0
        ITYPE=0
C
C NAG/GKS version.
C
C        CALL J06BAF(X,Y,NKLIN,ITYPE,IMARK,IFAIL)
C
C PGPLOT version.
C
        CALL PGPT(NKLIN,X,Y,4)
        WRITE(TEXT(1:4),'(I4)') I
        YTXT=Y(1)+YTXTOF
        IF (IDIREC .LT. 0)  YTXT=Y(NKLIN)+YTXTOF
C
C NAG/GKS version.
C
C        CALL J06YAF(XTXTOF,YTXT)
C        CALL J06YHF(TEXT,4)
C
C PGPLOT version.
C
        CALL PGTEXT(XTXTOF,YTXT,TEXT)
      END DO
C
C Use a cheap trick for output of whole polyline buffer.
C
C NAG/GKS version.
C
C      CALL J06YAF(XLEN,EMIN)
C      CALL GUWK(1,1)
      RETURN
      END
