C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPTICA                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Draws labelled and ticed energy axis.              *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 01.07.97/mbs                                       *
C *  Remarks....: Changed for use with PGPLOT graphics library.      *
C *                                                                  *
C *  Called from: BPGRPH,BPRDRW                                      *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSWIN,PGSVP,PGSCI,PGSCF,PGBOX     *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .                  ETCMAX,THRU,TICDIS,TICDS2,ITICSD,INUM)
C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL (A-H,O-Z)
      INCLUDE 'sizes'
 
      CHARACTER CDUMMY(1)
 
      PARAMETER (MARGIN=0,IPEN=1)
      PARAMETER (IFONT=2)

      CHARACTER*8 AXSTRV
 
      VPYEND=VPYOFF+VPYAXH
C
C NAG/GKS version.
C Set data range and viewport. Data range must be widened so that all
C labels will be fully visible.
C
C      DRMIN=EMIN-ERANGE/VPYAXH*VPYOFF
C      DRMAX=EMAX+ERANGE/VPYAXH*(1.0-VPYEND)
C      CALL J06WBF(0.0,1.0,DRMIN,DRMAX,MARGIN)
C      CALL J06WCF(0.0,1.0,0.0,1.0)
C
C PGPLOT version.
C Uses viewport settings for placement of axes.
C
      CALL PGSWIN(THRU,1.0,ETCMIN,ETCMAX)
      YSTART=VPYOFF+(ETCMIN-EMIN)/ERANGE*VPYAXH
      YEND=VPYOFF+(ETCMAX-EMIN)/ERANGE*VPYAXH
      CALL PGSVP(THRU,1.0,YSTART,YEND)
      AXSTRV='AB'
      IF (THRU .EQ. VPXOFF) AXSTRV='ABN'
C
C Select pen and font.
C
C NAG/GKS version.
C
C      CALL J06YMF(IPEN)
C
C PGPLOT version.
C
      CALL PGSCI(IPEN)
      CALL PGSCF(IFONT)
C
C Draw labeled energy axis.
C
C NAG/GKS version.
C
C      IAXIS=2
C      ILABS=0
C      NLABEL=1
C      NUMINT=0
C
C      IF ((ITICSD .EQ. 1) .OR. (ITICSD .EQ. 3)) THEN 
C        ITICK=1
C        IF ((INUM .EQ. 1) .AND. (TICDS2 .GT. 0.0)) THEN
C          CALL J06AQF(IAXIS,EMIN,ETCMAX,THRU,TICDS2,ETCMIN,NUMINT,ITICK,
C     .                INUM,ILABS,CDUMMY,NLABEL)
C        ENDIF
C        INUM=0
C        CALL J06AQF(IAXIS,EMIN,ETCMAX,THRU,TICDIS,ETCMIN,NUMINT,ITICK,
C     .              INUM,ILABS,CDUMMY,NLABEL)
C      ENDIF
C      IF ((ITICSD .EQ. 2) .OR. (ITICSD .EQ. 3)) THEN
C        ITICK=-1
C        IF ((INUM .EQ. 1) .AND. (TICDS2 .GT. 0.0)) THEN
C          CALL J06AQF(IAXIS,EMIN,ETCMAX,THRU,TICDS2,ETCMIN,NUMINT,ITICK,
C     .                INUM,ILABS,CDUMMY,NLABEL)
C        ENDIF
C        INUM=0
C        CALL J06AQF(IAXIS,EMIN,ETCMAX,THRU,TICDIS,ETCMIN,NUMINT,ITICK,
C     .              INUM,ILABS,CDUMMY,NLABEL)
C      ENDIF
C
C PGPLOT version.
C
      IF (TICDS2 .GT. 0.0) THEN
        ISTICS=NINT(TICDS2/TICDIS)
      ENDIF
      IF ((ITICSD .EQ. 1) .OR. (ITICSD .EQ. 3)) THEN
        CALL PGBOX('',TICDS2,ISTICS,AXSTRV//'ITSV',TICDS2,ISTICS)
      ENDIF
      IF ((ITICSD .EQ. 2) .OR. (ITICSD .EQ. 3)) THEN
        CALL PGBOX('',TICDS2,ISTICS,AXSTRV//'TSV',TICDS2,ISTICS)
      ENDIF 

      RETURN
      END