C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPTRAP                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Translates label and text statements into plotable *
C *               instructions. Special characters (greek) and plot- *
C *               ting instructions follow after an '\' in the       *
C *               string. If a plotting keyword is not recognized,   *
C *               it will be output as character string.             *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 26.06.97                                           *
C *  Revision...: -                                                  *
C *  Remarks....: Works with PGPLOT graphics subroutine library.     *
C *               Identical with DPTRAP routine of DOSPLOT.          *
C *                                                                  *
C *  Called from: BPGRPH                                             *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSCH,PGPTXT                       *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE BPTRAP(STRNG2,PENX,PENY,TANGLE,IPOS)

      IMPLICIT REAL(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      PARAMETER (MAXGRK=26)
      PARAMETER (MAXCMD=17)
 
      DIMENSION    LESCAP(MAXCMD)
      CHARACTER*80 TSTRNG,STRING,STRNG2
      CHARACTER*8  UGREEK(MAXGRK),LGREEK(MAXGRK),KEYWRD,ESCAPE(MAXCMD)
      CHARACTER*24 TESCAP(MAXCMD)
      CHARACTER    COUT,CTEMP,BSL
      LOGICAL      FEXIT

      PARAMETER (BSL='\\')
      DATA UGREEK/'ALPHA   ','BETA    ','CHI     ','DELTA   ',
     .            'EPSILON ','PHI     ','GAMMA   ','THETA   ',
     .            'IOTA    ','        ','KAPPA   ','LAMBDA  ',
     .            'MU      ','NU      ','OMIKRON ','PI      ',
     .            'PSI     ','RHO     ','SIGMA   ','TAU     ',
     .            'UPSILON ','        ','OMEGA   ','XI      ',
     .            'ETA     ','ZETA    '/
      DATA LGREEK/'alpha   ','beta    ','chi     ','delta   ',
     .            'epsilon ','phi     ','gamma   ','theta   ',
     .            'iota    ','        ','kappa   ','lambda  ',
     .            'mu      ','nu      ','omikron ','pi      ',
     .            'psi     ','rho     ','sigma   ','tau     ',
     .            'upsilon ','        ','omega   ','xi      ',
     .            'eta     ','zeta    '/

      DATA ESCAPE/'super   ','endsuper','sub     ','endsub  ',
     .            'back    ','space   ','cdot    ','cross   ',
     .            'angstrom',
     .            'arrowrgt','arrowup ','arrowlft','arrowdwn',
     .            'infinity','+-      ','piovera ','\       '/

      DATA TESCAP/'u','d','d','u','b',' ','.','x','A',
     .            '(2261)','(2262)','(2263)','(2264)',
     .            '(2270)','(2233)',
     .            'u\gp\b\d\(0240)\b\da\u','\'/

C
C Depending on your FORTRAN compiler, you may have to replace the
C \ character by \\ in the DATA statements.  
C
C      DATA ESCAPE/'super   ','endsuper','sub     ','endsub  ',
C     .            'back    ','space   ','cdot    ','cross   ',
C     .            'angstrom',
C     .            'arrowrgt','arrowup ','arrowlft','arrowdwn',
C     .            'infinity','+-      ','piovera ','\\       '/
C      DATA TESCAP/'u','d','d','u','b',' ','.','x','A',
C     .            '(2261)','(2262)','(2263)','(2264)',
C     .            '(2270)','(2233)',
C     .            'u\\gp\\b\\d\(0240)\\b\\da\\u','\\'/

      DATA LESCAP/1,1,1,1,1,1,1,1,1,6,6,6,6,6,6,22,1/
      
      DO I=1,LEN(TSTRNG)
        TSTRNG(I:I)=' '
        STRING(I:I)=' '
      END DO
 
      STRING=STRNG2
      LSTRNG=LEN(STRING)
      LSTR2=LSTRNG
      DO I=LSTR2,1,-1
        IF (STRING(I:I) .GT. ' ') EXIT
        LSTRNG=LSTRNG-1
      END DO
C
C Set initial font size.
C
      FNTHGT=FHEIGT
      CALL PGSCH(FNTHGT)

      TPOS=0.0
      IF (IPOS .EQ. 1) TPOS=0.5
      IF (IPOS .EQ. 2) TPOS=1.0
C
C Parse through string.
C
      ICNT=0
      ITCNT=1 
      DO I=1,LSTRNG
        ICNT=ICNT+1
        IF (ICNT .GT. LSTRNG) EXIT
        FEXIT=.FALSE.
        COUT=STRING(ICNT:ICNT)
C
C Check for keywords.
C
        IF (COUT .EQ. CHAR(92)) THEN
          DO J=1,8
            INDX=ICNT+J
            IF (STRING(INDX:INDX) .EQ. ' ') EXIT
              KEYWRD(J:J)=STRING(INDX:INDX)
          END DO
          DO K=J,8
            KEYWRD(K:K)=' '
          END DO
C
C Check for Greek characters.
C
          DO J=1,MAXGRK
            IF (KEYWRD .EQ. UGREEK(J)) THEN
              FEXIT=.TRUE.
              TSTRNG(ITCNT:ITCNT+2)=BSL//'g'//CHAR(J+64)
              ITCNT=ITCNT+3
            ELSE IF (KEYWRD .EQ. LGREEK(J)) THEN
              FEXIT=.TRUE.
              TSTRNG(ITCNT:ITCNT+2)=BSL//'g'//CHAR(J+96)
              ITCNT=ITCNT+3
            ELSE
            ENDIF
          END DO
C
C Convert keywrd to lowercase.
C
          DO J=1,8
            CTEMP=KEYWRD(J:J)
            IF ((CTEMP .GE. 'A') .AND. (CTEMP .LE. 'Z')) THEN
              KEYWRD(J:J)=CHAR(ICHAR(CTEMP)+32)
            ENDIF
          END DO

C
C Check for special commands.
C
          IF (.NOT. FEXIT) THEN
            DO J=1,MAXCMD
              IF (KEYWRD .EQ. ESCAPE(J)) THEN
                FEXIT=.TRUE.
                TSTRNG(ITCNT:ITCNT+LESCAP(J))=BSL//TESCAP(J)
                ITCNT=ITCNT+LESCAP(J)+1
              ENDIF
            ENDDO
          ENDIF
C
C No special commands found. Copy char.
C 
          IF (.NOT. FEXIT) THEN
            TSTRNG(ITCNT:ITCNT)=COUT
            ITCNT=ITCNT+1
          ELSE
            ICNT=INDX
          ENDIF
        ELSE
          TSTRNG(ITCNT:ITCNT)=COUT
          ITCNT=ITCNT+1
        ENDIF
      END DO

      CALL PGPTXT(PENX,PENY,TANGLE,TPOS,TSTRNG)
      RETURN
      END
