C ********************************************************************
C * CEDiT - Crystal Electronic Dipole-induced Transitions            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE DODELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates <d|O|d> offdiagonal elements of the     *
C *               transition matrix. The operator operates to the    *
C *               left function.                                     *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: August 1992                                        *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *                                                                  *
C ********************************************************************

      SUBROUTINE DODELE(M, HQZ, N2, SK11, SK12, SK21, SK22, C11, 
     .                  C12, C21, C22, R, PTR, DTR, FTR, LDIST,
     .                  POL, CURRI, CURRJ)

      IMPLICIT NONE

        DOUBLE PRECISION  M
        INTEGER           HQZ, N2
        DOUBLE PRECISION  SK11, SK12, SK21, SK22, C11, C12, C21, C22, R
        DOUBLE PRECISION  PTR(9), DTR(25), FTR(49), LDIST
        CHARACTER*1       POL
        INTEGER           CURRI, CURRJ

C CONST
      INCLUDE 'dconst.inc'
           
C VAR
      DOUBLE PRECISION A1(20), B1(20), A2(20), B2(20), A3(20) 
      DOUBLE PRECISION B3(20)
      DOUBLE PRECISION A4(20), B4(20), DDOS1, DDOS2, DDOS3, DDOS4
      DOUBLE PRECISION DDOP1, DDOP2, DDOP3, DDOP4, DDOD1, DDOD2, DDOD3
      DOUBLE PRECISION DDOD4, DDO1(5,5), DDO2(5,5), DDO3(5,5), DDO4(5,5)
      DOUBLE PRECISION PDOS1, PDOS2, PDOS3, PDOS4, PDOP1, PDOP2, PDOP3 
      DOUBLE PRECISION PDOP4
      DOUBLE PRECISION PDO1(3,5), PDO2(3,5), PDO3(3,5), PDO4(3,5)
      DOUBLE PRECISION FDOS1, FDOS2, FDOS3, FDOS4, FDOP1, FDOP2, FDOP3 
      DOUBLE PRECISION FDOP4
      DOUBLE PRECISION FDOD1, FDOD2, FDOD3, FDOD4, FDO1(5,7), FDO2(5,7)
      DOUBLE PRECISION FDO3(5,7), FDO4(5,7)
      INTEGER L1, L2, ML, MAX, N1
      DOUBLE PRECISION NORM1, NORM2

      INCLUDE 'dfns.inc'
      INCLUDE 'dnorm.inc'

C BEGIN

C Berechne zuerst den "normalen" d-d-Overlap

      N1 = HQZ
      L1 = 2
      L2 = 2
      ML = 0
      MAX = N1 + N2
      
      CALL ABFNS(A1, B1, SK11, SK21, R, MAX)
      CALL ABFNS(A2, B2, SK11, SK22, R, MAX)
      CALL ABFNS(A3, B3, SK12, SK21, R, MAX)
      CALL ABFNS(A4, B4, SK12, SK22, R, MAX)

      CALL LOVLAP(DDOS1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOS2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOS3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOS4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)

      ML = 1

      CALL LOVLAP(DDOP1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOP2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOP3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOP4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)
      DDOP1 = -DDOP1
      DDOP2 = -DDOP2
      DDOP3 = -DDOP3
      DDOP4 = -DDOP4

      ML = 2

      CALL LOVLAP(DDOD1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOD2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOD3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DDOD4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)

      CALL DDOVL(DDO1, DDOS1, DDOP1, DDOD1, DTR)
      CALL DDOVL(DDO2, DDOS2, DDOP2, DDOD2, DTR)
      CALL DDOVL(DDO3, DDOS3, DDOP3, DDOD3, DTR)
      CALL DDOVL(DDO4, DDOS4, DDOP4, DDOD4, DTR)

C Ende der Berechnung des d-d-Overlaps

C Berechnung des p-d-Overlaps nachdem man den Operator auf das d-Orbital
C angewandt hat.

      N1 = N1 + 1
      L1 = 1
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A1, B1, SK11, SK21, R, MAX)
      CALL ABFNS(A2, B2, SK11, SK22, R, MAX)
      CALL ABFNS(A3, B3, SK12, SK21, R, MAX)
      CALL ABFNS(A4, B4, SK12, SK22, R, MAX)

      CALL LOVLAP(PDOS1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PDOS2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PDOS3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PDOS4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)
      PDOS1 = -PDOS1
      PDOS2 = -PDOS2
      PDOS3 = -PDOS3
      PDOS4 = -PDOS4

      ML = 1

      CALL LOVLAP(PDOP1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PDOP2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PDOP3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PDOP4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)

      CALL PDOVL(PDO1, PDOS1, PDOP1, PTR, DTR)
      CALL PDOVL(PDO2, PDOS2, PDOP2, PTR, DTR)
      CALL PDOVL(PDO3, PDOS3, PDOP3, PTR, DTR)
      CALL PDOVL(PDO4, PDOS4, PDOP4, PTR, DTR)

C Berechne nun den f-d-Overlap nachdem man den Operator auf das
C d-Orbital angewandt hat.

      L1 = 3
      ML = 0

      CALL LOVLAP(FDOS1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FDOS2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML ,N1, N2)
      CALL LOVLAP(FDOS3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FDOS4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)
      FDOS1 = -FDOS1
      FDOS2 = -FDOS2
      FDOS3 = -FDOS3
      FDOS4 = -FDOS4

      ML = 1

      CALL LOVLAP(FDOP1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FDOP2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML ,N1, N2)
      CALL LOVLAP(FDOP3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FDOP4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)

      ML = 2

      CALL LOVLAP(FDOD1, A1, B1, SK11, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FDOD2, A2, B2, SK11, SK22, R, L1, L2,
     .            ML ,N1, N2)
      CALL LOVLAP(FDOD3, A3, B3, SK12, SK21, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FDOD4, A4, B4, SK12, SK22, R, L1, L2,
     .            ML, N1, N2)
      FDOD1 = -FDOD1
      FDOD2 = -FDOD2
      FDOD3 = -FDOD3
      FDOD4 = -FDOD4

      CALL DFOVL(FDO1, FDOS1, FDOP1, FDOD1, DTR, FTR)
      CALL DFOVL(FDO2, FDOS2, FDOP2, FDOD2, DTR, FTR)
      CALL DFOVL(FDO3, FDOS3, FDOP3, FDOD3, DTR, FTR)
      CALL DFOVL(FDO4, FDOS4, FDOP4, FDOD4, DTR, FTR)

      IF (CURRI .EQ. 5) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C11*C21 * I5X(PDO1(1,CURRJ-4), DDO1(1,CURRJ-4),
     .                       FDO1(CURRJ-4,6), FDO1(CURRJ-4,2), SK11) +
     .             C11*C22 * I5X(PDO2(1,CURRJ-4), DDO2(1,CURRJ-4),
     .                       FDO2(CURRJ-4,6), FDO2(CURRJ-4,2), SK11) +
     .             C12*C21 * I5X(PDO3(1,CURRJ-4), DDO3(1,CURRJ-4),
     .                       FDO3(CURRJ-4,6), FDO3(CURRJ-4,2), SK12) +
     .             C12*C22 * I5X(PDO4(1,CURRJ-4), DDO4(1,CURRJ-4),
     .                       FDO4(CURRJ-4,6), FDO4(CURRJ-4,2), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C11*C21 * I5Y(PDO1(2,CURRJ-4), DDO1(1,CURRJ-4),
     .                       FDO1(CURRJ-4,7), FDO1(CURRJ-4,3), SK11) +
     .             C11*C22 * I5Y(PDO2(2,CURRJ-4), DDO2(1,CURRJ-4),
     .                       FDO2(CURRJ-4,7), FDO2(CURRJ-4,3), SK11) + 
     .             C12*C21 * I5Y(PDO3(2,CURRJ-4), DDO3(1,CURRJ-4),
     .                       FDO3(CURRJ-4,7), FDO3(CURRJ-4,3), SK12) +
     .             C12*C22 * I5Y(PDO4(2,CURRJ-4), DDO4(1,CURRJ-4),
     .                       FDO4(CURRJ-4,7), FDO4(CURRJ-4,3), SK12))
        ELSE
          M = M * (C11*C21 * XYZ(DDO1(1,CURRJ-4),FDO1(CURRJ-4,5),SK11) +
     .             C11*C22 * XYZ(DDO2(1,CURRJ-4),FDO2(CURRJ-4,5),SK11) +
     .             C12*C21 * XYZ(DDO3(1,CURRJ-4),FDO3(CURRJ-4,5),SK12) +
     .             C12*C22 * XYZ(DDO4(1,CURRJ-4),FDO4(CURRJ-4,5),SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 6) THEN
        
        IF (POL .EQ. 'X') THEN
          M = M * (C11*C21 * I6XY(PDO1(1,CURRJ-4), DDO1(2,CURRJ-4), 
     .                            FDO1(CURRJ-4,2), SK11) +
     .             C11*C22 * I6XY(PDO2(1,CURRJ-4), DDO2(2,CURRJ-4),
     .                            FDO2(CURRJ-4,2), SK11) +
     .             C12*C21 * I6XY(PDO3(1,CURRJ-4), DDO3(2,CURRJ-4),
     .                            FDO3(CURRJ-4,2), SK12) +
     .             C12*C22 * I6XY(PDO4(1,CURRJ-4), DDO4(2,CURRJ-4),
     .                            FDO4(CURRJ-4,2), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C11*C21 * I6XY(PDO1(2,CURRJ-4), DDO1(2,CURRJ-4),
     .                            FDO1(CURRJ-4,3), SK11) +
     .             C11*C22 * I6XY(PDO2(2,CURRJ-4), DDO2(2,CURRJ-4),
     .                            FDO2(CURRJ-4,3), SK11) +
     .             C12*C21 * I6XY(PDO3(2,CURRJ-4), DDO3(2,CURRJ-4),
     .                            FDO3(CURRJ-4,3), SK12) +
     .             C12*C22 * I6XY(PDO4(2,CURRJ-4), DDO4(2,CURRJ-4),
     .                            FDO4(CURRJ-4,3), SK12))
        ELSE
          M = M * (C11*C21 * I6Z(PDO1(3,CURRJ-4), DDO1(2,CURRJ-4),
     .                           FDO1(CURRJ-4,1), SK11) +
     .             C11*C22 * I6Z(PDO2(3,CURRJ-4), DDO2(2,CURRJ-4),
     .                           FDO2(CURRJ-4,1), SK11) +
     .             C12*C21 * I6Z(PDO3(3,CURRJ-4), DDO3(2,CURRJ-4),
     .                           FDO3(CURRJ-4,1), SK12) +
     .             C12*C22 * I6Z(PDO4(3,CURRJ-4), DDO4(2,CURRJ-4),
     .                           FDO4(CURRJ-4,1), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 7) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C11*C21 * I7X(PDO1(2,CURRJ-4), DDO1(3,CURRJ-4),
     .                       FDO1(CURRJ-4,7), FDO1(CURRJ-4,3), SK11) +
     .             C11*C22 * I7X(PDO2(2,CURRJ-4), DDO2(3,CURRJ-4),
     .                       FDO2(CURRJ-4,7), FDO2(CURRJ-4,3), SK11) +
     .             C12*C21 * I7X(PDO3(2,CURRJ-4), DDO3(3,CURRJ-4),
     .                       FDO3(CURRJ-4,7), FDO3(CURRJ-4,3), SK12) +
     .             C12*C22 * I7X(PDO4(2,CURRJ-4), DDO4(3,CURRJ-4),
     .                       FDO4(CURRJ-4,7), FDO4(CURRJ-4,3), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C11*C21 * I7Y(PDO1(1,CURRJ-4), DDO1(3,CURRJ-4),
     .                       FDO1(CURRJ-4,6), FDO1(CURRJ-4,2), SK11) +
     .             C11*C22 * I7Y(PDO2(1,CURRJ-4), DDO2(3,CURRJ-4),
     .                       FDO2(CURRJ-4,6), FDO2(CURRJ-4,2), SK11) +
     .             C12*C21 * I7Y(PDO3(1,CURRJ-4), DDO3(3,CURRJ-4),
     .                       FDO3(CURRJ-4,6), FDO3(CURRJ-4,2), SK12) +
     .             C12*C22 * I7Y(PDO4(1,CURRJ-4), DDO4(3,CURRJ-4),
     .                       FDO4(CURRJ-4,6), FDO4(CURRJ-4,2), SK12))
        ELSE
          M = M * (C11*C21 * XYZ(DDO1(3,CURRJ-4),FDO1(CURRJ-4,4),SK11) +
     .             C11*C22 * XYZ(DDO2(3,CURRJ-4),FDO2(CURRJ-4,4),SK11) +
     .             C12*C21 * XYZ(DDO3(3,CURRJ-4),FDO3(CURRJ-4,4),SK12) +
     .             C12*C22 * XYZ(DDO4(3,CURRJ-4),FDO4(CURRJ-4,4),SK12))
        ENDIF
 
      ELSEIF (CURRI .EQ. 8) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C11*C21 * I8X(PDO1(3,CURRJ-4), DDO1(4,CURRJ-4),
     .                       FDO1(CURRJ-4,5), FDO1(CURRJ-4,1), SK11) +
     .             C11*C22 * I8X(PDO2(3,CURRJ-4), DDO2(4,CURRJ-4),
     .                       FDO2(CURRJ-4,5), FDO2(CURRJ-4,1), SK11) +
     .             C12*C21 * I8X(PDO3(3,CURRJ-4), DDO3(4,CURRJ-4),
     .                       FDO3(CURRJ-4,5), FDO3(CURRJ-4,1), SK12) +
     .             C12*C22 * I8X(PDO4(3,CURRJ-4), DDO4(4,CURRJ-4),
     .                       FDO4(CURRJ-4,5), FDO4(CURRJ-4,1), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C11*C21 * XYZ(DDO1(4,CURRJ-4),FDO1(CURRJ-4,4),SK11) +
     .             C11*C22 * XYZ(DDO2(4,CURRJ-4),FDO2(CURRJ-4,4),SK11) +
     .             C12*C21 * XYZ(DDO3(4,CURRJ-4),FDO3(CURRJ-4,4),SK12) +
     .             C12*C22 * XYZ(DDO4(4,CURRJ-4),FDO4(CURRJ-4,4),SK12))  
        ELSE
          M = M * (C11*C21 * I8Z(PDO1(1,CURRJ-4), DDO1(4,CURRJ-4),
     .                           FDO1(CURRJ-4,2), SK11) +
     .             C11*C22 * I8Z(PDO2(1,CURRJ-4), DDO2(4,CURRJ-4),
     .                           FDO2(CURRJ-4,2), SK11) +
     .             C12*C21 * I8Z(PDO3(1,CURRJ-4), DDO3(4,CURRJ-4),
     .                           FDO3(CURRJ-4,2), SK12) +
     .             C12*C22 * I8Z(PDO4(1,CURRJ-4), DDO4(4,CURRJ-4),
     .                           FDO4(CURRJ-4,2), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 9) THEN
 
        IF (POL .EQ. 'X') THEN
          M = M * (C11*C21 * XYZ(DDO1(5,CURRJ-4),FDO1(CURRJ-4,4),SK11) +
     .             C11*C22 * XYZ(DDO2(5,CURRJ-4),FDO2(CURRJ-4,4),SK11) +
     .             C12*C21 * XYZ(DDO3(5,CURRJ-4),FDO3(CURRJ-4,4),SK12) +
     .             C12*C22 * XYZ(DDO4(5,CURRJ-4),FDO4(CURRJ-4,4),SK12)) 
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C11*C21 * I9Y(PDO1(3,CURRJ-4), DDO1(5,CURRJ-4),
     .                       FDO1(CURRJ-4,5), FDO1(CURRJ-4,1), SK11) +
     .             C11*C22 * I9Y(PDO2(3,CURRJ-4), DDO2(5,CURRJ-4),
     .                       FDO2(CURRJ-4,5), FDO2(CURRJ-4,1), SK11) +
     .             C12*C21 * I9Y(PDO3(3,CURRJ-4), DDO3(5,CURRJ-4),
     .                       FDO3(CURRJ-4,5), FDO3(CURRJ-4,1), SK12) +
     .             C12*C22 * I9Y(PDO4(3,CURRJ-4), DDO4(5,CURRJ-4), 
     .                       FDO4(CURRJ-4,5), FDO4(CURRJ-4,1), SK12))
        ELSE
          M = M * (C11*C21 * I9Z(PDO1(2,CURRJ-4), DDO1(5,CURRJ-4),
     .                           FDO1(CURRJ-4,3), SK11) +
     .             C11*C22 * I9Z(PDO2(2,CURRJ-4), DDO2(5,CURRJ-4),
     .                           FDO2(CURRJ-4,3), SK11) +
     .             C12*C21 * I9Z(PDO3(2,CURRJ-4), DDO3(5,CURRJ-4),
     .                           FDO3(CURRJ-4,3), SK12) +
     .             C12*C22 * I9Z(PDO4(2,CURRJ-4), DDO4(5,CURRJ-4),
     .                           FDO4(CURRJ-4,3), SK12))
        ENDIF

      ENDIF

      RETURN   
      END
C END
