C ********************************************************************
C * CEDiT - Crystal Electronic Dipole-induced Transitions            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE EULERD                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates the Euler-matrix for the projection     *
C *               of the d-orbitals.                                 *
C *                                                                  *
C *  Authors....: unknown                                            *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: July 1992 (Ruedi Rytz)                             *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE EULERD(CB, SB, SA, CA, DTR)

      IMPLICIT NONE

        DOUBLE PRECISION  CB, SB, SA, CA, DTR(25)

C CONST
      DOUBLE PRECISION SQRT3
      PARAMETER (SQRT3 = 1.73205D0)

C VAR
      DOUBLE PRECISION  CA2, SA2, CB2, SB2, CBSB, CASA, CB2SB2

C BEGIN
      CA2=CA**2
      SA2=SA*SA
      CB2=CB*CB
      SB2=SB*SB
      CBSB= CB*SB
      CASA= CA*SA
      CB2SB2= CB2-SB2
      DTR(1)= SQRT3*.5D0*SA2*CB2SB2
      DTR(2)= 1.D0-1.5D0*SA2
      DTR(3)= SQRT3*CBSB*SA2
      DTR(4)= SQRT3*CASA*CB
      DTR(5)= SQRT3*CASA*SB
      DTR(6)= CASA*CB2SB2
      DTR(7)= -SQRT3*CASA
      DTR(8)= 2.D0*CASA*CBSB
      DTR(9)= CB*(CA2-SA2)
      DTR(10)= SB*(CA2-SA2)
      DTR(11)= -2.D0*SA*CBSB
      DTR(12)= 0.0D0
      DTR(13)= SA* CB2SB2
      DTR(14)= -CA*SB
      DTR(15)=  CA*CB
      DTR(16)= .5D0*(1.D0+CA2)*CB2SB2
      DTR(17)= .5D0*SQRT3*SA2
      DTR(18)=  CBSB*(1.D0+CA2)
      DTR(19)= -CASA*CB
      DTR(20)= -CASA*SB
      DTR(21)= -2.D0*CA*CBSB
      DTR(22)= 0.0
      DTR(23)= CA*CB2SB2
      DTR(24)= SA*SB
      DTR(25)= -SA*CB

      RETURN
      END
C END
