C ********************************************************************
C * CEDiT - Crystal Electronic Dipole-induced Transitions            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE EULERF                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates the Euler-matrix for the projection of  *
C *               the f-orbitals.                                    *
C *                                                                  *
C *  Authors....: unknown                                            *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: July 1992 (Ruedi Rytz)                             *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE EULERF(CB, SB, SA, CA, FTR)

      IMPLICIT NONE

        DOUBLE PRECISION CB, SB, SA, CA, FTR(49)

C CONST
      DOUBLE PRECISION  SQRT6, SQRT10, SQRT15
      PARAMETER(SQRT6 = 2.449489D0, SQRT10 = 3.162277D0) 
      PARAMETER(SQRT15= 3.872982D0)

C VAR
      DOUBLE PRECISION  CA2, SA2, SB2, CB2, S3B
      DOUBLE PRECISION  C3B, S2A, C2B, SA3, S2B, C2A

C BEGIN
      CA2=CA*CA
      SA2=SA*SA
      CB2=CB*CB
      SB2=SB*SB
      S2B=2.0D00*SB*CB
      SA3=SA2*SA
      C2B=(CB2-SB2)
      S3B=(C2B*SB + S2B*CB)
      C3B=(C2B*CB - S2B*SB)
      S2A=2.0D0*SA*CA
      C2A=CA2-SA2

      FTR(1)=0.5D0*CA*(5.0D0*CA2 -3.0D0)
      FTR(2)=SQRT6*0.25D0*CB*SA*(5.0D0*CA2 -1.0D0)
      FTR(3)=SQRT6*0.25D0*SB*SA*(5.0D0*CA2 -1.0D0)
      FTR(4)=SQRT15*0.5D0*S2B*CA*SA2
      FTR(5)=SQRT15*0.5D0*C2B*CA*SA2
      FTR(6)=SQRT10*0.25D0*C3B*SA3
      FTR(7)=SQRT10*0.25D0*S3B*SA3
      FTR(8)=-SQRT6*0.25D0*SA*(5.0D0*CA2 -1.0D0)
      FTR(9)=0.25D0*CB*CA*(1.5D1*CA2 -1.1D1)
      FTR(10)=0.25D0*SB*CA*(1.5D1*CA2 -1.1D1)
      FTR(11)=SQRT10*0.25D0*S2B*SA*(3.0D0*CA2 - 1.0D0)
      FTR(12)=SQRT10*0.25D0*C2B*SA*(3.0D0*CA2 - 1.0D0)
      FTR(13)=SQRT15*0.25D0*C3B*CA*SA2
      FTR(14)=SQRT15*0.25D0*S3B*CA*SA2
      FTR(15)=0.0D0
      FTR(16)=-0.25D0*SB*(5.0D0*CA2 -1.0D0)
      FTR(17)=0.25D0*CB*(5.0D0*CA2 -1.0D0)
      FTR(18)=SQRT10*0.25D0*C2B*S2A
      FTR(19)=-SQRT10*S2B*S2A*0.25D0
      FTR(20)=-SQRT15*0.25D0*S3B*SA2
      FTR(21)=SQRT15*0.25D0*C3B*SA2
      FTR(22)=0.0D0
      FTR(23)=SQRT10*0.5D0*SB*CA*SA
      FTR(24)=-SQRT10*0.5D0*CB*CA*SA
      FTR(25)=C2B*C2A
      FTR(26)=-S2B*C2A
      FTR(27)=-SQRT6*0.25D0*S3B*S2A
      FTR(28)=SQRT6*0.25D0*C3B*S2A
      FTR(29)=SQRT15*0.5D0*CA*SA2
      FTR(30)=SQRT10*0.25D0*CB*SA*(1.0D0 -3.0D0*CA2)
      FTR(31)=SQRT10*0.25D0*SB*SA*(1.0D0 -3.0D0*CA2)
      FTR(32)=0.5D0*S2B*CA*(3.0D0*CA2 -1.0D0)
      FTR(33)=0.5D0*C2B*CA*(3.0D0*CA2 -1.0D0)
      FTR(34)=SQRT6*0.25D0*C3B*SA*(1.0D0 + CA2)
      FTR(35)=SQRT6*0.25D0*S3B*SA*(1.0D0 + CA2)
      FTR(36)=-SQRT10*0.25D0*SA3
      FTR(37)=SQRT15*0.25D0*CB*CA*SA2
      FTR(38)=SQRT15*0.25D0*SB*CA*SA2
      FTR(39)=-SQRT6*0.25D0*S2B*SA*(1.0D0 + CA2)
      FTR(40)=-SQRT6*0.25D0*C2B*SA*(1.0D0 + CA2)
      FTR(41)=0.25D0*C3B*CA*(3.0D0 + CA2)
      FTR(42)=0.25D0*S3B*CA*(3.0D0 + CA2)
      FTR(43)=0.0D0
      FTR(44)=-SQRT15*0.25D0*SB*SA2
      FTR(45)=SQRT15*0.25D0*CB*SA2
      FTR(46)=-SQRT6*0.25D0*C2B*S2A
      FTR(47)=SQRT6*0.25D0*S2B*S2A
      FTR(48)=-0.25D0*S3B*(1.0D0 +3.0D0*CA2)
      FTR(49)=0.25D0*C3B*(1.0D0 + 3.0D0*CA2)

      RETURN
      END
