C ********************************************************************
C * CEDiT - Crystal Electronic Dipole-induced Transitions            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE SODELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates the <s|O|d> elements of the transition  *
C *               matrix.                                            *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: July 1992                                          *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *               SDOVL                                              *
C *               PDOVL                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE SODELE(M, HQZ, N2, SK1, SK21, SK22, C1, C2,
     .                  R, PTR, DTR, LDIST, POL, CURRJ)

      IMPLICIT NONE

        DOUBLE PRECISION M
        INTEGER          HQZ, N2
        DOUBLE PRECISION SK1, SK21, SK22, C1, C2, R, PTR(9), DTR(25)
        DOUBLE PRECISION LDIST
        CHARACTER*1      POL
        INTEGER          CURRJ

C VAR
       DOUBLE PRECISION A(20), B(20), A1(20), B1(20), FUNC
       DOUBLE PRECISION SDO, SDOS1, SDOS2, SDO1(5), SDO2(5), PDOS1 
       DOUBLE PRECISION PDOS2 
       DOUBLE PRECISION PDOP1, PDOP2, PDO, PDO1(3,5), PDO2(3,5), NORM
       INTEGER          L1, L2, ML, MAX, N1

C Anweisungsfunktion

      FUNC(PDO, SDO) =
     .     NORM * PDO + LDIST * SDO

      INCLUDE 'snorm.inc'

C Ende der Anweisungsfunktion

C BEGIN

C Berechne zuerst den "normalen" s-d-Overlap

      N1 = HQZ

      L1 = 0
      L2 = 2
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK21, R, MAX)
      CALL LOVLAP(SDOS1, A, B, SK1, SK21, R, L1, L2,
     .            ML, N1, N2)

      CALL ABFNS(A1, B1, SK1, SK22, R, MAX)
      CALL LOVLAP(SDOS2, A1, B1, SK1, SK22, R, L1, L2,
     .            ML, N1, N2)

      CALL SDOVL(SDO1, SDOS1, DTR)
      CALL SDOVL(SDO2, SDOS2, DTR)

C Berechne nun den p-d-Overlap nachdem man den Operator auf das
C s-Orbital angewendet hat.

      N1 = N1 + 1
      L1 = 1
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK21, R, MAX)
      CALL LOVLAP(PDOS1, A, B, SK1, SK21, R, L1, L2,
     .            ML, N1, N2)

      CALL ABFNS(A1, B1, SK1, SK22, R, MAX)
      CALL LOVLAP(PDOS2, A1, B1, SK1, SK22, R, L1, L2,
     .            ML, N1, N2)

      PDOS1 = -PDOS1
      PDOS2 = -PDOS2     

      ML = 1

      CALL LOVLAP(PDOP1, A, B, SK1, SK21, R, L1, L2,
     .            ML, N1, N2)

      CALL LOVLAP(PDOP2, A1, B1, SK1, SK22, R, L1, L2,
     .            ML, N1, N2)

      CALL PDOVL(PDO1, PDOS1, PDOP1, PTR, DTR)
      CALL PDOVL(PDO2, PDOS2, PDOP2, PTR, DTR)

      IF (POL .EQ. 'X') THEN
        M = M * (C1 * FUNC(PDO1(1,CURRJ-4), SDO1(CURRJ-4) ) +
     .           C2 * FUNC(PDO2(1,CURRJ-4), SDO2(CURRJ-4) ))
      ELSEIF (POL .EQ. 'Y') THEN
        M = M * (C1 * FUNC(PDO1(2,CURRJ-4), SDO1(CURRJ-4) ) +
     .           C2 * FUNC(PDO2(2,CURRJ-4), SDO2(CURRJ-4) )) 
      ELSE
        M = M * (C1 * FUNC(PDO1(3,CURRJ-4), SDO1(CURRJ-4) ) +
     .           C2 * FUNC(PDO2(3,CURRJ-4), SDO2(CURRJ-4) ))
      ENDIF

      RETURN
      END

C END
