C ********************************************************************
C * DOSPLOT -  Plots density of states (DOS)                         *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DPGRPH                                                          *
C *                                                                  *
C *  Routine of DOSPLOT                                              *
C *                                                                  *
C *  Usage......: Changes graph characteristics.                     *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg.+phys. Chemistry               *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 12.02.93/mbs, 18.10.93/mbs,06.09.94/mbs,           *
C *               18.06.97/mbs                                       *
C *  Remarks....: Horizontal axes can be annotated.                  *
C *               Simplification of curse closing algorithm.         *
C *               List of buffer contents.                           *
C *               Major revision for use with PGPLOT graphics        *
C *               subroutine library.                                *                  
C *                                                                  *
C *  Called from: DOSPLT                                             *
C *                                                                  *
C *  Calls......: DPPLOT,DPBRDR,DPAXES,DPTITL,DPTICA,DPTICS,         *
C *               DPTRAP,DPSELC,DPFRMI,DPLIST,DPRDRW                 *
C *               PGPLOT routines PGPAGE,PGSWIN,PGSVP,PGCURS         *
C *                                                                  *
C ********************************************************************

      SUBROUTINE DPGRPH(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .                  VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .                  EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .                  DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .                  CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .                  DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .                  NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .                  ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,
     .                  IBFNDX,ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,
     .                  BUFTIT,FSELEC,FARBTR,FCLOSE,FFLINE)
C
C NAG/GKS version.
C
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)

      INCLUDE 'sizes'
      PARAMETER(MARGIN=0,IVERT=2,IHORIZ=1,ILEFT=0)

      DIMENSION    ENERGY(MXMESH),TEXTXY(MXTEXT,2),DOFFST(MAXBUF)
      DIMENSION    ILTYPE(2*MAXBUF),ISHADE(2*MAXBUF),IBFNDX(2*MAXBUF)
      DIMENSION    TXTANG(MXTEXT),ITXPOS(MXTEXT)
      REAL         CURSX,CURSY
      LOGICAL      FARBTR(2*MAXBUF),FSELEC(MAXSEL),FCLOSE(2*MAXBUF),
     .             FARBIT,FFLINE
      CHARACTER*80 CDOSTI,CCOPTI,CENETI,CTEXT(MXTEXT)
      CHARACTER*72 BUFTIT(2*MAXBUF)
      CHARACTER    CANSWR
C
C NAG/GKS version.
C
C      REAL         DEVXMN,DEVXMX,DEVYMN,DEVYMX
C      CHARACTER    CDMSTR
C
C PGPLOT version.
C
      INTEGER      PGCURS 

      VPXEND=VPXOFF+VPXAXW
      VPYEND=VPYOFF+VPYAXH
C
C
C NAG/GKS version. 
C
C Define GKS specific constants.
C
C      IWSID=1
C      IWSTYP=0
C      IDEVIC=1
C      ITRNFR=1
C
C Inquire device dimensions.
C
C      DEVXMN=0.0
C      DEVYMN=0.0
C      CALL GQDSP(IWSTYP,IERR,IUNITS,DEVXMX,DEVYMX,IRASX,IRASY)
C
   10 WRITE(6,20)
   20 FORMAT(/,' Graph modification menu:',//,
     .         ' ( 1) Change energy range',/,
     .         ' ( 2) Change x range',/,
     .         ' ( 3) Add border',/,
     .         ' ( 4) Add axes tic marks and tic labels',/,
     .         ' ( 5) Add axes titles',/,
     .         ' ( 6) Add optional text',/,
     .         ' ( 7) Close or open a curve (toggle)',/,
     .         ' ( 8) Change linetype of a curve',/,
     .         ' ( 9) Fill area below a DOS curve',/,
     .         ' (10) Mark fermi energy',/,
     .         ' (11) Change buffer ranges',/,
     .         ' (12) Change DOS buffer drawing offsets',/,
     .         ' (13) List curve buffer contents',//,
     .         ' ( 0) Return to main menu',/)
   30 WRITE(6,'('' Selection: '',$)')
      READ (5,'(I2)',ERR=30) ISEL
      IF ((ISEL .LT. 0) .OR. (ISEL .GT. MAXSEL)) GOTO 30
C
C ---------------------------------------------------------------------
C Change energy range.
C
      IF (ISEL .EQ. 1) THEN
        FSELEC(4)=.FALSE.
        WRITE(6,'(/,'' Actual energy range: '',F10.5,/,
     .           '' Lower energy:        '',F10.5,/,
     .           '' Higher energy:       '',F10.5,/)') 
     .        EMAX-EMIN,EMIN,EMAX
   70   WRITE(6,'('' Lower energy:  '',$)')
        READ(5,'(F10.5)',ERR=70) EMIN
  100   WRITE(6,'('' Higher energy: '',$)')
        READ(5,'(F10.5)',ERR=100) EMAX
        IF (EMAX .LE. EMIN) GOTO 70
C
C Redraw of curves. First clear output.
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C ---------------------------------------------------------------------
C Change x range.
C
      ELSE IF (ISEL .EQ. 2) THEN
        FSELEC(4)=.FALSE.
        WRITE(6,'('' x range is '',F6.3,'' times the amplitude range.'',
     .        /)') XRANGE
  150   WRITE(6,'('' x range: '',$)')
        READ(5,'(F6.3)',ERR=150) XRANGE
        IF (XRANGE .LE. 0.0) GOTO 150
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C ---------------------------------------------------------------------
C Add border.
C
      ELSE IF (ISEL .EQ. 3) THEN
        FSELEC(ISEL)=.TRUE.
        CALL DPBRDR(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPYOFF,VPYAXH,IBUFDS,
     .              IBUFCP)
C
C ---------------------------------------------------------------------
C Add axes tic marks and tic labels.
C
      ELSE IF (ISEL .EQ. 4) THEN
        FSELEC(ISEL)=.TRUE.
        ERANGE=EMAX-EMIN
        WRITE(6,'(/,'' Actual energy range: '',F10.5,/,
     .           '' Lower energy:        '',F10.5,/,
     .           '' Higher energy:       '',F10.5,/)') ERANGE,EMIN,EMAX
        WRITE(6,'('' Tic range settings apply to all energy axes.'',/)')

  190   WRITE(6,'('' Tic start energy:          '',$)')
        READ(5,'(F10.5)',ERR=190) ETCMIN
        IF (ETCMIN .LT. EMIN) ETCMIN=EMIN
  220   WRITE(6,'('' Tic end energy:            '',$)')
        READ(5,'(F10.5)',ERR=220) ETCMAX
        IF (ETCMAX .GT. EMAX) ETCMAX=EMAX
        IF (ETCMAX .LT. ETCMIN) GOTO 190
  240   WRITE(6,'('' Distance between two tics: '',$)')
        READ(5,'(F10.5)',ERR=240) ETCDIS
        IF (ETCDIS .LE. 0.0) GOTO 240

        WRITE(6,'(/,'' Left axis.'')')
        ITSDLF=0
        CALL DPTICS(ITSDLF)
  270   WRITE(6,'('' Tics per label (<1: no labels): '',$)')
        READ(5,'(I2)',ERR=270) ITCPLB

        ETCDS2=0.0
        IF (ITCPLB .GT. 0) THEN
          IF (ETCDIS .GT. ERANGE) THEN
            ETCDIS=ERANGE
            ITCPLB=1
          ENDIF
C
C NAG/GKS version.
C
C          ETCDS2=ETCDIS*DBLE(ITCPLB)
C
C PGPLOT version.
C
          ETCDS2=ETCDIS*REAL(ITCPLB)
        ENDIF
C
C Annotate left energy axis.
C
        INUM=1
        THRU=VPXOFF
        CALL DPTICA(VPXOFF,VPYOFF,VPXAXW,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .              ETCMAX,THRU,ETCDIS,ETCDS2,ITSDLF,INUM,IVERT)
C
C If middle energy axis exists, ask for tic decoration and annotate it.
C
        IF ((IBUFDS .GT. 0) .AND. (IBUFCP .GT. 0)) THEN
          WRITE(6,'(/,'' Middle axis.'')')
          ITSDMD=0
          CALL DPTICS(ITSDMD)
          INUM=0
          IF (VPXDOO .EQ. VPXOFF) THEN
            THRU=VPXCOO
          ELSE
            THRU=VPXDOO
          ENDIF
          CALL DPTICA(VPXOFF,VPYOFF,VPXAXW,VPYAXH,EMIN,EMAX,ERANGE,
     .                ETCMIN,ETCMAX,THRU,ETCDIS,ETCDS2,ITSDMD,INUM,
     .                IVERT)
        ENDIF
C
C Ask for tic decoration of right axis and annotate it.
C
        IF (FSELEC(3)) THEN
          WRITE(6,'(/,'' Right axis. '',$)')
          ITSDRT=0
          CALL DPTICS(ITSDRT)
          INUM=0
          THRU=VPXEND
          CALL DPTICA(VPXOFF,VPYOFF,VPXAXW,VPYAXH,EMIN,EMAX,ERANGE,
     .                ETCMIN,ETCMAX,THRU,ETCDIS,ETCDS2,ITSDRT,INUM,
     .                IVERT)
        ENDIF
C
C If only DOS plot exists, ask for tic annotation of horizontal axes.
C
        FARBIT=.FALSE.
        DO I=1,MAXBUF
          IF (FARBTR(I)) FARBIT=.TRUE.
        END DO
        IF ((IBUFDS .GT. 0) .AND. (IBUFCP .EQ. 0) .AND. .NOT. (FARBIT))
     .  THEN
          WRITE(6,'(/,'' Lower horizontal DOS axis.'')')
          ITSDLO=1
          CALL DPTICS(ITSDLO)
          ITSDUP=0
          IF (FSELEC(3)) THEN
            WRITE(6,'(/,'' Upper horizontal DOS axis.'')')
            ITSDUP=1
            CALL DPTICS(ITSDUP)
          ENDIF

          IF ((ITSDLO .GT. 0) .OR. (ITSDUP .GT. 0)) THEN
            DRANGE=XRANGE*(DOSMX-DOSMN)
            XDOSMN=XRANGE*DOSMN
            XDOSMX=XRANGE*DOSMX
            WRITE(6,'('' DOS axis range:   '',F10.5,/,
     .            '' DOS axis minimum: '',F10.5,/,
     .            '' DOS axis maximum: '',F10.5,/)') 
     .            DRANGE,XDOSMN,XDOSMX

  350       WRITE(6,'('' DOS axis tic start value: '',$)')
            READ(5,'(F10.5)',ERR=350) DTCMIN
            IF (DTCMIN .LT. XDOSMN) DTCMIN=XDOSMN
  380       WRITE(6,'('' DOS axis tic end value:   '',$)')
            READ(5,'(F10.5)',ERR=380) DTCMAX
            IF (DTCMAX .GT. XDOSMX) DTCMAX=XDOSMX
            IF (DTCMAX .LE. DTCMIN) GOTO 350
  400       WRITE(6,'('' Distance between tics:    '',$)')
            READ(5,'(F10.5)',ERR=400) DTCDIS
            IF (DTCDIS .LE. 0.0) GOTO 400

            IF (ITSDLO .GT. 0) THEN
  420         WRITE(6,'('' Tics per label (<1: no labels): '',$)')
              READ(5,'(I2)',ERR=420) ITCPLB
              DTCDS2=0.0
              IF (ITCPLB .GT. 0) THEN
                IF (DTCDIS .GT. DRANGE) THEN
                  DTCDIS=DRANGE
                  ITCPLB=1
                ENDIF
C
C NAG/GKS version.
C
C                DTCDS2=DTCDIS*DBLE(ITCPLB)
C
C PGPLOT version.
C
                DTCDS2=DTCDIS*REAL(ITCPLB)
              ENDIF
C
C Annotate DOS axis.
C
              INUM=1
              THRU=VPYOFF
              CALL DPTICA(VPXDOO,VPYOFF,VPXDOW,VPYAXH,XDOSMN,XDOSMX,
     .                    DRANGE,DTCMIN,DTCMAX,THRU,DTCDIS,DTCDS2,
     .                    ITSDLO,INUM,IHORIZ)
            ENDIF
            IF (ITSDUP .GT. 0) THEN
              INUM=0
              THRU=VPYEND
              CALL DPTICA(VPXDOO,VPYOFF,VPXDOW,VPYAXH,XDOSMN,XDOSMX,
     .                    DRANGE,DTCMIN,DTCMAX,THRU,DTCDIS,DTCDS2,
     .                    ITSDUP,INUM,IHORIZ)
            ENDIF
          ENDIF
C
C If only COOP plot exists, ask for tic annotation of horizontal axes.
C
        FARBIT=.FALSE.
        DO I=MAXBUF+1,2*MAXBUF
          IF (FARBTR(I)) FARBIT=.TRUE.
        END DO
        ELSE IF ((IBUFDS .EQ. 0) .AND. (IBUFCP .GT. 0) .AND. .NOT.
     .  (FARBIT)) THEN
          WRITE(6,'(/,'' Lower horizontal COOP axis.'')')
          ITSCLO=1
          CALL DPTICS(ITSCLO)
          ITSCUP=0
          IF (FSELEC(3)) THEN
            WRITE(6,'(/,'' Upper horizontal COOP axis.'')')
            ITSCUP=1
            CALL DPTICS(ITSCUP)
          ENDIF

          IF ((ITSCLO .GT. 0) .OR. (ITSCUP .GT. 0)) THEN
            CRANGE=XRANGE*COOPMX*2.0
            XCOPMX=XRANGE*COOPMX
            XCOPMN=-XCOPMX
            WRITE(6,'('' COOP axis range:   '',F10.5,/,
     .            '' COOP axis minimum: '',F10.5,/,
     .            '' COOP axis maximum: '',F10.5,/)') 
     .            CRANGE,XCOPMN,XCOPMX
  480       WRITE(6,'('' COOP axis tic start value: '',$)')
            READ(5,'(F10.5)',ERR=480) CTCMIN
            IF (CTCMIN .LT. XCOPMN) CTCMIN=XCOPMN
  500       WRITE(6,'('' COOP axis tic end value:   '',$)')
            READ(5,'(F10.5)',ERR=500) CTCMAX
            IF (CTCMAX .GT. XCOPMX) CTCMAX=XCOPMX
            IF (CTCMAX .LE. CTCMIN) GOTO 480
  520       WRITE(6,'('' Distance between tics:    '',$)')
            READ(5,'(F10.5)',ERR=520) CTCDIS
            IF (CTCDIS .LE. 0.0) GOTO 520

            IF (ITSCLO .GT. 0) THEN
  540         WRITE(6,'('' Tics per label (<1: no labels): '',$)')
              READ(5,'(I2)',ERR=540) ITCPLB

              CTCDS2=0.0
              IF (ITCPLB .GT. 0) THEN
                IF (CTCDIS .GT. CRANGE) THEN
                  CTCDIS=CRANGE
                  ITCPLB=1
                ENDIF
C
C NAG/GKS version.
C
C                CTCDS2=CTCDIS*DBLE(ITCPLB)
C
C PGPLOT version.
C
                CTCDS2=CTCDIS*REAL(ITCPLB)
              ENDIF
C
C Annotate COOP axis
C
              INUM=1
              THRU=VPYOFF
              CALL DPTICA(VPXCOO,VPYOFF,VPXCOW,VPYAXH,XCOPMN,XCOPMX,
     .                    CRANGE,CTCMIN,CTCMAX,THRU,CTCDIS,CTCDS2,
     .                    ITSCLO,INUM,IHORIZ)
            ENDIF
            IF (ITSCUP .GT. 0) THEN
              INUM=0
              THRU=VPYEND
              CALL DPTICA(VPXCOO,VPYOFF,VPXCOW,VPYAXH,XCOPMN,XCOPMX,
     .                    CRANGE,CTCMIN,CTCMAX,THRU,CTCDIS,CTCDS2,
     .                    ITSCUP,INUM,IHORIZ)
            ENDIF
          ENDIF
        ELSE
        ENDIF
C
C NAG/GKS version.
C
C        CALL GUWK(1,1)
C
C ---------------------------------------------------------------------
C Add axes titles.
C
      ELSE IF (ISEL .EQ. 5) THEN
        FSELEC(ISEL)=.TRUE.
C
C Ask for title for horizontal DOS plot axis, if existent.
C
        IF (IBUFDS .GT. 0) THEN
  560     WRITE(6,'('' DOS axis title: '',$)')
          READ(5,'(A80)',ERR=560) CDOSTI
        ENDIF
C
C Ask for title for horizontal COOP plot axis, if existent.
C
        IF (IBUFCP .GT. 0) THEN
  590     WRITE(6,'('' COOP axis title: '',$)')
          READ(5,'(A80)',ERR=590) CCOPTI
        ENDIF
C
C Ask for title for energy axis.
C
  620   WRITE(6,'('' Energy axis title: '',$)')
        READ(5,'(A80)',ERR=620) CENETI

        CALL DPTITL(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPYOFF,VPYAXH,
     .              IBUFDS,IBUFCP,CDOSTI,CCOPTI,CENETI)
C
C----------------------------------------------------------------------
C Add optional text.
C
      ELSE IF (ISEL .EQ. 6) THEN
        FSELEC(ISEL)=.TRUE.
C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(0.0,VPXFUL,0.0,VPYFUL,MARGIN)
C        CALL J06WCF(0.0,VPXFUL,0.0,VPYFUL)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,VPXFUL,0.0,VPYFUL)
        CALL PGSVP(0.0,VPXFUL,0.0,VPYFUL)
C
C Change cursor.
C
C NAG/GKS version.
C
C        IPET=2
C        IOPMOD=0
C        IECHO=1
C        CALL GSLCM(IWSID,IDEVIC,IOPMOD,IECHO)
C        CALL GINLC(IWSID,IDEVIC,ITRNFR,0.5,0.5,IPET,DEVXMN,DEVXMX,
C     .             DEVYMN,DEVYMX,0,CDMSTR)
C
        IF (ITXT .GT. 0) THEN
  650     WRITE(6,'('' Clear old text (y/n)? '',$)')
          READ(5,'(A1)') CANSWR
          IF (INDEX('YyNn',CANSWR) .EQ. 0) GOTO 650
          IF (INDEX('Yy',CANSWR) .GT. 0) THEN
            ITXT=0
          ENDIF
        ENDIF
C
C NAG/GKS version
C     
C        WRITE(6,'('' After typing of text, position text with mouse.''))
C
C PGPLOT version.
C
  690   WRITE(6,'('' Text: '',$)')
        READ(5,'(A80)') CTEXT(ITXT+1)
        IF (CTEXT(ITXT+1) .GT. ' ') THEN
          ITXT=ITXT+1
C
C NAG/GKS version.
C
C          CALL GRQLC(IWSID,IDEVIC,ISTAT,ITRFRM,CURSX,CURSY)
C          TEXTXY(ITXT,1)=DBLE(CURSX)
C          TEXTXY(ITXT,2)=DBLE(CURSY)
C          CALL DPTRAN(CTEXT(ITXT),TEXTXY(ITXT,1),TEXTXY(ITXT,2),ILEFT)
C
C PGPLOT version.
C
  700     WRITE(6,'('' Position (left=0, center=1, right=2): '',$)')
          READ(5,'(I2)',ERR=700) IPOS
          IF ((IPOS .LT. 0) .OR. (IPOS .GT. 2)) GOTO 700
          ITXPOS(ITXT)=IPOS

  710     WRITE(6,'('' Text angle: '',$)')
          READ(5,'(F10.5)',ERR=710) TXTANG(ITXT)

          WRITE(6,'('' Position mouse pointer, then type any key.'')')
          CURSX=0.5
          CURSY=0.5
          IERR=PGCURS(CURSX,CURSY,CANSWR)
          TEXTXY(ITXT,1)=CURSX
          TEXTXY(ITXT,2)=CURSY

          CALL DPTRAP(CTEXT(ITXT),TEXTXY(ITXT,1),TEXTXY(ITXT,2),
     .                TXTANG(ITXT),ITXPOS(ITXT))
          GOTO 690
        ENDIF
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINe)

C
C----------------------------------------------------------------------
C Close or open a curve (toggle)
C
      ELSE IF (ISEL .EQ. 7) THEN
        IOPT=0
        CALL DPSELC(DOSMN,DOSMX,COOPMX,IBUFDS,IBUFCP,ILTYPE,ISHADE,
     .              IBFNDX,IOPT,NMESH,BUFTIT,FARBTR,FCLOSE)
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C----------------------------------------------------------------------
C Change linetype of a curve.
C
      ELSE IF (ISEL .EQ. 8) THEN
        IOPT=2
        CALL DPSELC(DOSMN,DOSMX,COOPMX,IBUFDS,IBUFCP,ILTYPE,ISHADE,
     .              IBFNDX,IOPT,NMESH,BUFTIT,FARBTR,FCLOSE)
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C----------------------------------------------------------------------
C Fill area below a DOS curve.
C
      ELSE IF (ISEL .EQ. 9) THEN
        IOPT=1
        CALL DPSELC(DOSMN,DOSMX,COOPMX,IBUFDS,IBUFCP,ILTYPE,ISHADE,
     .              IBFNDX,IOPT,NMESH,BUFTIT,FARBTR,FCLOSE)
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C----------------------------------------------------------------------
C Mark fermi energy.
C
      ELSE IF (ISEL .EQ. 10) THEN
        FSELEC(ISEL)=.TRUE.
  720   WRITE(6,'('' Fermi energy: '',$)')
        READ(5,'(F10.5)',ERR=720) EFERMI
        IF ((EFERMI .LT. EMIN) .OR. (EFERMI .GT. EMAX)) GOTO 720
  750   WRITE(6,'('' Mark with line? (y/n) '',$)')
        READ(5,'(A1)') CANSWR
        IF (INDEX('YyNn',CANSWR) .EQ. 0) GOTO 750
        FFLINE=(INDEX('Yy',CANSWR) .GT. 0)
        CALL DPFRMI(VPXOFF,VPYOFF,VPXFUL,VPYFUL,VPXAXW,VPYAXH,VPXEND,
     .              VPYEND,EMIN,EMAX,EFERMI,FFLINE)
C
C----------------------------------------------------------------------
C Change buffer ranges.
C
      ELSE IF (ISEL .EQ. 11) THEN
        IF (IBUFDS .GT. 0) THEN
          WRITE(6,'('' DOS buffer minimum: '',F10.5,/,
     .          '' DOS buffer maximum: '',F10.5,//,
     .          '' Enter new values:''))') DOSMN,DOSMX
  790     WRITE(6,'('' Minimum: '',$)')
          READ(5,'(F10.5)',ERR=790) DOSMN

  820     WRITE(6,'('' Maximum: '',$)')
          READ(5,'(F10.5)',ERR=820) DOSMX
          IF (DOSMX .LE. DOSMN) GOTO 790
        ENDIF
        IF (IBUFCP .GT. 0) THEN
          WRITE(6,'('' COOP buffer maximum: '',F10.5,//,
     .          '' Enter new value:'')') COOPMX
  850     WRITE(6,'('' Maximum: '',$)')
          READ(5,'(F10.5)',ERR=850) COOPMX
          IF (COOPMX .LE. 0.0) GOTO 850
        ENDIF
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C ---------------------------------------------------------------------
C Change DOS buffer drawing offset.
C
      ELSE IF (ISEL .EQ. 12) THEN
        IF (IBUFDS .GT. 0) THEN
          WRITE(6,'('' Change DOS buffer drawing offset.'',/)')
880       WRITE(6,'('' Enter DOS buffer number ('',I1,''-'',I1,
     .          ''): '',$)') 1,IBUFDS
          READ(5,'(I1)',ERR=880) ICHANG
          IF ((ICHANG .LT. 1) .OR. (ICHANG .GT. IBUFDS)) GOTO 880

          WRITE(6,'(/,'' Minimum of all DOS buffers: '',F10.5,/,
     .          '' Maximum of all DOS buffers: '',F10.5,/,
     .          '' Offset of DOS buffer '',I1,'': '',F10.5,/)') 
     .          DOSMN,DOSMX,ICHANG,DOFFST(ICHANG)

930       WRITE(6,'('' Enter offset of DOS buffer '',I1,'': '',$)') 
     .          ICHANG
          READ(5,'(F10.5)',ERR=930) DOFFST(ICHANG)
        ENDIF
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL DPRDRW(VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPXOFF,VPXAXW,
     .              VPXFUL,VPYOFF,VPYAXH,VPYFUL,EMIN,EMAX,ENERGY,
     .              EFERMI,ETCMIN,ETCMAX,ETCDIS,ETCDS2,DTCMIN,
     .              DTCMAX,DTCDIS,DTCDS2,CTCMIN,CTCMAX,CTCDIS,
     .              CTCDS2,XDOSMN,XDOSMX,XCOPMX,XRANGE,
     .              DOSMN,DOSMX,COOPMX,TEXTXY,TXTANG,DOFFST,
     .              NMESH,IBUFDS,IBUFCP,ITXT,ITSDLF,ITSDMD,ITSDRT,
     .              ITSDUP,ITSDLO,ITSCUP,ITSCLO,ILTYPE,ISHADE,IBFNDX,
     .              ITXPOS,CDOSTI,CCOPTI,CENETI,CTEXT,BUFTIT,
     .              FSELEC,FARBTR,FCLOSE,FFLINE)
C
C ---------------------------------------------------------------------
C List curve buffer contents.
C
      ELSE IF (ISEL .EQ. 13) THEN
        CALL DPLIST(IBUFDS,IBUFCP,BUFTIT)
      ELSE
      ENDIF

      IF (ISEL .GT. 0) GOTO 10

      RETURN
      END