C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  ARRAYS                                                          *
C *                                                                  *
C *  Routine of EHMACC,                                              *
C *  version for VMS, VAX FORTRAN                                    *
C *                                                                  *
C *  Usage......: Defines a single one-dimensional array a to be di- *
C *               vided into several two- and one-dimensional arrays *
C *               which are used later.                              *
C *               The size of the array A approximately determines   *
C *               the overall size of the program EHMACC.            *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 29.04.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Dimension of array A can be individually set by    *
C *               NDIMMX, the maximum number of atomic orbitals per  *
C *               unit cell. Because many of the atoms used are of   *
C *               s,p type, the relation NDIMMX=4*NATTMX (maximum    *
C *               number of atoms per unit cell) approximately       *
C *               holds. NDIMMX and NATMMX can be set in the inclu-  *
C *               defile EHMPAR.                                     *
C *                                                                  *
C *  Called from: PART2                                              *
C *                                                                  *
C *  Calls......: SMAT,HMAT                                          *
C *                                                                  *
C ********************************************************************

      SUBROUTINE ARRAYS(NAT,ND)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      INCLUDE 'sizes'

      PARAMETER (ITOT=3*NDIMMX*NDIMMX+3*NDIMMX)

      DIMENSION A(ITOT)
     
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /PASS/NATOM,NDIM

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      NATOM=NAT
      NDIM=ND
      NDIMSQ=NDIM*NDIM
      NATOM2=NATOM+NATOM

      J1=1
      J2=J1+NDIMSQ
      J3=J2+NDIM
      J4=J3+NDIMSQ
      J5=J4+NDIM
      J6=J5+NDIMSQ
      J7=J6+NDIM
      ITOTC=J7+2
      IF (ITOTC .GT. ITOT) THEN
        SLINE='Dimension of the problem too large, modify array.'
        CALL WRITEL(SLINE)
        CALL CLEXIT
      ENDIF

      I1=1
      I2=I1+NDIMSQ
      I3=I2+NDIMSQ
      I4=I2
      I5=I4+NATOM
      I6=I5+NATOM
      I7=I6+NATOM
      I8=I7+NATOM2
      I9=I8+NATOM

      CALL SMAT(A(I1),A(I4),A(I5),A(I6),A(I7),A(I8),A(I9),NATOM2,
     .          NATOM,NDIM)
      CALL HMAT(A(I1),A(I2),A(I3),NATOM,NDIM)
      RETURN
      END
