C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BUILDZ                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Builds molecular of crystal geometries by bond     *
C *               lengths, bond angles, and dihedral angles (Z       *
C *               matrix).                                           *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 02.05.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: VEC,VPROD                                          *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE BUILDZ (CZ)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      PARAMETER(NATMAP=NATMMX+20)
 
      DIMENSION CZ(NATMMX,3),ALPHA(NATMAP),BETA(NATMAP),U1(3),U2(3),
     .          U3(3),U4(3),VJ(3),VP(3)
 
      COMMON /ZMATN/ANZ(NATMMX)
      CHARACTER ANZ*7
 
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /ZMAT/BL(NATMMX),ALPH(NATMMX),BET(NATMMX),IZ(NATMMX,3),NZ
 
      DATA ZERO/0.0D0/,ONE/1.0D0/
      DATA TENM5/1.0D-5/
      DATA TORAD/1.7453292520D-2/
 
      WRITE(IOUT,10)
   10 FORMAT(//,4X,'z matrix :',//,3X,'an',6X,'z1',9X,'bl',14X,'z2',9X,
     .       'alpha',11X,'z3',9X,'beta'/)
      WRITE(IOUT,20) ANZ(1)
   20 FORMAT(1X,A7)
      IF (NZ .LE. 1) GOTO 70
      I1=1
      WRITE(IOUT,30) ANZ(2),IZ(2,1),BL(2),I1
   30 FORMAT(1X,A7,I5,F14.7,'(',I3,')   ')
      IF (NZ .LE. 2) GOTO 70
      I1=I1+1
      I2=NZ
      WRITE(IOUT,40) ANZ(3),IZ(3,1),BL(3),I1,IZ(3,2),ALPH(3),I2
   40 FORMAT(1X,A7,I5,F14.7,'(',I3,')   ',I5,F14.7,'(',I3,')   ')
      IF (NZ .LE. 3) GOTO 70
      I3=2*NZ-3
 
      DO 50 I=4,NZ
        I1=I1+1
        I2=I2+1
        I3=I3+1
   50   WRITE(IOUT,60) ANZ(I),IZ(I,1),BL(I),I1,IZ(I,2),ALPH(I),I2,
     .                 IZ(I,3),BET(I),I3
   60 FORMAT(1X,A7,I5,F14.7,'(',I3,')   ',I5,F14.7,'(',I3,')   ',I5,
     .       F14.7,'(',I3,')   ')
 
   70 DO 80 I=1,NZ
        DO 80 J=1,3
   80 CZ(I,J)=ZERO
 
      DO 90 I=1,NZ
        ALPHA(I)=ALPH(I)*TORAD
   90 BETA(I)=BET(I)*TORAD
 
      CZ(2,3)=BL(2)
      IF (NZ-3) 210,100,100
  100 CZ(3,1)=BL(3)*DSIN(ALPHA(3))
      IF (IZ(3,1)-1) 120,110,120
  110 CZ(3,3)=BL(3)*DCOS(ALPHA(3))
      GOTO 130
  120 CZ(3,3)=CZ(2,3)-BL(3)*DCOS(ALPHA(3))
 
  130 DO 150 I=4,NZ
        IF (DABS(CZ(I-1,1))-TENM5) 140,160,160
  140   CZ(I,1)=BL(I)*DSIN(ALPHA(I))
        ITEMP=IZ(I,1)
        JTEMP=IZ(I,2)
  150 CZ(I,3)=CZ(ITEMP,3)-BL(I)*DCOS(ALPHA(I))*DSIGN(ONE,CZ(ITEMP,3)
     .  -CZ(JTEMP,3))
 
  160 K=I
      IF (K-NZ) 170,170,210
 
  170 DO 200 J=K,NZ
        DCAJ=DCOS(ALPHA(J))
        DSAJ=DSIN(ALPHA(J))
        DCBJ=DCOS(BETA(J))
        DSBJ=DSIN(BETA(J))
        CALL VEC(U1,CZ,IZ(J,2),IZ(J,3))
        CALL VEC(U2,CZ,IZ(J,1),IZ(J,2))
        CALL VPROD(VP,U1,U2)
        R=DSQRT(ONE-(U1(1)*U2(1)+U1(2)*U2(2)+U1(3)*U2(3))**2)
        DO 180 I=1,3
  180   U3(I)=VP(I)/R
        CALL VPROD(U4,U3,U2)
        DO 190 I=1,3
          VJ(I)=BL(J)*(-U2(I)*DCAJ+U4(I)*DSAJ*DCBJ+U3(I)*DSAJ*DSBJ)
          ITEMP=IZ(J,1)
  190   CZ(J,I)=VJ(I)+CZ(ITEMP,I)
  200 CONTINUE
 
  210 CONTINUE
      RETURN
      END
