C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  CTQL2                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Computes the eigenvalues and eigenvectors of a     *
C *               complex hermitian tridiagonal matrix.              *
C *                                                                  *
C *               parameters                                         *
C *                                                                  *
C *               n    --- order of matrix                           *
C *               nd   --- dimension of a and b                      *
C *               d    --- diagonal given                            *
C *               e    --- real part of codiagonal given             *
C *                        (last n-1 locations)                      *
C *               f    --- imag part of the lower codiagonal         *
C *               a    --- real part of eigenvectors                 *
C *               b    --- imag part of eigenvectors                 *
C *               ifail --- receives value of 1 instead of zero      *
C *                        if some eigenvalue takes more than 30     *
C *                        iterations.                               *
C *                                                                  *
C *               eigenvalues are obtained in increasing of          *
C *               magnitude in vector d, eigenvectors are stored     *
C *               by columns.  arrays a and b should be preset to    *
C *               some unitary matrix such as the identity matrix    *
C *               or the matrix produced by ctred2.                  *
C *                                                                  *
C *  Author.....: s. CBORIS                                          *
C *  Date.......: 20.06.1974                                         *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: CBORIS                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE CTQL2(N,ND,D,E,F,A,B,IFAIL)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(ND,ND),B(ND,ND),D(ND),E(ND),F(ND)
C
C     ***************************************
C     *                                     *
C     * next line of program defines        *
C     * machine dependent constant chep     *
C     * defined as the smallest real        *
C     * number for which                    *
C     *                                     *
C     *        1.0+chep .gt. 1.0            *
C     *                                     *
C     ***************************************
C
      CHEP=2.0**(-56)
C
C     first make real codiagonal moved down
C     to first location
C
      IF (N .EQ. 1) GOTO 30
      DO 20 K=2,N
        R=DSQRT(E(K)*E(K)+F(K)*F(K))
        IF (DABS(R) .LT. 1.0D-37) R=0.0
        IF (R .EQ. 0.0) GOTO 20
        C=E(K)/R
        S=F(K)/R
C
C     accumulate rotation
C
        DO 10 I=1,N
          P=A(I,K)*C-B(I,K)*S
          B(I,K)=A(I,K)*S+B(I,K)*C
   10   A(I,K)=P
C
C     transform next e
C
        IF (K .EQ. N) GOTO 20
        L=K+1
        P=E(L)*C-F(L)*S
        F(L)=E(L)*S+F(L)*C
        E(L)=P
   20 E(K-1)=R
   30 E(N)=0.0
C
C     initialize
C
      BB=0.0
      FF=0.0
      IFAIL=1
C
C     main loop
C
      DO 130 L=1,N
        J=0
        H=CHEP*(DABS(D(L))+DABS(E(L)))
        IF (BB .LT. H) BB=H
C
C     look for small subdiagonal element
C
        DO 40 M=L,N
          IF (DABS(E(M)) .LE. BB) GOTO 50
   40   CONTINUE
   50   IF (M .EQ. L) GOTO 120
C
C     next iteration
C
   60   IF (J .EQ. 30) RETURN 
        J=J+1
C
C     form shift
C
        P=(D(L+1)-D(L))/(2.0*E(L))
        R=DSQRT(1.0+P*P)
        H=D(L)-E(L)/(P+DSIGN(R,P))

        DO 70 I=L,N
   70   D(I)=D(I)-H
        FF=FF+H
C
C     ql transformation
C
        P=D(M)
        C=1.0
        S=0.0
        MA=M-1

        DO 110 IA=L,MA
          I=MA-IA+L
          I1=I+1
          G=C*E(I)
          H=C*P
          IF (DABS(P) .LT. DABS(E(I))) GOTO 80

          C=E(I)/P
          R=DSQRT(C*C+1.0)
          E(I1)=S*P*R
          S=C/R
          C=1.0/R
          GOTO 90

   80     C=P/E(I)
          R=DSQRT(C*C+1.0)
          E(I1)=S*E(I)*R
          S=1.0/R
          C=C/R

   90     P=C*D(I)-S*G
          D(I1)=H+S*(C*G+S*D(I))
C
C     form vector
C
          DO 100 K=1,N
            HR=A(K,I1)
            HI=B(K,I1)
            A(K,I1)=S*A(K,I)+C*HR
            B(K,I1)=S*B(K,I)+C*HI
            A(K,I)=C*A(K,I)-S*HR
  100     B(K,I)=C*B(K,I)-S*HI

  110   CONTINUE

        E(L)=S*P
        D(L)=C*P
        IF (DABS(E(L)) .GT. BB) GOTO 60
C
C     root found
C
  120   D(L)=D(L)+FF
  130 CONTINUE
C
C     order eigenvalues and eigenvectors
C
      DO 160 I=1,N
        K=I
        DO 140 J=I,N
          IF (D(J) .LT. D(K)) K=J
  140   CONTINUE

        IF (K .EQ. I) GOTO 160
        P=D(I)
        D(I)=D(K)
        D(K)=P

        DO 150 J=1,N
          P=A(J,I)
          A(J,I)=A(J,K)
          A(J,K)=P
          P=B(J,I)
          B(J,I)=B(J,K)
  150   B(J,K)=P
  160 CONTINUE
      IFAIL=0

      RETURN
      END
