C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DISTC                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates the interatomic distances within and    *
C *               between unit cells.                                *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 8.05.92/mbs (Martin Braendle c/o        *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: INMODI                                             *
C *                                                                  *
C *  Calls......: SUPPR                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE DISTC(CX1,ACX1,N1,CX2,ACX2,N2,DMIN,DMAX,ICHO1,NMAX,
     .                 DDMAX,DDMIN)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION   CX1(NATMMX,3),CX2(NATMMX,3),D(3),DD1(NBONMX),
     .            IDTR1(NBONMX,3),INDEXX(NBONMX)
      CHARACTER*7 ACX1(NATMMX),ACX2(NATMMX),IDAT1(NBONMX,2)
      CHARACTER   ACI1*7,ACI2*7,ACJ1*7,ACJ2*7

      COMMON /BOND2/IDTR(NBONMX,3),DDX(NBONMX),IDBOND,INEXT(NBONMX),
     .              IFIRST,ITYPE(14)
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /CELL/DIST(3,3)

      COMMON /BOND1/IDAT(NBONMX,2)
      CHARACTER*7 IDAT

      COMMON /INDX/NCRYST,NCEL1,NCEL2,NCEL3,XK(3),WEIGHT,TWOPI,ND1,
     .             NGRAD,NFULL,CEL1,CEL2
      LOGICAL CEL1,CEL2

C
C icho1=0 : there is a next card
C icho1=1 : last card ===> ordering
C
      DO 210 M1=1,N1
C
C unit cell ia+ib+ic i=0,j=0,k=0
C
        DO 40 M2=1,N2
          DO 10 K=1,3
            D(K)=CX2(M2,K)-CX1(M1,K)
   10     CONTINUE
          XD=D(1)*D(1)+D(2)*D(2)+D(3)*D(3)
          XD=DSQRT(XD)
          IF ((XD .LT. DMIN) .OR. (XD .GT. DMAX)) GOTO 40
C
C suppress unwanted bonds
C
          IF (IDBOND .EQ. 0) GOTO 30
          ACI1=ACX1(M1)
          ACJ1=ACX2(M2)
          DO 20 IJ=1,IDBOND
            ACI2=IDAT(IJ,1)
            ACJ2=IDAT(IJ,2)
            XDP=XD+1.0D-5
            XDM=XD-1.0D-5
            IF (((ACI1 .NE. ACI2) .OR. (ACJ1 .NE. ACJ2)) .AND. 
     .          ((ACI1 .NE. ACJ2) .OR. (ACJ1 .NE. ACI2))) GOTO 20
            IF ((DDX(IJ) .LT. XDM) .OR. (DDX(IJ) .GT. XDP)) GOTO 20
            IF ((IDTR(IJ,1) .NE. 0) .OR. (IDTR(IJ,2) .NE. 0) .OR. 
     .          (IDTR(IJ,3) .NE. 0)) GOTO 20
            GOTO 40
   20     CONTINUE
C
C store the bond m1-m2
C
   30     IF ((XD .GT. DDMAX) .AND. (IDBOND .GE. NMAX)) GOTO 40
          IDBOND=IDBOND+1
          IDAT(IDBOND,1)=ACX1(M1)
          IDAT(IDBOND,2)=ACX2(M2)
          DDX(IDBOND)=XD
          IF (XD .LT. DDMIN) DDMIN=XD
          IF (XD .GT. DDMAX) THEN
            DDMAX=XD
            IDM=IDBOND
          ENDIF
          IDTR(IDBOND,1)=0
          IDTR(IDBOND,2)=0
          IDTR(IDBOND,3)=0
C
C replace the largest bond by a smallest bond if idbond>nmax
C
          IF (IDBOND .GE. NMAX) CALL SUPPR(IDM,DDMAX)
   40   CONTINUE
C
C
C unit cell ia+ib+ic i=0,j=1,k=0
C
        IF (NCRYST .EQ. 0) GOTO 210
        IF (NCRYST .EQ. 1) GOTO 90
        DO 80 M2=1,N2
          DO 50 L=1,3
            D(L)=CX2(M2,L)+DIST(2,L)-CX1(M1,L)
   50     CONTINUE
          XD=D(1)*D(1)+D(2)*D(2)+D(3)*D(3)
          XD=DSQRT(XD)
          IF ((XD .LT. DMIN) .OR. (XD .GT. DMAX)) GOTO 80
          IF (IDBOND .EQ. 0) GOTO 70
          ACI1=ACX1(M1)
          ACJ1=ACX2(M2)
          DO 60 IJ=1,IDBOND
            ACI2=IDAT(IJ,1)
            ACJ2=IDAT(IJ,2)
            IF ((ACI1 .NE. ACI2) .OR. (ACJ1 .NE. ACJ2)) GOTO 60
            IF (XD .NE. DDX(IJ)) GOTO 60
            IF ((IDTR(IJ,1) .NE. 0) .OR. (IDTR(IJ,2) .NE. 1) .OR. 
     .          (IDTR(IJ,3) .NE. 0)) GOTO 60
            GOTO 80
   60     CONTINUE
   70     IF ((XD .GT. DDMAX) .AND. (IDBOND .GE. NMAX)) GOTO 80
          IDBOND=IDBOND+1
          IDAT(IDBOND,1)=ACX1(M1)
          IDAT(IDBOND,2)=ACX2(M2)
          DDX(IDBOND)=XD
          IF (XD .LT. DDMIN) DDMIN=XD
          IF (XD .GT. DDMAX) THEN
            DDMAX=XD
            IDM=IDBOND
          ENDIF
          IDTR(IDBOND,1)=0
          IDTR(IDBOND,2)=1
          IDTR(IDBOND,3)=0
          IF (IDBOND .GE. NMAX) CALL SUPPR(IDM,DDMAX)
   80   CONTINUE
C
C unit cell ia+ib+ic i=1,j=-1,0,1,k=0
C
   90   DO 140 J=-1,1
          IF (J .NE. 0 .AND. NCRYST .EQ. 1) GOTO 140
          DO 130 M2=1,N2
            DO 100 L=1,3
              D(L)=CX2(M2,L)+DIST(1,L)+J*DIST(2,L)-CX1(M1,L)
  100       CONTINUE
            XD=D(1)*D(1)+D(2)*D(2)+D(3)*D(3)
            XD=DSQRT(XD)
            IF ((XD .LT. DMIN) .OR. (XD .GT. DMAX)) GOTO 130
            IF (IDBOND .EQ. 0) GOTO 120
            ACI1=ACX1(M1)
            ACJ1=ACX2(M2)
            DO 110 IJ=1,IDBOND
              ACI2=IDAT(IJ,1)
              ACJ2=IDAT(IJ,2)
              IF ((ACI1 .NE. ACI2) .OR. (ACJ1 .NE. ACJ2)) GOTO 110
              IF (XD .NE. DDX(IJ)) GOTO 110
              IF ((IDTR(IJ,1) .NE. 1) .OR. (IDTR(IJ,2) .NE. J) .OR. 
     .            (IDTR(IJ,3) .NE. 0)) GOTO 110
              GOTO 130
  110       CONTINUE
C
C store the bond m1-m2
C
  120       IF ((XD .GT. DDMAX) .AND. (IDBOND .GE. NMAX)) GOTO 130
            IDBOND=IDBOND+1
            IDAT(IDBOND,1)=ACX1(M1)
            IDAT(IDBOND,2)=ACX2(M2)
            DDX(IDBOND)=XD
            IF (XD .LT. DDMIN) DDMIN=XD
            IF (XD .GT. DDMAX) THEN
              DDMAX=XD
              IDM=IDBOND
            ENDIF
            IDTR(IDBOND,1)=1
            IDTR(IDBOND,2)=J
            IDTR(IDBOND,3)=0
            IF (IDBOND .GE. NMAX) CALL SUPPR(IDM,DDMAX)
  130     CONTINUE
  140   CONTINUE
C
C
C unit cell ia+ib+ic i=-1,0,1,j=-1,0,1,k=1
C
        IF (NCRYST .LT. 3) GOTO 210
        DO 200 J=-1,1
          DO 190 I=-1,1
            DO 180 M2=1,N2
              DO 150 L=1,3
                D(L)=CX2(M2,L)+I*DIST(1,L)+J*DIST(2,L)+DIST(3,
     .                  L)-CX1(M1,L)
  150         CONTINUE
              XD=D(1)*D(1)+D(2)*D(2)+D(3)*D(3)
              XD=DSQRT(XD)
              IF ((XD .LT. DMIN) .OR. (XD .GT. DMAX)) GOTO 180
              IF (IDBOND .EQ. 0) GOTO 170
              ACI1=ACX1(M1)
              ACJ1=ACX2(M2)
              DO 160 IJ=1,IDBOND
                ACI2=IDAT(IJ,1)
                ACJ2=IDAT(IJ,2)
                IF ((ACI1 .NE. ACI2) .OR. (ACJ1 .NE. ACJ2)) GOTO 160
                IF (XD .NE. DDX(IJ)) GOTO 160
                IF ((IDTR(IJ,1) .NE. I) .OR. (IDTR(IJ,2) .NE. J) .OR. 
     .              (IDTR(IJ,3) .NE. 1)) GOTO 160
                GOTO 180
  160         CONTINUE
C
C store the bond m1-m2
C
  170         IF ((XD .GT. DDMAX) .AND. (IDBOND .GE. NMAX)) GOTO 180
              IDBOND=IDBOND+1
              IDAT(IDBOND,1)=ACX1(M1)
              IDAT(IDBOND,2)=ACX2(M2)
              DDX(IDBOND)=XD
              IF (XD .LT. DDMIN) DDMIN=XD
              IF (XD .GT. DDMAX) THEN
                DDMAX=XD
                IDM=IDBOND
              ENDIF
              IDTR(IDBOND,1)=I
              IDTR(IDBOND,2)=J
              IDTR(IDBOND,3)=1
              IF (IDBOND .GE. NMAX) CALL SUPPR(IDM,DDMAX)
  180       CONTINUE
  190     CONTINUE
  200   CONTINUE
C
C end of loop
C
  210 CONTINUE

      IF (ICHO1 .EQ. 0) RETURN 
C
C 1) ordering on the first atom name
C
      DO 220 I=1,IDBOND
        INDEXX(I)=I
  220 CONTINUE

      IDM1=IDBOND-1
      DO 240 I=1,IDM1
        IP1=I+1
        DO 230 J=IP1,IDBOND
          II=INDEXX(I)
          IJ=INDEXX(J)
          ACI1=IDAT(II,1)
          ACJ1=IDAT(IJ,1)
          IF (ACI1(1:5) .LE. ACJ1(1:5)) GOTO 230
          ITEMP=INDEXX(I)
          INDEXX(I)=INDEXX(J)
          INDEXX(J)=ITEMP
  230   CONTINUE
  240 CONTINUE

      DO 250 I=1,IDBOND
        IDAT1(I,1)=IDAT(INDEXX(I),1)
        IDAT1(I,2)=IDAT(INDEXX(I),2)
        DD1(I)=DDX(INDEXX(I))
        IDTR1(I,1)=IDTR(INDEXX(I),1)
        IDTR1(I,2)=IDTR(INDEXX(I),2)
        IDTR1(I,3)=IDTR(INDEXX(I),3)
  250 CONTINUE
C
C 2) ordering on the second atom name with the same first atom name
C
      DO 260 I=1,IDBOND
        INDEXX(I)=I
  260 CONTINUE

      IDM1=IDBOND-1
      DO 280 I=1,IDM1
        IP1=I+1
        DO 270 J=IP1,IDBOND
          II=INDEXX(I)
          IJ=INDEXX(J)
          ACI1=IDAT1(II,1)
          ACJ1=IDAT1(IJ,1)
          ACI2=IDAT1(II,2)
          ACJ2=IDAT1(IJ,2)
          IF (ACI1(1:5) .NE. ACJ1(1:5)) GOTO 280
          IF (ACI2(1:5) .LE. ACJ2(1:5)) GOTO 270
          ITEMP=INDEXX(I)
          INDEXX(I)=INDEXX(J)
          INDEXX(J)=ITEMP
  270   CONTINUE
  280 CONTINUE

      DO 290 I=1,IDBOND
        IDAT(I,1)=IDAT1(INDEXX(I),1)
        IDAT(I,2)=IDAT1(INDEXX(I),2)
        DDX(I)=DD1(INDEXX(I))
        IDTR(I,1)=IDTR1(INDEXX(I),1)
        IDTR(I,2)=IDTR1(INDEXX(I),2)
        IDTR(I,3)=IDTR1(INDEXX(I),3)
  290 CONTINUE
C
C 3) ordering on the second atom name with the same first atom name and
C    the same second atom name.
C
      DO 300 I=1,IDBOND
        INDEXX(I)=I
  300 CONTINUE

      IDM1=IDBOND-1
      DO 320 I=1,IDM1
        IP1=I+1
        DO 310 J=IP1,IDBOND
          II=INDEXX(I)
          IJ=INDEXX(J)
          ACI1=IDAT(II,1)
          ACJ1=IDAT(IJ,1)
          ACI2=IDAT(II,2)
          ACJ2=IDAT(IJ,2)
          IF (ACI1(1:5) .NE. ACJ1(1:5) .OR. ACI2(1:5) .NE. ACJ2(1:5)) 
     .    GOTO 320
          IF (DDX(II) .LE. DDX(IJ)) GOTO 310
          ITEMP=INDEXX(I)
          INDEXX(I)=INDEXX(J)
          INDEXX(J)=ITEMP
  310   CONTINUE
  320 CONTINUE

      DO 330 I=1,IDBOND
        IDAT1(I,1)=IDAT(INDEXX(I),1)
        IDAT1(I,2)=IDAT(INDEXX(I),2)
        DD1(I)=DDX(INDEXX(I))
        IDTR1(I,1)=IDTR(INDEXX(I),1)
        IDTR1(I,2)=IDTR(INDEXX(I),2)
        IDTR1(I,3)=IDTR(INDEXX(I),3)
  330 CONTINUE

      DO 340 I=1,IDBOND
        IDAT(I,1)=IDAT1(I,1)
        IDAT(I,2)=IDAT1(I,2)
        DDX(I)=DD1(I)
        IDTR(I,1)=IDTR1(I,1)
        IDTR(I,2)=IDTR1(I,2)
        IDTR(I,3)=IDTR1(I,3)
  340 CONTINUE

      RETURN
      END
