C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GETPOS                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Reads atomic positions either from the Z matrix or *
C *               from the xyz coordinates.                          *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 04.05.92/mbs, 11.11.94/mbs (Martin      *
C *               Braendle, c/o Calzaferri group, Inst. f. Inorg.    *
C *               and Phys. Chem., Freiestr. 3, CH-3012 Berne)       *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *               Reading in of fractional coordinates.              *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: WRITEL,READL,READA                                 *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE GETPOS(ICH)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      CHARACTER STRING*80
 
      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2
 
      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM, ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),EXPD(NSYMMX),
     .              EXPD2(NSYMMX),EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),CP1(NSYMMX),
     .              CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),CF1(NSYMMX),
     .              CF2(NSYMMX),COULS(NSYMMX,NAEQMX),COULP(NSYMMX), 
     .              COULD(NSYMMX),COULF(NSYMMX),C(NATMMX,3)

      COMMON /FRACTC/FC(NATMMX,3)

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      COMMON /ZMAT/BL(NATMMX),ALPH(NATMMX),BET(NATMMX),IZ(NATMMX,3),NZ
 
      COMMON /ZMATN/ANZ(NATMMX)
      CHARACTER ANZ*7
 
      IIA=ICHAR('A')
      IIZ=ICHAR('Z')
      NZ=0
      IF (INDEX(SLINE,'POS') .EQ. 0) THEN
        SLINE='Card POS expected.'
        CALL WRITEL(SLINE)
      ENDIF
   10 IF (INDEX(SLINE,'POS') .EQ. 0) GOTO 40
      NZ=NZ+1
      IF (NZ .GT. NATMMX) THEN
        SLINE='Maximum number of atoms exceeded.'
        CALL WRITEL(SLINE)
        CALL CLEXIT
      ENDIF
 
      LEN=ISTART(3)-ISTART(2)
      IF (LEN .GT. 7) LEN=7
      STRING=SLINE(ISTART(2):ISTART(3)-1)
      IS=ICHAR(STRING(2:2))
      IF ((IS .LT. IIA) .OR. (IS .GT. IIZ)) THEN
        DO 20 I=LEN,1,-1
          STRING(I+1:I+1)=STRING(I:I)
   20   CONTINUE
        STRING(1:1)=' '
      ENDIF
      IF (LEN .LT. 7) THEN
        DO 30 I=LEN+1,7
          STRING(I:I)=' '
   30   CONTINUE
      ENDIF
 
      ANZ(NZ)=STRING(1:7)
      IF (ICH .EQ. 0) THEN
        IZ(NZ,1)=INT(READA(SLINE,ISTART(3),80))
        IZ(NZ,2)=INT(READA(SLINE,ISTART(5),80))
        IZ(NZ,3)=INT(READA(SLINE,ISTART(7),80))
        BL(NZ)=READA(SLINE,ISTART(4),80)
        ALPH(NZ)=READA(SLINE,ISTART(6),80)
        BET(NZ)=READA(SLINE,ISTART(8),80)
      ELSE IF (ICH .EQ. 1) THEN 
        AC(NZ)=STRING(1:7)
        C(NZ,1)=READA(SLINE,ISTART(3),80)
        C(NZ,2)=READA(SLINE,ISTART(4),80)
        C(NZ,3)=READA(SLINE,ISTART(5),80)
      ELSE
        AC(NZ)=STRING(1:7)
        FC(NZ,1)=READA(SLINE,ISTART(3),80)
        FC(NZ,2)=READA(SLINE,ISTART(4),80)
        FC(NZ,3)=READA(SLINE,ISTART(5),80)        
      ENDIF
      CALL READL
      GOTO 10
 
   40 CONTINUE
 
      RETURN
      END
