C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  INMODI                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates CELL parameters, checks atom positions, *
C *               determines interatomic distances.                  *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 11.05.92/mbs, 28.12.92/mbs, 20.02.93/mbs*
C *               12.07.94/mbs (Martin Braendle c/o Calzaferri       *
C *               group, Institute f. Inorg. and Phys. Chemistry,    *
C *               University of Berne,  Freiestr. 3, CH-3012 Berne)  *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *               Generation of SCHAKAL compatible file.             *
C *               RHO handling is done in WRTKEY. Check is done if   *
C *               number of unit cells times size of translation     *
C *               exceeds RHOMAX (s. also comment in WRTKEY)./mbs    *
C *               Adapted for automatic geometry variation./mbs      *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: ADDK,WRITEL,GETNAM,GETNUM,READL,READA,DISTC,PRBOND,*
C *               RBOND                                              *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE INMODI
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      DIMENSION   C1(NATMMX,3),C2(NATMMX,3),ID(2,3)
      SAVE        ID

      CHARACTER*7 AC1(NATMMX),AC2(NATMMX),CELDEF(2,3),ACC,ACI1,ACJ1,
     .            ACI2,ACJ2
      CHARACTER   STRING*80,MOT*12
      LOGICAL     FSCALE,FFRACT
 
      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2
 
      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),EXPD(NSYMMX),
     .              EXPD2(NSYMMX),EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),CP1(NSYMMX),
     .              CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),CF1(NSYMMX),
     .              CF2(NSYMMX),COULS(NSYMMX,NAEQMX),COULP(NSYMMX),
     .              COULD(NSYMMX),COULF(NSYMMX),C(NATMMX,3)
 
      COMMON /BOND1/IDAT(NBONMX,2)
      CHARACTER*7 IDAT
 
      COMMON /BOND2/IDTR(NBONMX,3),DDX(NBONMX),IDBOND,INEXT(NBONMX),
     .              IFIRST,ITYPE(14)
      COMMON /CELL/DIST(3,3)
 
      COMMON /FRACTC/FC(NATMMX,3)

      COMMON /INDX/NCRYST,NCEL1,NCEL2,NCEL3,XK(3),WEIGHT,TWOPI,ND1,
     .             NGRAD,NFULL,CEL1,CEL2
      LOGICAL CEL1,CEL2
 
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
 
      COMMON /KEYWRD/ KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      COMMON /LATCON/ALAT,BLAT,CLAT,ALPLAT,BETLAT,GAMLAT

      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,IREST,IRESTV
 
      COMMON /VARGEO/VARINC(NVARMX),IVRLIN(NVARMX),ISTPMX(NVARMX),
     .               IVARTP(NVARMX),ISTEPS(NVARMX),IVARY,NVARY,
     .               NVARLN,NNCPL,LCOUPL(NVARMX+1)
      LOGICAL LCOUPL
 
      COMMON /ZMAT/BL(NATMMX),ALPH(NATMMX),BET(NATMMX),IZ(NATMMX,3),NZ
 
      COMMON /ZMATN/ANZ(NATMMX)
      CHARACTER ANZ*7
 
      PARAMETER(RHOMAX=45.0D0)
C
C If geometry is varied (second to NVARYth step), skip reading of CELL
C cards.
C
      FFRACT=(INDEX(KEYWRD,'FRACT') .GT. 0)         
      IF (NVARY .EQ. 0) THEN
        DO 7 I=1,2
          DO 5 J=1,3
            CELDEF(I,J)='       '
5         CONTINUE
7       CONTINUE

C
C read the "cell parameters" in the case of BAND option
C
        IPOS=INDEX(KEYWRD,'BAND')
        IF ((INDEX(SLINE,'CELL') .NE. 0) .AND. (IPOS .EQ. 0)) THEN
          MOT='BAND'
          CALL ADDK(MOT)
          IPOS=1
        ENDIF
        ISOUS=0
        IF (IPOS .EQ. 0) GOTO 50
        IF (INDEX(SLINE,'CELL') .EQ. 0) THEN
          STRING='CELL line expected.'
          CALL WRITEL(STRING)
        ENDIF
        NCEL1=0
        NCEL2=0
        NCEL3=0

        IF (INDEX(SLINE,'CELL') .NE. 0) THEN
          NCRYST=1
          IF (.NOT. FFRACT) THEN 
            CALL GETNAM(2,STRING,ITT)
            CELDEF(1,1)=STRING(1:7)
            CALL GETNUM(NN,STRING,ANZ,NZ)
            ID(1,1)=NN
            CALL GETNAM(4,STRING,ITT)
            CELDEF(2,1)=STRING(1:7)
            IF (STRING(2:2) .EQ. '?') ISOUS=ISOUS+1
            CALL GETNUM(NN,STRING,ANZ,NZ)
            ID(2,1)=NN
            IPOS2=ISTART(6)
          ELSE
            ISOUS=ISOUS+1
            IPOS2=ISTART(3)
          ENDIF
          NCEL1=INT(READA(SLINE,IPOS2,80))
          CALL READL
        ENDIF
 
        IF (INDEX(SLINE,'CELL') .NE. 0) THEN   
          NCRYST=2
          IF (.NOT. FFRACT) THEN
            CALL GETNAM(2,STRING,ITT)
            CELDEF(1,2)=STRING(1:7)
            CALL GETNUM(NN,STRING,ANZ,NZ)
            ID(1,2)=NN
            CALL GETNAM(4,STRING,ITT)
            CELDEF(2,2)=STRING(1:7)
            IF (STRING(2:2) .EQ. '?') ISOUS=ISOUS+1
            CALL GETNUM(NN,STRING,ANZ,NZ)
            ID(2,2)=NN
            IPOS2=ISTART(6)
          ELSE
            ISOUS=ISOUS+1
            IPOS2=ISTART(3)
          ENDIF
          NCEL2=INT(READA(SLINE,IPOS2,80))
          CALL READL
        ENDIF 
 
        IF (INDEX(SLINE,'CELL') .NE. 0) THEN
          NCRYST=3
          IF (.NOT. FFRACT) THEN
            CALL GETNAM(2,STRING,ITT)
            CELDEF(1,3)=STRING(1:7)
            CALL GETNUM(NN,STRING,ANZ,NZ)
            ID(1,3)=NN
            CALL GETNAM(4,STRING,ITT)
            CELDEF(2,3)=STRING(1:7)
            IF (STRING(2:2) .EQ. '?') ISOUS=ISOUS+1
            CALL GETNUM(NN,STRING,ANZ,NZ)
            ID(2,3)=NN
            IPOS2=ISTART(6)
          ELSE
            ISOUS=ISOUS+1
            IPOS2=ISTART(3)
          ENDIF
          NCEL3=INT(READA(SLINE,IPOS2,80))
          CALL READL
 
          IF (INDEX(SLINE,'CELL') .NE. 0) THEN
            SLINE='Too many CELL cards'
            CALL WRITEL(SLINE)
            CALL CLEXIT
          ENDIF
        ENDIF
      ENDIF
 
C
C In the case of fractional coordinates, cell vectors have been
C calculated in routine FTOCAR.
C
      IF (.NOT. FFRACT) THEN
        DO 20 I=1,NCRYST
          NNF=ID(2,I)
          NNI=ID(1,I)
          DO 10 J=1,3
   10     DIST(I,J)=C(NNF,J)-C(NNI,J)
   20   CONTINUE
      ENDIF
C
C Remove the dummy positions.
C Generate in every case a SCHAKAL compatible file. 28.12.92/mbs
C For fractional coordinates output only the first varied geometry.
C
   50 IF (FFRACT) THEN
        IF (NVARY .EQ. 0) THEN        
          WRITE(8,30) ALAT,BLAT,CLAT,ALPLAT,BETLAT,GAMLAT
   30     FORMAT('CELL ',6F12.5)
        ENDIF
      ELSE
        IF (NVARY .EQ. 0) THEN
          WRITE(8,40)
   40     FORMAT('CELL 1. 1. 1. 90. 90. 90.')
        ELSE
          WRITE(8,70) IVARY,IVARY
   70     FORMAT('TITL Variation # ',I6,/,
     .           'CELL 1. 1. 1. 90. 90. 90.',/,
     .           'GROU ',I6)
        ENDIF
      ENDIF
      INATOM=0
      NDUM=0
      DO 110 I=1,NZ
        ACC=ANZ(I)
        IF (ACC(2:2) .EQ. '?') THEN
          NDUM=NDUM+1
          IF (NDUM .EQ. 1) WRITE(IOUT,80)
   80     FORMAT(//,2X,'Cartesian coordinates of dummies :',//,
     .           19X,'#',14X,'x',14X,'y',14X,'z',/)
          WRITE(IOUT,150) ACC,NDUM,(C(I,J),J=1,3)
          GOTO 110
        ENDIF
        INATOM=INATOM+1
        AC(INATOM)=ANZ(I)
        DO 90 J=1,3
          C(INATOM,J)=C(I,J)
   90   CONTINUE
        IF (FFRACT) THEN
          IF (NVARY .EQ. 0) THEN
            WRITE(8,100) AC(I)(1:2),AC(I)(4:7),
     .                 (FC(I,IK),IK=1,3)
          ENDIF
        ELSE 
          WRITE(8,100) AC(INATOM)(1:2),AC(INATOM)(4:7),
     .                 (C(INATOM,IK),IK=1,3)
        ENDIF
  100   FORMAT('ATOM ',A,A,3(5X,F12.8))
  110 CONTINUE
      IF (FFRACT) THEN
        IF (NVARY .EQ. 0) WRITE(8,120)
      ELSE
        IF (INDEX(KEYWRD,'VARIATION') .EQ. 0) THEN
          WRITE(8,120)
  120     FORMAT('BOX',/,'END')
        ELSE
          ITEMP=IVARY
          IF (NVARY .EQ. 0) ITEMP=1
          WRITE(8,130) ITEMP
  130     FORMAT('BOX',/,'END  ',I6)
        ENDIF
      ENDIF
C
C print out the cartesian coordinates
C
      WRITE(IOUT,140)
  140 FORMAT(//,2X,'Cartesian coordinates:',//,
     .       4X,'Name',6X,'#',14X,'x',14X,'y',14X,'z',/)
      WRITE(IOUT,150) (AC(I),I,(C(I,J),J=1,3),I=1,INATOM)
  150 FORMAT(3X,A7,I5,3X,3F15.6)
      INATOM=INATOM-NCRYST+ISOUS
C
C check for superposition of atoms
C
      DO 170 I=1,INATOM-1
        DO 160 J=I+1,INATOM
          XI=C(I,1)
          XJP=C(J,1)+1.0D-3
          XJM=C(J,1)-1.0D-3
          IF ((XI .LT. XJM) .OR. (XI .GT. XJP)) GOTO 160
          YI=C(I,2)
          YJP=C(J,2)+1.0D-3
          YJM=C(J,2)-1.0D-3
          IF ((YI .LT. YJM) .OR. (YI .GT. YJP)) GOTO 160
          ZI=C(I,3)
          ZJP=C(J,3)+1.0D-3
          ZJM=C(J,3)-1.0D-3
          IF ((ZI .LT. ZJM) .OR. (ZI .GT. ZJP)) GOTO 160
          SLINE='Two atoms are on the same positions : '//AC(I)//AC(J)
          CALL WRITEL(SLINE)
          CALL CLEXIT
  160   CONTINUE
  170 CONTINUE
C
C check for identical names
C
      DO 190 I=1,INATOM-1
        DO 180 J=I+1,INATOM
          IF (AC(I) .NE. AC(J)) GOTO 180
          SLINE='Two atoms have the same name: '//AC(I)
          CALL WRITEL(SLINE)
          CALL CLEXIT
  180   CONTINUE
  190 CONTINUE
C
C print out the "cell parameters"
C
      IF (IPOS .EQ. 0) GOTO 270
      WRITE(IOUT,200) NCRYST,NCEL1,NCEL2,NCEL3
  200 FORMAT(//,2X,'Crystal dimension =',I3,/,
     .       2X,'Nearest neighbors =',I3,' ,',I3,' ,',I3)
 

      WRITE(IOUT,210)
  210 FORMAT(/,2X,'Unit cell basis vectors :',//,
     .       2X,' Atom 1 ',2X,'Atom 2 ',11X,'x',19X,'y',19X,'z')
      DO 230 K=1,NCRYST
        WRITE(IOUT,220) CELDEF(1,K),CELDEF(2,K),(DIST(K,J),J=1,3)
  220   FORMAT(3X,A7,3X,A7,2X,F15.8,5X,F15.8,5X,F15.8)
  230 CONTINUE
C
C Check cell dimensions. 20.02.93/mbs
C
      FSCALE=.FALSE.
      DO 250 K=1,NCRYST
        X=DIST(K,1)*DIST(K,1)+DIST(K,2)*DIST(K,2)+DIST(K,3)*DIST(K,3)
        X=DSQRT(X)
        IF (X .GT. 0.5D0) GOTO 240
        SLINE='Cell parameters incorrect.'
        CALL WRITEL(SLINE)
        CALL CLEXIT
  240   IF (K .EQ. 1) THEN
          X1=X
          IF (DBLE(NCEL1+1)*X1 .GT. RHOMAX) THEN
            FSCALE=.TRUE.
            XSCALE=X1
            NCEL1=INT(RHOMAX/XSCALE+0.5D0)-1
          ENDIF
        ENDIF
        IF (K .EQ. 2) THEN
          X2=X
          IF (DBLE(NCEL2+1)*X2 .GT. RHOMAX) THEN
            FSCALE=.TRUE.
            XSCALE=SQRT(X1*X1+X2*X2)
            NCEL1=INT(RHOMAX/XSCALE+0.5D0)-1
            NCEL2=INT(RHOMAX/XSCALE+0.5D0)-1
          ENDIF
        ENDIF
        IF (K .EQ. 3) THEN
          X3=X
          IF (DBLE(NCEL3+1)*X3 .GT. RHOMAX) THEN
            FSCALE=.TRUE.
            XSCALE=SQRT(X1*X1+X2*X2+X3*X3)
            NCEL1=INT(RHOMAX/XSCALE+0.5D0)-1
            NCEL2=INT(RHOMAX/XSCALE+0.5D0)-1
            NCEL3=INT(RHOMAX/XSCALE+0.5D0)-1
          ENDIF
        ENDIF
  250 CONTINUE
      IF (FSCALE) THEN
        WRITE(IOUT,260) NCEL1,NCEL2,NCEL3
  260   FORMAT(//,2X,'The number of nearest neighbors has been scaled.',
     .         //,2X,'Nearest neighbors =',I3,' ,',I3,' ,',I3)
      ENDIF
C
C calculate the distances if required
C DISTM= and DISTN= options
C
  270 WRITE(IOUT,280) RHO
  280 FORMAT(/,'  Rho=',F6.2,/)
 
      IDBOND=0
      DDMAX=0.0D0
      DDMIN=100.0D0
      NMAXS=NBONMX-1
      IF (INDEX(KEYWRD,'DIST') .EQ. 0) GOTO 290
      DMMIN=1.0D-5
      I=INDEX(KEYWRD,'DISTM=')
      IF (I .EQ. 0) THEN
        IF (INDEX(KEYWRD,'BAND') .NE. 0) THEN
          DMMAX=3.5D0
        ELSE
          DMMAX=5.0D0
        ENDIF
      ENDIF
      IF (I .NE. 0) THEN
        DMMAX=DISTL
      ENDIF
 
      I=INDEX(KEYWRD,'DISTN=')
      IF (I .EQ. 0) THEN
        NMAX=NBONMX-1
      ELSE
        NMAX=IDISTN
        NMAX=NMAX+1
        IF (NMAX .GT. 2000) NMAX=NBONMX
        NMAXS=NMAX-1
      ENDIF
 
      ICHO1=0
      IF (INDEX(SLINE,'DIST') .EQ. 0) ICHO1=1
      CALL DISTC(C,AC,INATOM,C,AC,INATOM,DMMIN,DMMAX,ICHO1,NMAX,DDMAX,
     .           DDMIN)
  290 CONTINUE
C
C DIST cards
C
  300 NMAX=NBONMX-1
      IF (INDEX(SLINE,'DIST') .EQ. 0) GOTO 360
      DMIN=READA(SLINE,ISTART(6),80)
      DMAX=READA(SLINE,ISTART(8),80)
      IF (DMAX .LT. DMIN) THEN
        TEMP=DMIN
        DMIN=DMAX
        DMAX=TEMP
      ENDIF
 
      IT=0
      CALL GETNAM(2,STRING,IT)
      ACI1=STRING(1:7)
      ACI2=ACI1
      NAT1=0
      DO 320 I=1,INATOM
        ACC=AC(I)
        IF (STRING(1:2) .EQ. 'XX') ACI1=AC(I)
        IF (STRING(4:5) .EQ. 'XX') ACI1(4:5)=ACC(4:5)
        DO 310 K=1,7
          IF (K .GT. 2) THEN
            IF ((ACC(K:K) .NE. ACI1(K:K)) .AND. (ACI1(K:K) .NE. ' '))
     .      GOTO 320
          ELSE
            IF (ACC(K:K) .NE. ACI1(K:K)) GOTO 320
          ENDIF
  310   CONTINUE
        NAT1=NAT1+1
        C1(NAT1,1)=C(I,1)
        C1(NAT1,2)=C(I,2)
        C1(NAT1,3)=C(I,3)
        AC1(NAT1)=AC(I)
  320 CONTINUE
 
      IT=0
      CALL GETNAM(4,STRING,IT)
      ACJ1=STRING(1:7)
      ACJ2=ACJ1
      NAT2=0
      DO 340 I=1,INATOM
        ACC=AC(I)
        IF (STRING(1:2) .EQ. 'XX') ACJ1=AC(I)
        IF (STRING(4:5) .EQ. 'XX') ACJ1(4:5)=ACC(4:5)
        DO 330 K=1,7
          IF (K .GT. 2) THEN
            IF ((ACC(K:K) .NE. ACJ1(K:K)) .AND. (ACJ1(K:K) .NE. ' '))
     .      GOTO 340
          ELSE
            IF (ACC(K:K) .NE. ACJ1(K:K)) GOTO 340
          ENDIF
  330   CONTINUE
        NAT2=NAT2+1
        C2(NAT2,1)=C(I,1)
        C2(NAT2,2)=C(I,2)
        C2(NAT2,3)=C(I,3)
        AC2(NAT2)=AC(I)
  340 CONTINUE
 
      CALL READL
      STRING='Error in DIST card: Atom unknown.'
      IF ((NAT1 .EQ. 0) .OR. (NAT2 .EQ. 0)) CALL WRITEL(STRING)
      IF ((NAT1 .GE. 1) .AND. (NAT2 .GE. 1)) THEN
        WRITE(IOUT,350) ACI2,ACJ2,DMIN,DMAX
  350   FORMAT('  DIST    from ',A7,3X,'to ',A7,3X,'between',F10.5,'A',
     .         2X,'and',F10.5,'A')
        CALL DISTC(C1,AC1,NAT1,C2,AC2,NAT2,DMIN,DMAX,ICHO1,NMAX,DDMAX,
     .             DDMIN)
        IF (INDEX(SLINE,'DIST') .EQ. 0) ICHO1=1
        CALL DISTC(C2,AC2,NAT2,C1,AC1,NAT1,DMIN,DMAX,ICHO1,NMAX,DDMAX,
     .             DDMIN)
      ENDIF
      GOTO 300
C
C print out the distances
C
  360 IF (IDBOND .EQ. 0) GOTO 400
      WRITE(IOUT,370) DMMAX
  370 FORMAT(/,2X,'Distances :',//,3X,'dmax (input)=',F10.5)
      WRITE(IOUT,380) NMAXS,DDMIN,DDMAX
  380 FORMAT(3X,'Restriction to ',I4,' bonds,  dmin = ',F10.5,
     .       '  dmax = ',F10.5)
      WRITE(IOUT,390) IDBOND
  390 FORMAT(3X,'A total of ',I4,' bonds has been generated',/)
      CALL PRBOND
C
C sort the bonds
C
  400 IOVLP=INDEX(KEYWRD,'OVLPOP')
      IF ((IDBOND .EQ. 0) .AND. (IOVLP .NE. 0)) THEN
        SLINE='At least one bond is needed for the option OVLPOP.'
        CALL WRITEL (SLINE)
        CALL CLEXIT
      ENDIF
      IF (IOVLP .NE. 0) CALL RBOND
C
C stop if option CHECKGEO
C
      IF (INDEX(KEYWRD,'CHECKGEO') .NE. 0) CALL CLEXIT
 
      RETURN
      END
