C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  MOVLAP                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates the overlap matrix.                     *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 05.05.92/mbs, 20.02.93/mbs              *
C *               (Martin Braendle c/o  Calzaferri group, Instit. f. *
C *               Inorg. and Phys. Chem., Freiestr. 3, CH-3012 Berne)*
C *  Remarks....: Slight modification of COMMON blocks.              *
C *               Comment about parameter RHOMAX see routine WRTKEY. *
C *                                                                  *
C *  Called from: SMAT                                               *
C *                                                                  *
C *  Calls......: MOV                                                *
C *                                                                  *
C ********************************************************************

      SUBROUTINE MOVLAP(S,SORB,NATOM2,NATOM,NDIM)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'  

      DIMENSION S(NDIM,NDIM)
      DIMENSION PTR(9),DTR(25),FTR(49)
      INTEGER SORB(NATOM2)

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),
     .              EXPD(NSYMMX),EXPD2(NSYMMX),
     .              EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),
     .              CF1(NSYMMX),CF2(NSYMMX),COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),X(NATMMX),
     .              Y(NATMMX),Z(NATMMX)

      COMMON /EQS/NAEQS(NATMMX)
      INTEGER NAEQS

      COMMON /DEX/ADD(3),NB,NE,ISUB,NSUB
      COMMON /LOCLAP/SK1,SK2,RR,L1,L2,M,N1,N2,MAXCAL
      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,IREST,IRESTV

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      PARAMETER (RHOMAX=45.0D0)

      EQUIVALENCE (PTR(3),CA),(PTR(8),CB)

      DATA SQRT3 /1.732050807568877D0/,
     .     SQRT6 /2.449489743D0/,
     .     SQRT10/3.16227766D0/,
     .     SQRT15/3.872983346D0/
      DATA AUI/1.889644746D0/
      DATA PTR(9) /0.0D0/,
     .     DTR(12)/0.0D0/,
     .     DTR(22)/0.0D0/,
     .     FTR(15)/0.0D0/,
     .     FTR(22)/0.0D0/,
     .     FTR(43)/0.0D0/

      IF (NB .GT. NATOM) GOTO 10
      ASSIGN 30 TO IBRNCH
      GOTO 20
   10 ASSIGN 40 TO IBRNCH
      IM1=NATOM
   20 CONTINUE

      DO 400 I=NB,NE
        ICO=I-ISUB
        KEYI=ISYM(ICO)
        IORBS=SORB(I)
        JND=IORBS-NSUB

        GOTO IBRNCH,(30,40)
   30   IM1=I-1
   40   CONTINUE

        DO 390 J=1,IM1
          KEYJ=ISYM(J)
          JORBS=SORB(J)

          DELX=X(ICO)-X(J)+ADD(1)
          DELY=Y(ICO)-Y(J)+ADD(2)
          DELZ=Z(ICO)-Z(J)+ADD(3)

          RT2=DELX*DELX+DELY*DELY
          RR=DSQRT(RT2+DELZ*DELZ)
          IF (RR .GT. RHO) RR=RHOMAX
          IF (RT2 .GT. 1.0D-10) GOTO 50
          CB=1.0D0
          SB=0.0D0
          SA=0.0D0
          GOTO 60
   50     T=DSQRT(RT2)
          CB=DELX/T
          SB=DELY/T
          SA=T/RR
   60     CA=DELZ/RR
C
C  The transformation matrices are calculated explicitly.
C  PTR is the matrix for projecting the x,y,z orbitals
C  onto the local system. the elements are ordered so that first
C  x then y then z is projected onto the z' axis (sigma).
C  Then the 3 are projected onto the x' axis and then the y' (pi).
C  The d orbitals are handled similarly. The order of projection
C  is x2-y2,z2,xy,xz,yz first onto z2'(sigma) and then onto xz' and
C  yz'(pi). Finally the 5 orbitals are projected onto x'2-y'2 and
C  then xy' (delta).
C  For the f-orbitals, the ordering of the orbitals is the same
C  as the order of the projections:
C  z3 (sigma), xz2 & yz2 (pi), xyz & z(x2-y2) (delta),
C  x(x2-3y2) & y(3x2-y2) (phi)
C
C  Those PTR and DTR which are zero are initialized in a data state-
C  ment. CA and CB have been equivalenced to PTR(3) and PTR(8)
C  respectively to save time.
C
          PTR(1)=SA*CB
          PTR(2)=SA*SB
C ...     ptr(3)=ca
          PTR(4)=CA*CB
          PTR(5)=CA*SB
          PTR(6)=-SA
          PTR(7)=-SB
C ...     ptr(8)=cb
C ...     ptr(9)=0.0D0
          IF (ND(KEYI)+ND(KEYJ) .EQ. 0) GOTO 80

          CA2=CA*CA
          SA2=SA*SA
          CB2=CB*CB
          SB2=SB*SB
          CBSB=CB*SB
          CASA=CA*SA
          CB2SB2=CB2-SB2
          DTR(1)=SQRT3*0.5D0*SA2*CB2SB2
          DTR(2)=1.0D0-1.5D0*SA2
          DTR(3)=SQRT3*CBSB*SA2
          DTR(4)=SQRT3*CASA*CB
          DTR(5)=SQRT3*CASA*SB
          DTR(6)=CASA*CB2SB2
          DTR(7)=-SQRT3*CASA
          DTR(8)=2.0D0*CASA*CBSB
          DTR(9)=CB*(CA2-SA2)
          DTR(10)=SB*(CA2-SA2)
          DTR(11)=-2.0D0*SA*CBSB
C ...     dtr(12)=0.0D0
          DTR(13)=SA*CB2SB2
          DTR(14)=-PTR(5)
          DTR(15)=PTR(4)
          IF (ND(KEYI)*ND(KEYJ) .EQ. 0) GOTO 70

          DTR(16)=0.5D0*(1.0D0+CA2)*CB2SB2
          DTR(17)=0.5D0*SQRT3*SA2
          DTR(18)=CBSB*(1.0D0+CA2)
          DTR(19)=-CASA*CB
          DTR(20)=-CASA*SB
          DTR(21)=-2.0D0*CA*CBSB
C ...     dtr(22)=0.0D0
          DTR(23)=CA*CB2SB2
          DTR(24)=PTR(2)
          DTR(25)=-PTR(1)
   70     IF (NF(KEYI)+NF(KEYJ) .EQ. 0) GOTO 80

          S2B=2.0D0*SB*CB
          SA3=SA2*SA
          C2B=(CB2-SB2)
          S3B=(C2B*SB+S2B*CB)
          C3B=(C2B*CB-S2B*SB)
          S2A=2.0D0*SA*CA
          FTR(1)=0.5D0*CA*(5.0D0*CA2-3.0D0)
          FTR(2)=SQRT6*0.25D0*CB*SA*(5.0D0*CA2-1.0D0)
          FTR(3)=SQRT6*0.25D0*SB*SA*(5.0D0*CA2-1.0D0)
          FTR(4)=SQRT15*0.5D0*S2B*CA*SA2
          FTR(5)=SQRT15*0.5D0*C2B*CA*SA2
          FTR(6)=SQRT10*0.25D0*C3B*SA3
          FTR(7)=SQRT10*0.25D0*S3B*SA3
          FTR(8)=-SQRT6*0.25D0*SA*(5.0D0*CA2-1.0D0)
          FTR(9)=0.25D0*CB*CA*(15.0D0*CA2-11.0D0)
          FTR(10)=0.25D0*SB*CA*(15.0D0*CA2-11.0D0)
          FTR(11)=SQRT10*0.25D0*S2B*SA*(3.0D0*CA2-1.0D0)
          FTR(12)=SQRT10*0.25D0*C2B*SA*(3.0D0*CA2-1.0D0)
          FTR(13)=SQRT15*0.25D0*C3B*CA*SA2
          FTR(14)=SQRT15*0.25D0*S3B*CA*SA2
C ...     ftr(15)=0.0D0
          FTR(16)=-0.25D0*SB*(5.0D0*CA2-1.0D0)
          FTR(17)=0.25D0*CB*(5.0D0*CA2-1.0D0)
          FTR(18)=SQRT10*0.25D0*C2B*S2A
          FTR(19)=-SQRT10*S2B*S2A*0.25D0
          FTR(20)=-SQRT15*0.25D0*S3B*SA2
          FTR(21)=SQRT15*0.25D0*C3B*SA2
          IF (ND(KEYI)*ND(KEYJ) .EQ. 0) GOTO 80

          C2A=CA2-SA2
C ...     ftr(22)=0.0D0
          FTR(23)=SQRT10*0.5D0*SB*CA*SA
          FTR(24)=-SQRT10*0.5D0*CB*CA*SA
          FTR(25)=C2B*C2A
          FTR(26)=-S2B*C2A
          FTR(27)=-SQRT6*0.25D0*S3B*S2A
          FTR(28)=SQRT6*0.25D0*C3B*S2A
          FTR(29)=SQRT15*0.5D0*CA*SA2
          FTR(30)=SQRT10*0.25D0*CB*SA*(1.0D0-3.0D0*CA2)
          FTR(31)=SQRT10*0.25D0*SB*SA*(1.0D0-3.0D0*CA2)
          FTR(32)=0.5D0*S2B*CA*(3.0D0*CA2-1.0D0)
          FTR(33)=0.5D0*C2B*CA*(3.0D0*CA2-1.0D0)
          FTR(34)=SQRT6*0.25D0*C3B*SA*(1.0D0+CA2)
          FTR(35)=SQRT6*0.25D0*S3B*SA*(1.0D0+CA2)
          IF (NF(KEYI)*NF(KEYJ) .EQ. 0) GOTO 80

          FTR(36)=-SQRT10*0.25D0*SA3
          FTR(37)=SQRT15*0.25D0*CB*CA*SA2
          FTR(38)=SQRT15*0.25D0*SB*CA*SA2
          FTR(39)=-SQRT6*0.25D0*S2B*SA*(1.0D0+CA2)
          FTR(40)=-SQRT6*0.25D0*C2B*SA*(1.0D0+CA2)
          FTR(41)=0.25D0*C3B*CA*(3.0D0+CA2)
          FTR(42)=0.25D0*S3B*CA*(3.0D0+CA2)
C ...     ftr(43)=0.0D0
          FTR(44)=-SQRT15*0.25D0*SB*SA2
          FTR(45)=SQRT15*0.25D0*CB*SA2
          FTR(46)=-SQRT6*0.25D0*C2B*S2A
          FTR(47)=SQRT6*0.25D0*S2B*S2A
          FTR(48)=-0.25D0*S3B*(1.0D0+3.0D0*CA2)
          FTR(49)=0.25D0*C3B*(1.0D0+3.0D0*CA2)
   80     RR=RR*AUI
C
C     (s(i):s(j)).
          DO 83 NCOUNT=1,NAEQS(KEYJ)
            N2=NS(KEYJ,NCOUNT)
            DO 85 KCOUNT=1,NAEQS(KEYI)
              N1=NS(KEYI,KCOUNT)
              IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 85
              CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,0,0,1)
              S(JORBS+NCOUNT-1,JND+KCOUNT-1)=SIGMA
   85       CONTINUE   
   83     CONTINUE
          IS=JND+NAEQS(KEYI)-1

C
C     (p(i):s(j)).
C

   90     N1=NP(KEYI)
          DO 110 NCOUNT=1,NAEQS(KEYJ)
            N2=NS(KEYJ,NCOUNT)
            IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 110
            CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,1,0,1)
            SIGMA=-SIGMA
            DO 100 IC=1,3
  100       S(JORBS+NCOUNT-1,IS+IC)=PTR(IC)*SIGMA
  110     CONTINUE
          JS=JORBS+NAEQS(KEYJ)-1

C
C     (s(i):p(j)).
C
          N2=NP(KEYJ)
          DO 120 KCOUNT=1,NAEQS(KEYI)
            N1=NS(KEYI,KCOUNT)
            IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 120
            CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,0,1,1)
            DO 130 IC=1,3
  130       S(JS+IC,JND+KCOUNT-1)=PTR(IC)*SIGMA
  120     CONTINUE

C
C     (p(i):p(j)).
C
          N1=NP(KEYI)
          N2=NP(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 150
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,1,1,2)
          SIGMA=-SIGMA
          DO 140 JC=1,3
            DO 140 IC=JC,3
              S(JS+JC,IS+IC)=PTR(JC)*PTR(IC)*SIGMA+(PTR(JC+3)*
     .                       PTR(IC+3)+PTR(JC+6)*PTR(IC+6))*PI
  140     S(JS+IC,IS+JC)=S(JS+JC,IS+IC)

C
C     (s(i):d(j)).
C

  150     N2=ND(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 170
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,0,2,1)
          JD=JS+3
          DO 160 IC=1,5
  160     S(JD+IC,IS)=DTR(IC)*SIGMA
C
C     (p(i):d(j)).
C
  170     N1=NP(KEYI)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 190
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,1,2,2)
          SIGMA=-SIGMA
          DO 180 IC=1,3
            DO 180 JC=1,5
  180     S(JD+JC,IS+IC)=DTR(JC)*PTR(IC)*SIGMA+(DTR(JC+5)*PTR(IC+3)+
     .                   DTR(JC+10)*PTR(IC+6))*PI
C
C     (d(i):s(j)).
C
  190     N1=ND(KEYI)
          N2=NS(KEYJ,1)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 210
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,2,0,1)
          ID=IS+3
          DO 200 IC=1,5
  200     S(JS,ID+IC)=DTR(IC)*SIGMA
C
C     (d(i):p(j)).
C
  210     N2=NP(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 230
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,2,1,2)
          PI=-PI
          ID=IS+3
          DO 220 JC=1,3
            DO 220 IC=1,5
  220     S(JS+JC,ID+IC)=PTR(JC)*DTR(IC)*SIGMA+(PTR(JC+3)*
     .                      DTR(IC+5)+PTR(JC+6)*DTR(IC+10))*PI
C
C     (d(i):d(j)).
C
  230     N2=ND(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 250
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,2,2,3)
          PI=-PI
          JD=JS+3
          DO 240 IC=1,5
            ID=IS+3
            DO 240 JC=1,5
              S(JD+JC,ID+IC)=DTR(IC)*DTR(JC)*SIGMA+(DTR(IC+5)*DTR(JC+5)+
     .                       DTR(IC+10)*DTR(JC+10))*PI+(DTR(IC+15)*
     .                       DTR(JC+15)+DTR(IC+20)*DTR(JC+20))*DELTA
  240     S(JD+IC,ID+JC)=S(JD+JC,ID+IC)
C
C     (s(i):f(j))
C
  250     N1=NS(KEYI,1)
          N2=NF(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 270
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,0,3,1)
          JF=JS+8
          DO 260 IC=1,7
  260     S(JF+IC,IS)=FTR(IC)*SIGMA
C
C     (p(i):f(j))
C
  270     N1=NP(KEYI)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 290
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,1,3,2)
          SIGMA=-SIGMA
          JF=JS+8
          DO 280 IC=1,3
            DO 280 JC=1,7
  280     S(JF+JC,IS+IC)=FTR(JC)*PTR(IC)*SIGMA+(FTR(JC+7)*PTR(IC+3)+
     .                    FTR(JC+14)*PTR(IC+6))*PI
C
C     (f(i):s(j))
C
  290     N1=NF(KEYI)
          N2=NS(KEYJ,1)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 310
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,3,0,1)
          SIGMA=-SIGMA
          IF=IS+8
          DO 300 IC=1,7
  300     S(JS,IF+IC)=FTR(IC)*SIGMA
C
C     (f(i):p(j))
C
  310     N2=NP(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 330
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,3,1,2)
          SIGMA=-SIGMA
          IF=IS+8
          DO 320 JC=1,3
            DO 320 IC=1,7
  320     S(JS+JC,IF+IC)=PTR(JC)*FTR(IC)*SIGMA+(PTR(JC+3)*FTR(IC+7)+
     .                      PTR(JC+6)*FTR(IC+14))*PI
C
C     (f(i):d(j))
C
  330     N2=ND(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 350
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,3,2,3)
          SIGMA=-SIGMA
          DELTA=-DELTA
          JD=JS+3
          IF=IS+8
          DO 340 JC=1,5
            DO 340 IC=1,7
  340     S(JD+JC,IF+IC)=FTR(IC)*DTR(JC)*SIGMA+(FTR(IC+7)*DTR(JC+5)+
     .                   FTR(IC+14)*DTR(JC+10))*PI+(FTR(IC+21)*
     .                   DTR(JC+15)+FTR(IC+28)*DTR(JC+20))*DELTA
C
C     (f(i):f(j))
C
  350     N2=NF(KEYJ)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 370
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,3,3,4)
          SIGMA=-SIGMA
          DELTA=-DELTA
          JF=JS+8
          IF=IS+8
          DO 360 JC=1,7
            DO 360 IC=1,7
  360     S(JF+JC,IF+IC)=FTR(IC)*FTR(JC)*SIGMA+(FTR(IC+7)*FTR(JC+7)+
     .                   FTR(IC+14)*FTR(JC+14))*PI+(FTR(IC+21)*
     .                   FTR(JC+21)+FTR(IC+28)*FTR(JC+28))*DELTA+
     .                   (FTR(IC+35)*FTR(JC+35)+FTR(IC+42)*FTR(JC+42))*
     .                   PHI
C
C     (d(i):f(j))
C
  370     N1=ND(KEYI)
          IF ((N1 .EQ. 0) .OR. (N2 .EQ. 0)) GOTO 390
          CALL MOV(SIGMA,PI,DELTA,PHI,ICO,J,RR,N1,N2,2,3,3)
          PI=-PI
          ID=IS+3
          JF=JS+8
          DO 380 JC=1,7
            DO 380 IC=1,5
  380     S(JF+JC,ID+IC)=DTR(IC)*FTR(JC)*SIGMA+(DTR(IC+5)*FTR(JC+7)+
     .                   DTR(IC+10)*FTR(JC+14))*PI+(DTR(IC+15)*
     .                   FTR(JC+21)+DTR(IC+20)*FTR(JC+28))*DELTA
  390   CONTINUE
  400 CONTINUE

      IF (NB .LE. NATOM) RETURN
 
      DO 420 I=1,NDIM
        IF (I .EQ. NDIM) GOTO 420
        L=I+1
        DO 410 J=L,NDIM
          S(I,J)=S(I,J)-S(J,I)
  410   S(J,I)=S(I,J)+2.0D0*S(J,I)
  420 S(I,I)=2.0D0*S(I,I)

      RETURN
      END
