C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  PARAM                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Reads the atomic parameters, AODELETE and AOKEEP   *
C *               options, the number of valence electrons, and the  *
C *               charge.                                            *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 06.05.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne), 13.05.92/mbs,         *
C *               05.06.92/mbs, 17.02.93/mbs, 04.07.94/mbs,          *
C *               02.06.98/db                                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *               Pointer from orbital to atom numbers added.        *
C *               Reading in of electron configuration (card #8,     *
C *               ELCONF) added. The electron configurations of the  *
C *               atoms can be used to calculate the sum over the    *
C *               occupied AO levels and the electrostatic two-body  *
C *               repulsion term. These energies are only calculated *
C *               if no AOs are deleted (maybe changed later)./mbs   *
C *               For distance-dependent Wolfsberg-Helmholz formula, *
C *               assign (individual) (kappa,delta)-values in sub-   *
C *               routine GETKD./mbs                                 *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: WRITEL,READL,ADDK,GAOOCC,GETKD,GETNAM              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE PARAM 

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'  

      DIMENSION   IDZETA(NSYMMX,4),IPARAM(NSYMMX),IDUMMY(NDIMMX)
      CHARACTER   TEX1*11,TEX2*11,LINE*80,TRUNCS*80
      CHARACTER*2 TEXT(4),ELEMS(NSYMMX),ELEMT
      CHARACTER*7 ELEM(NSYMMX),AC1
      CHARACTER*10 TEXTF(20)

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),
     .              EXPD(NSYMMX),EXPD2(NSYMMX),
     .              EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),
     .              CF1(NSYMMX),CF2(NSYMMX),COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),
     .              C(NATMMX,3)

      COMMON /EQS/NAEQS(NATMMX)
      INTEGER NAEQS

C
C 13.05.92/mbs
      COMMON /ATOMPT/IATMPT(NDIMMX)
      COMMON /BDATA/NSD(103),NPD(103),NDD(103),NFD(103),COULSD(103),
     .              EXPSS(103),EXPSD1(103),EXPSD2(103),CSD1(103),
     .              CSD2(103),COULPD(103),EXPPS(103),EXPPD1(103),
     .              EXPPD2(103),CPD1(103),CPD2(103),COULDD(103),
     .              EXPDS(103),EXPDD1(103),EXPDD2(103),CDD1(103),
     .              CDD2(103),COULFD(103),EXPFS(103),EXPFD1(103),
     .              EXPFD2(103),CFD1(103),CFD2(103)
      COMMON /BDATA1/IVELD(103)
      COMMON /DELET/IND(NDIMMX),INDIM,IVAL,NORBE(NATMMX),NDEL
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,IREST,IRESTV
      COMMON /TEMP1/ NORBEF(NATMMX)

      COMMON /AOOUT2/INDT(NDIMMX,2)
      CHARACTER INDT*10

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /AVELEC/ZCORE(NATMMX),USECOR(NATMMX)
      LOGICAL USECOR

      COMMON /BDATA2/SYMBD(103)
      CHARACTER SYMBD*2
C
C 05.06.92/mbs
      COMMON /AELCON/AOOCC(NATMMX,4),USECON(NATMMX)
      LOGICAL USECON

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      NDEL=0
C
C search of elements
C AC(I) contains the element names. I ranges from 1 to INATOM, where INATOM 
C is the number atoms in the unit cell. The NELEM DIFFERENT symbols are
C stored in SYMBOL.
C
      NELEM=1
      AC1=AC(1)
      SYMBOL(1)=AC1(1:2)
      ISYM(1)=1

      DO 30 I=2,INATOM
        AC1=AC(I)
        IFOUND=0
        DO 10 J=1,NELEM
          IF (AC1(1:2) .EQ. SYMBOL(J)) THEN
            ISYM(I)=J
            IFOUND=1
            GOTO 20
          ENDIF
   10   CONTINUE

   20   IF (IFOUND .EQ. 0) THEN
          IF (NELEM .EQ. NSYMMX-2) THEN
            SLINE='Number of elements limited.'
            CALL WRITEL (SLINE)
            CALL CLEXIT
          ELSE
            NELEM=NELEM+1
            SYMBOL(NELEM)=AC1(1:2)
            ISYM(I)=NELEM
          ENDIF
        ENDIF
   30 CONTINUE
C
C DZETA cards
C
      IBL=0
      DO 50 I=1,NELEM
        DO 40 J=1,4
          IDZETA(I,J)=0
   40   CONTINUE
        IPARAM(I)=0
   50 CONTINUE

   60 IF (INDEX(SLINE,'DZETA') .EQ. 0) GOTO 100
      IF (IBL .EQ. 0) WRITE (IOUT,110)
      IBL=1
      TEX1='DZETA   '
      IS=INDEX(SLINE,'S.')
      IP=INDEX(SLINE,'P.')
      ID=INDEX(SLINE,'D.')
      IF=INDEX(SLINE,'F.')
      NN=1
      IF (IS .NE. 0) THEN
        NN=NN+1
        TEXT(1)=' s'
      ELSE
        TEXT(1)=' '
      ENDIF
      IF (IP .NE. 0) THEN
        NN=NN+1
        TEXT(2)=' p'
      ELSE
        TEXT(2)='  '
      ENDIF
      IF (ID .NE. 0) THEN
        NN=NN+1
        TEXT(3)=' d'
      ELSE
        TEXT(3)='  '
      ENDIF
      IF (IF .NE. 0) THEN
        NN=NN+1
        TEXT(4)=' f'
      ELSE
        TEXT(4)='  '
      ENDIF

      NN=NN+2
      K=0
      DO 80 I=NN,40
        IPOS=ISTART(I)
        IF (IPOS .EQ. 81) GOTO 80
        ELEMT=SLINE(IPOS:IPOS+1)
        IF (ELEMT(2:2) .EQ. ' ') THEN
          ELEMT(2:2)=ELEMT(1:1)
          ELEMT(1:1)=' '
        ENDIF
        DO 70 J=1,NELEM
          IF (ELEMT .EQ. SYMBOL(J)) THEN
            K=K+1
            ELEMS(K)=ELEMT
            IF (IS .NE. 0) IDZETA(J,1)=1
            IF (IP .NE. 0) IDZETA(J,2)=1
            IF (ID .NE. 0) IDZETA(J,3)=1
            IF (IF .NE. 0) IDZETA(J,4)=1
          ENDIF
   70   CONTINUE
   80 CONTINUE
      WRITE (IOUT,90) TEX1, (TEXT(L),L=1,4), (ELEMS(L),L=1,K)
   90 FORMAT(2X,A8,4A2,' of ',20(1X,A2))
      CALL READL
      GOTO 60
C
C STO cards
C
  100 ITEST=1
      WRITE (IOUT,110)
  110 FORMAT(' ')
  120 IF (INDEX(SLINE,'STO') .EQ. 0) GOTO 240
      IPOS=ISTART(2)
      ELEMT=SLINE(IPOS:IPOS+1)
      IF (ELEMT(2:2) .EQ. ' ') THEN
        ELEMT(2:2)=ELEMT(1:1)
        ELEMT(1:1)=' '
      ENDIF
      IFOUND=0
      DO 130 I=1,NELEM
        IF (ELEMT .EQ. SYMBOL(I)) THEN
          IFOUND=1
          NUMB=I
          IPARAM(I)=1
        ENDIF
  130 CONTINUE
      IF (IFOUND .EQ. 0) NUMB=40
      IF (IFOUND .EQ. 0) GOTO 150
      DO 140 J=1,103
        IF (SYMBOL(NUMB) .NE. SYMBD(J)) GOTO 140
        NUMB1=J
  140 CONTINUE
  150 CONTINUE

      DO 145 KCOUNT=1,NAEQMX
        NS(NUMB,KCOUNT)=0
  145 CONTINUE

      NP(NUMB)=0
      ND(NUMB)=0
      NF(NUMB)=0

      II=2
C
C  NAEQS stores the number of different s orbitals per element
C
      TRUNCS=SLINE
      NAEQS(NUMB)=0     
      DO 147 KCOUNT=1,NAEQMX 
        IF (INDEX(TRUNCS,'S.') .NE. 0) THEN
          II=II+1
          NS(NUMB,KCOUNT)=INT(READA(SLINE,ISTART(II),80))
          TRUNCS=TRUNCS(INDEX(TRUNCS,'S.')+2:80)
          NAEQS(NUMB)=NAEQS(NUMB)+1
        ENDIF
  147 CONTINUE

      IF (INDEX(SLINE,'P.') .NE. 0) THEN
        II=II+1
        NP(NUMB)=INT(READA(SLINE,ISTART(II),80))
      ENDIF

      IF (INDEX(SLINE,'D.') .NE. 0) THEN
        II=II+1
        ND(NUMB)=INT(READA(SLINE,ISTART(II),80))
      ENDIF

      IF (INDEX(SLINE,'F.') .NE. 0) THEN
        II=II+1
        NF(NUMB)=INT(READA(SLINE,ISTART(II),80))
      ENDIF

      CALL READL

      DO 165 KCOUNT=1,NAEQS(NUMB)

        IF (NS(NUMB,KCOUNT) .EQ. 0) GOTO 170
        IF (IFOUND .EQ. 0) GOTO 160
        IF (NS(NUMB,KCOUNT) .NE. NSD(NUMB1)) ITEST=0
        COULS(NUMB,KCOUNT)=READA(SLINE,ISTART(1),80)
        IF (COULS(NUMB,KCOUNT) .EQ. 0.0D0) ITEST=0
        IF (ISTART(2) .LE. 80) EXPS(NUMB,KCOUNT)=
     .                         READA(SLINE,ISTART(2),80)
        IF (EXPS(NUMB,KCOUNT) .EQ. 0.0D0) ITEST=0
        IF (ISTART(3) .LE. 80) CS1(NUMB,KCOUNT)=
     .                         READA(SLINE,ISTART(3),80)
        IF (CS1(NUMB,KCOUNT) .EQ. 0.0D0) CS1(NUMB,KCOUNT)=1.0D0
        IF (CS1(NUMB,KCOUNT) .LT. 0.9999D0) IDZETA(NUMB,KCOUNT)=1
        IF (ISTART(4) .LE. 80) EXPS2(NUMB,KCOUNT)=
     .                         READA(SLINE,ISTART(4),80)
        IF (ISTART(5) .LE. 80) CS2(NUMB,KCOUNT)=
     .                         READA(SLINE,ISTART(5),80)
        IF (CS2(NUMB,KCOUNT) .GT. 0.0001D0) IDZETA(NUMB,KCOUNT)=1
  160   CALL READL

  165 CONTINUE 

  170 IF (NP(NUMB) .EQ. 0) GOTO 190
      IF (IFOUND .EQ. 0) GOTO 180
      IF (NP(NUMB) .NE. NPD(NUMB1)) ITEST=0
      COULP(NUMB)=READA(SLINE,ISTART(1),80)
      IF (COULP(NUMB) .EQ. 0.0) ITEST=0
      IF (ISTART(2) .LE. 80) EXPP(NUMB)=READA(SLINE,ISTART(2),80)
      IF (EXPP(NUMB) .EQ. 0.0D0) ITEST=0
      IF (ISTART(3) .LE. 80) CP1(NUMB)=READA(SLINE,ISTART(3),80)
      IF (CP1(NUMB) .EQ. 0.0D0) CP1(NUMB)=1.0D0
      IF (CP1(NUMB) .LT. 0.9999D0) IDZETA(NUMB,2)=1
      IF (ISTART(4) .LE. 80) EXPP2(NUMB)=READA(SLINE,ISTART(4),80)
      IF (ISTART(5) .LE. 80) CP2(NUMB)=READA(SLINE,ISTART(5),80)
      IF (CP2(NUMB) .GT. 0.0001D0) IDZETA(NUMB,2)=1
  180 CALL READL

  190 IF (ND(NUMB) .EQ. 0) GOTO 210
      IF (IFOUND .EQ. 0) GOTO 200
      IF (ND(NUMB) .NE. NDD(NUMB1)) ITEST=0
      COULD(NUMB)=READA(SLINE,ISTART(1),80)
      IF (COULD(NUMB) .EQ. 0.0D0) ITEST=0
      IF (ISTART(2) .LE. 80) EXPD(NUMB)=READA(SLINE,ISTART(2),80)
      IF (EXPD(NUMB) .EQ. 0.0D0) ITEST=0
      IF (ISTART(3) .LE. 80) CD1(NUMB)=READA(SLINE,ISTART(3),80)
      IF (CD1(NUMB) .EQ. 0.0) CD1(NUMB)=1.0D0
      IF (CD1(NUMB) .LT. 0.9999D0) IDZETA(NUMB,3)=1
      IF (ISTART(4) .LE. 80) EXPD2(NUMB)=READA(SLINE,ISTART(4),80)
      IF (ISTART(5) .LE. 80) CD2(NUMB)=READA(SLINE,ISTART(5),80)
      IF (CD2(NUMB) .GT. 0.0001D0) IDZETA(NUMB,3)=1
  200 CALL READL

  210 IF (NF(NUMB) .EQ. 0) GOTO 230
      IF (IFOUND .EQ. 0) GOTO 220
      IF (NF(NUMB) .NE. NFD(NUMB1)) ITEST=0
      COULF(NUMB)=READA(SLINE,ISTART(1),80)
      IF (COULF(NUMB) .EQ. 0.0) ITEST=0
      IF (ISTART(2) .LE. 80) EXPF(NUMB)=READA(SLINE,ISTART(2),80)
      IF (EXPF(NUMB) .EQ. 0.0D0) ITEST=0
      IF (ISTART(3) .LE. 80) CF1(NUMB)=READA(SLINE,ISTART(3),80)
      IF (CF1(NUMB) .EQ. 0.0D0) CF1(NUMB)=1.0D0
      IF (CF1(NUMB) .LT. 0.9999D0) IDZETA(NUMB,4)=1
      IF (ISTART(4) .LE. 80) EXPF2(NUMB)=READA(SLINE,ISTART(4),80)
      IF (ISTART(5) .LE. 80) CF2(NUMB)=READA(SLINE,ISTART(5),80)
      IF (CF2(NUMB) .GT. 0.0001D0) IDZETA(NUMB,4)=1
  220 CALL READL

  230 GOTO 120
C
C search in the data block file
C
C We do NOT want to include the specialty of more than one orbital of the
C same kind (with different principal quantum number) in this place.
C Hence we set NAEQS(I)=1, otherwise we would have to adopt blkdat.f to 
C include more than one s orbital per element what makes no sense. It is
C much better then to explicitely state the STO in the appropriate KEYWRD
C statement!
C
  240 DO 380 I=1,NELEM
C       NAEQS(I)=1
C This statement has been moved to line 386
C 02.06.98 / db
        IF (IPARAM(I) .EQ. 1) GOTO 380
        IFOUND=0
        DO 250 J=1,103
          IF (SYMBOL(I) .EQ. SYMBD(J)) GOTO 260
  250   CONTINUE
        GOTO 380
  260   IPARAM(I)=1
        NAEQS(I)=1
        NS(I,1)=NSD(J)
        NP(I)=NPD(J)
        ND(I)=NDD(J)
        NF(I)=NFD(J)
        IF ((NS(I,1) .EQ. 0) .AND. (NP(I) .EQ. 0) .AND. (ND(I) .EQ. 0) 
     .      .AND. (NF(I) .EQ. 0)) ITEST=0

        IF (NS(I,1) .EQ. 0) GOTO 290
        COULS(I,1)=COULSD(J)
        IF (COULS(I,1) .EQ. 0.0D0) ITEST=0
        IF (IDZETA(I,1) .GT. 0.9999D0) GOTO 280
        EXPS(I,1)=EXPSS(J)
        IF (EXPS(I,1) .EQ. 0.0D0) THEN
          IDZETA(I,1)=1
          WRITE (IOUT,270) SYMBOL(I)
  270     FORMAT(2X,'Single zeta not available for the s orbital of ',
     .           A2)
          GOTO 280
        ENDIF
        EXPS2(I,1)=0.0D0
        CS1(I,1)=1.0D0
        CS2(I,1)=0.0D0
        GOTO 290
  280   EXPS(I,1)=EXPSD1(J)
        IF (EXPS(I,1) .EQ. 0.0D0) ITEST=0
        EXPS2(I,1)=EXPSD2(J)
        CS1(I,1)=CSD1(J)
        IF (CS1(I,1) .EQ. 0.0D0) ITEST=0
        CS2(I,1)=CSD2(J)

  290   IF (NP(I) .EQ. 0) GOTO 320
        COULP(I)=COULPD(J)
        IF (COULP(I) .EQ. 0.0D0) ITEST=0
        IF (IDZETA(I,2) .GT. 0.9999D0) GOTO 310
        EXPP(I)=EXPPS(J)
        IF (EXPP(I) .EQ. 0.0D0) THEN
          IDZETA(I,2)=1
          WRITE (IOUT,300) SYMBOL(I)
  300     FORMAT(2X,'Single zeta not available for the p orbitals of ',
     .           A2)
          GOTO 310
        ENDIF
        EXPP2(I)=0.0D0
        CP1(I)=1.0D0
        CP2(I)=0.0D0
        GOTO 320
  310   EXPP(I)=EXPPD1(J)
        IF (EXPP(I) .EQ. 0.0D0) ITEST=0
        EXPP2(I)=EXPPD2(J)
        CP1(I)=CPD1(J)
        IF (CP1(I) .EQ. 0.0D0) ITEST=0
        CP2(I)=CPD2(J)

  320   IF (ND(I) .EQ. 0) GOTO 350
        COULD(I)=COULDD(J)
        IF (COULD(I) .EQ. 0.0D0) ITEST=0
        IF (IDZETA(I,3) .GT. 0.9999D0) GOTO 340
        EXPD(I)=EXPDS(J)
        IF (EXPD(I) .EQ. 0.0D0) THEN
          IDZETA(I,3)=1
          WRITE (IOUT,330) SYMBOL(I)
  330     FORMAT(2X,'Single zeta not available for the d orbitals of ',
     .           A2)
          GOTO 340
        ENDIF
        EXPD2(I)=0.0D0
        CD1(I)=1.0D0
        CD2(I)=0.0D0
        GOTO 350
  340   EXPD(I)=EXPDD1(J)
        IF (EXPD(I) .EQ. 0.0D0) ITEST=0
        EXPD2(I)=EXPDD2(J)
        CD1(I)=CDD1(J)
        IF (CD1(I) .EQ. 0.0D0) ITEST=0
        CD2(I)=CDD2(J)

  350   IF (NF(I) .EQ. 0) GOTO 380
        COULF(I)=COULFD(J)
        IF (COULF(I) .EQ. 0.0D0) ITEST=0
        IF (IDZETA(I,4) .GT. 0.9999D0) GOTO 370
        EXPF(I)=EXPFS(J)
        IF (EXPF(I) .EQ. 0.0D0) THEN
          IDZETA(I,4)=1
          WRITE (IOUT,360) SYMBOL(I)
  360     FORMAT(2X,'Single zeta not available for the f orbitals of ',
     .           A2)
          GOTO 370
        ENDIF
        EXPF2(I)=0.0D0
        CF1(I)=1.0D0
        CF2(I)=0.0D0
        GOTO 380
  370   EXPF(I)=EXPFD1(J)
        IF (EXPF(I) .EQ. 0.0D0) ITEST=0
        EXPF2(I)=EXPFD2(J)
        CF1(I)=CFD1(J)
        IF (CF1(I) .EQ. 0.0D0) ITEST=0
        CF2(I)=CFD2(J)
  380 CONTINUE
C
C normalization
C
      DO 420 I=1,NELEM

        DO 385 KCOUNT=1,NAEQS(I)
          IF (CS2(I,KCOUNT) .EQ. 0.0D0) GOTO 385
          S=(4.0D0*EXPS(I,KCOUNT)*EXPS2(I,KCOUNT)/
     .      (EXPS(I,KCOUNT)+EXPS2(I,KCOUNT))**2.0D0)**
     .      (DBLE(NS(I,KCOUNT))+0.5D0)
          S=1.0D0/DSQRT(CS1(I,KCOUNT)**2.0D0+
     .      CS2(I,KCOUNT)**2.0D0+2.0D0*S*CS1(I,KCOUNT)*
     .      CS2(I,KCOUNT))
          CS1(I,KCOUNT)=S*CS1(I,KCOUNT)
          CS2(I,KCOUNT)=S*CS2(I,KCOUNT)
  385   CONTINUE

  390   IF (CP2(I) .EQ. 0.0D0) GOTO 400
        S=(4.0D0*EXPP(I)*EXPP2(I)/(EXPP(I)+EXPP2(I))**2.0D0)**
     .    (DBLE(NP(I)+0.5))
        S=1.0D0/DSQRT(CP1(I)**2.0D0+CP2(I)**2.0D0+2.0D0*S*CP1(I)*CP2(I))
        CP1(I)=S*CP1(I)
        CP2(I)=S*CP2(I)

  400   IF (CD2(I) .EQ. 0.0D0) GOTO 410
        S=(4.0D0*EXPD(I)*EXPD2(I)/(EXPD(I)+EXPD2(I))**2.0D0)**
     .    (DBLE(ND(I))+0.5D0)
        S=1.0D0/DSQRT(CD1(I)**2.0D0+CD2(I)**2.0D0+2.0D0*S*CD1(I)*CD2(I))
        CD1(I)=S*CD1(I)
        CD2(I)=S*CD2(I)

  410   IF (CF2(I) .EQ. 0.0) GOTO 420
        S=(4.0D0*EXPF(I)*EXPF2(I)/(EXPF(I)+EXPF2(I))**2.0D0)**
     .    (DBLE(NF(I))+0.5D0)
        S=1.0D0/DSQRT(CF1(I)**2.0D0+CF2(I)**2.0D0+2.0D0*S*CF1(I)*CF2(I))
        CF1(I)=S*CF1(I)
        CF2(I)=S*CF2(I)
  420 CONTINUE
C
C output
C
      DO 520 I=1,NELEM

        WRITE (IOUT,430) I, SYMBOL(I)
  430   FORMAT(2X,'Element #',I2,'  : ',A2)

        DO 455 KCOUNT=1,NAEQS(I)

          IF (NS(I,KCOUNT) .NE. 0) THEN
            IF (IDZETA(I,KCOUNT) .EQ. 0) THEN
              WRITE(IOUT,440) NS(I,KCOUNT),COULS(I,KCOUNT),
     .                        EXPS(I,KCOUNT),CS1(I,KCOUNT)
  440         FORMAT(2X,I2,'s      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .               F10.5)
            ELSE
              WRITE(IOUT,450) NS(I,KCOUNT),COULS(I,KCOUNT),
     .                        EXPS(I,KCOUNT),CS1(I,KCOUNT),
     .                        EXPS2(I,KCOUNT),CS2(I,KCOUNT)
  450         FORMAT(2X,I2,'s      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .               F10.5,/,25X,'  zeta2=',F10.5,'  c2=',F10.5)
            ENDIF
          ENDIF

  455  CONTINUE

        IF (NP(I) .NE. 0) THEN
          IF (IDZETA(I,2) .EQ. 0) THEN
            WRITE(IOUT,460) NP(I),COULP(I),EXPP(I),CP1(I)
  460       FORMAT(2X,I2,'p      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .             F10.5)
          ELSE
            WRITE(IOUT,470) NP(I),COULP(I),EXPP(I),CP1(I),EXPP2(I),
     .                      CP2(I)
  470       FORMAT(2X,I2,'p      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .             F10.5,/,25X,'  zeta2=',F10.5,'  c2=',F10.5)
          ENDIF
        ENDIF
        IF (ND(I) .NE. 0) THEN
          IF (IDZETA(I,3) .EQ. 0) THEN
            WRITE(IOUT,480) ND(I),COULD(I),EXPD(I),CD1(I)
  480       FORMAT(2X,I2,'d      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .             F10.5)
          ELSE
            WRITE(IOUT,490) ND(I),COULD(I),EXPD(I),CD1(I),EXPD2(I),
     .                      CD2(I)
  490       FORMAT(2X,I2,'d      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .             F10.5,/,25X,'  zeta2=',F10.5,'  c2=',F10.5)
          ENDIF
        ENDIF

        IF (NF(I) .NE. 0) THEN
          IF (IDZETA(I,4) .EQ. 0) THEN
            WRITE(IOUT,500) NF(I),COULF(I),EXPF(I),CF1(I)
  500       FORMAT(2X,I2,'f      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .             F10.5)
          ELSE
            WRITE(IOUT,510) NF(I),COULF(I),EXPF(I),CF1(I),EXPF2(I),
     .                      CF2(I)
  510       FORMAT(2X,I2,'f      Hii=',F10.4,'  zeta1=',F10.5,'  c1=',
     .             F10.5,/,25X,'  zeta2=',F10.5,'  c2=',F10.5)
          ENDIF
        ENDIF
  520 CONTINUE

      IF (ITEST .EQ. 0) THEN
        LINE='Suspicious STO parameters, if ok use the FORCE option'
        CALL WRITEL(LINE)
      ENDIF
C
C type and number of orbitals
C
C We store the name of the atom in the first column of INDT and the name of
C the orbital in the second column, respectively.
C
      INDIM=0
      DO 550 I=1,INATOM
        N=0
        IOLNDM=INDIM
        DO 530 J=1,NELEM
          IF (ISYM(I) .NE. J) GOTO 530
          DO 525 KCOUNT=1,NAEQS(J)
            IF (NS(J,KCOUNT) .NE. 0) THEN
              IF (INDIM .GE. NDIMMX) GOTO 570
              INDT(INDIM+1,1)=AC(I)
              INDT(INDIM+1,2)='s'
              INDIM=INDIM+1
              N=N+1
            ENDIF
  525     CONTINUE
          IF (NP(J) .NE. 0) THEN
            IF (INDIM .GE. NDIMMX-2) GOTO 570
            INDT(INDIM+1,1)=AC(I)
            INDT(INDIM+1,2)='px'
            INDT(INDIM+2,1)=AC(I)
            INDT(INDIM+2,2)='py'
            INDT(INDIM+3,1)=AC(I)
            INDT(INDIM+3,2)='pz'
            INDIM=INDIM+3
            N=N+3
          ENDIF
          IF (ND(J) .NE. 0) THEN
            IF (INDIM .GE. NDIMMX-4) GOTO 570
            INDT(INDIM+1,1)=AC(I)
            INDT(INDIM+1,2)='dx2-y2'
            INDT(INDIM+2,1)=AC(I)
            INDT(INDIM+2,2)='dz2'
            INDT(INDIM+3,1)=AC(I)
            INDT(INDIM+3,2)='dxy'
            INDT(INDIM+4,1)=AC(I)
            INDT(INDIM+4,2)='dxz'
            INDT(INDIM+5,1)=AC(I)
            INDT(INDIM+5,2)='dyz'
            INDIM=INDIM+5
            N=N+5
          ENDIF
          IF (NF(J) .NE. 0) THEN
            IF (INDIM .GE. NDIMMX-6) GOTO 570
            INDT(INDIM+1,1)=AC(I)
            INDT(INDIM+1,2)='fz3'
            INDT(INDIM+2,1)=AC(I)
            INDT(INDIM+2,2)='fxz2'
            INDT(INDIM+3,1)=AC(I)
            INDT(INDIM+3,2)='fyz2'
            INDT(INDIM+4,1)=AC(I)
            INDT(INDIM+4,2)='fxyz'
            INDT(INDIM+5,1)=AC(I)
            INDT(INDIM+5,2)='fz(x2-y2)'
            INDT(INDIM+6,1)=AC(I)
            INDT(INDIM+6,2)='fx(x2-3y2)'
            INDT(INDIM+7,1)=AC(I)
            INDT(INDIM+7,2)='fy(3x2-y2)'
            INDIM=INDIM+7
            N=N+7
          ENDIF
          NORBEF(I)=N
          NORBE(I)=N
  530   CONTINUE
C
C Generate pointer from orbital numbers to atom number.
C 13.05.92/mbs
C
        DO 540 J=IOLNDM+1,INDIM
          IATMPT(J)=I
  540   CONTINUE
  550 CONTINUE

      WRITE(IOUT,560) INDIM
  560 FORMAT(//,2X,'Number of orbitals before restriction=',I4)
      GOTO 580

  570 LINE='Maximum number of atomic orbitals exceeded.'
      CALL WRITEL(LINE)
      CALL CLEXIT
C
C Read in atomic electron configurations, if given by card ELCONF.
C 05.06.92/mbs
C
  580 IF (INDEX(SLINE,'ELCONF') .GT. 0) THEN
        IPOS=INDEX(KEYWRD,'AUFBAU')
        IF (IPOS .GT. 0) THEN
          DO 590 I=IPOS,LEN(KEYWRD)-7
            KEYWRD(I:I)=KEYWRD(I+7:I+7)
  590     CONTINUE
        ENDIF
      ENDIF

  600 IF (INDEX(SLINE,'ELCONF') .GT. 0) THEN
        CALL GETNAM (2,AC1,IT)
        IFOUND=0
        DO 610 I=1,INATOM
          IF (ICHAR(AC1(7:7)) .GT. 32) THEN
            IF (AC1 .EQ. AC(I)) THEN
              USECON(I)=.TRUE.
              IFOUND=1
              AOOCC(I,1)=READA(SLINE,ISTART(3),80)
              AOOCC(I,2)=READA(SLINE,ISTART(4),80)
              AOOCC(I,3)=READA(SLINE,ISTART(5),80)
              AOOCC(I,4)=READA(SLINE,ISTART(6),80)
            ENDIF
          ELSE IF (ICHAR(AC1(5:5)) .GT. 32) THEN
            IF (AC1(1:5) .EQ. AC(I)(1:5)) THEN
              USECON(I)=.TRUE.
              IFOUND=1
              AOOCC(I,1)=READA(SLINE,ISTART(3),80)
              AOOCC(I,2)=READA(SLINE,ISTART(4),80)
              AOOCC(I,3)=READA(SLINE,ISTART(5),80)
              AOOCC(I,4)=READA(SLINE,ISTART(6),80)
            ENDIF
          ELSE
            IF (AC1(1:2) .EQ. AC(I)(1:2)) THEN
              USECON(I)=.TRUE.
              IFOUND=1
              AOOCC(I,1)=READA(SLINE,ISTART(3),80)
              AOOCC(I,2)=READA(SLINE,ISTART(4),80)
              AOOCC(I,3)=READA(SLINE,ISTART(5),80)
              AOOCC(I,4)=READA(SLINE,ISTART(6),80)
            ENDIF
          ENDIF
  610   CONTINUE

        IF (IFOUND .EQ. 0) WRITE(IOUT,620) AC1
  620   FORMAT('ELCONF: Atom ',A7,' not defined.')
        CALL READL
        GOTO 600
      ENDIF ! IF (INDEX(SLINE,'ELCONF') .GT. 0)

  630 IF (INDEX(SLINE,'CORECHG') .GT. 0) THEN
        CALL GETNAM(2,AC1,IT)
        IFOUND=0
        DO 640 I=1,INATOM
          IF (ICHAR(AC1(7:7)) .GT. 32) THEN
            IF (AC1 .EQ. AC(I)) THEN
              USECOR(I)=.TRUE.
              IFOUND=1
              ZCORE(I)=READA(SLINE,ISTART(3),80)
            ENDIF
          ELSE IF (ICHAR(AC1(5:5)) .GT. 32) THEN
            IF (AC1(1:5) .EQ. AC(I)(1:5)) THEN
              USECOR(I)=.TRUE.
              IFOUND=1
              ZCORE(I)=READA(SLINE,ISTART(3),80)
            ENDIF
          ELSE
            IF (AC1(1:2) .EQ. AC(I)(1:2)) THEN
              USECOR(I)=.TRUE.
              IFOUND=1
              ZCORE(I)=READA(SLINE,ISTART(3),80)
            ENDIF
          ENDIF
  640   CONTINUE

        IF (IFOUND .EQ. 0) WRITE(IOUT,650) AC1
  650    FORMAT('CORECHG: Atom ',A7,' not defined.')
        CALL READL
        GOTO 630
      ENDIF ! IF (INDEX(SLINE,'CORECHG') .GT. 0)
C
C Always set up (kappa,delta) matrix.
C
      CALL GETKD
C
C deletion
C
C We do NOT want to include the specialty of more than one orbital of the same 
C kind (with different principal quantum number) in this place.
C
      DO 660 I=1,INDIM
        IND(I)=0
        IDUMMY(I)=0
  660 CONTINUE

      IF (INDEX(SLINE,'AODELETE') .EQ. 0) GOTO 720
  670 IF (INDEX(SLINE,'AODELETE') .EQ. 0) GOTO 690
      CALL SEARCH (TEXTF,NN,ELEM,KK,NORBEF,IDUMMY,NORBE,IND)
      CALL READL
      TEXTF(1)=' AODELETE '
      TEX1='   '
      TEX2='     of    '
      WRITE(IOUT,680) TEXTF(1),TEX1,(TEXTF(L),L=2,NN),TEX2,
     .                (ELEM(L),L=1,KK)
  680 FORMAT(2X,130A)
      GOTO 670

  690 DO 700 I=1,INDIM
        IF (IND(I) .EQ. 1) NDEL=NDEL+1
  700 CONTINUE
      WRITE(IOUT,710) NDEL
  710 FORMAT(2X,I3,' orbitals deleted.')

  720 IKEEP=0
  730 IF (INDEX(SLINE,'AOKEEP') .EQ. 0) GOTO 740
      CALL SEARCH (TEXTF,NN,ELEM,KK,NORBEF,IDUMMY,NORBE,IND)
      CALL READL
      TEXTF(1)=' AOKEEP'
      TEX1='   '
      TEX2='     of    '
      WRITE(IOUT,680) TEXTF(1),TEX1,(TEXTF(L),L=2,NN),TEX2, 
     .                (ELEM(L),L=1,KK)
      IKEEP=1
      GOTO 730

  740 IF (IKEEP .EQ. 0) GOTO 770
      NKEEP=0
      DO 750 I=1,INDIM
        IF (IND(I) .EQ. 0) THEN
          IND(I)=1
        ELSE
          IND(I)=0
          NKEEP=NKEEP+1
        ENDIF
  750 CONTINUE
      WRITE(IOUT,760) NKEEP
  760 FORMAT(2X,I3,' orbitals kept.')
      NDEL=INDIM-NKEEP
C
C modification of norbe and indt
C
  770 J1=1
      II=0
      DO 790 I=1,INATOM
        J2=J1+NORBE(I)-1
        J11=J1
        DO 780 J=J11,J2
          J1=J1+1
          IF (IND(J) .EQ. 1) THEN
            NORBE(I)=NORBE(I)-1
          ELSE
            II=II+1
            INDT(II,1)=INDT(J,1)
            INDT(II,2)=INDT(J,2)
          ENDIF
  780   CONTINUE
  790 CONTINUE

      IF (NDEL .NE. 0) THEN
        WRITE(IOUT,800) II
  800   FORMAT(2X,'Number of orbitals after restriction=',I4)
      ELSE
        WRITE(IOUT,810)
  810   FORMAT(2X,'no restriction')
      ENDIF

      IF ((NDEL .GT. 0) .AND. (INDEX(KEYWRD,'SYMMETRY') .GT. 0)) THEN
        SLINE='Keyword SYMMETRY incompatible with AODELETE or AOKEEP.'
        CALL WRITEL(SLINE)
      ENDIF
C
C Calculate all atomic orbital occupations. Different models will be
C used according to electron configurations (default), Aufbau principle
C (Keywrd AUFBAU) or weighting by 1/Hii ().
C 05.06.92/mbs
C
      IF (((INDEX(KEYWRD,'REPUL') .GT. 0) .OR. 
     .     (INDEX(KEYWRD,'EAOORB') .GT. 0)) .AND. (NDEL .EQ. 0)) THEN
        CALL GAOOCC
      ENDIF
C
C
C # of valence electrons per unit cell and charge
C
      LCHARG=0
      II1=INDEX(KEYWRD,'CHARGE=')
      IF (II1 .NE. 0) LCHARG=ICHARG

      IVAL=NELEC
      II2=INDEX(KEYWRD,'NELEC=')

      IVALC=0
      DO 850 K=1,NELEM
        DO 820 L=1,103
          IF (SYMBOL(K) .EQ. SYMBD(L)) GOTO 830
  820   CONTINUE
  830   DO 840 M=1,INATOM
          AC1=AC(M)
          IF ( .NOT. USECOR(M)) THEN
            IF (AC1(1:2) .EQ. SYMBOL(K)) IVALC=IVALC+IVELD(L)
          ELSE
            IF (AC1(1:2) .EQ. SYMBOL(K)) IVALC=IVALC+INT(ZCORE(M))
          ENDIF
  840   CONTINUE
  850 CONTINUE
      IVALC=IVALC-LCHARG

      IF (II2 .EQ. 0) NELEC=IVALC
      IF (II2 .EQ. 0) IVAL=IVALC

      WRITE(IOUT,860) NELEC
  860 FORMAT(/,2X,'Number of valence electrons per unit cell=',I4)
      IF (II1 .NE. 0) WRITE(IOUT,870) ICHARG
  870 FORMAT(2X,'Charge of the system=',I4)
      WRITE (IOUT,110)

      IF (II2 .NE. 0) THEN
        IF (IVAL .NE. IVALC) THEN
          LINE='Number of calculated electrons different'//
     .         ' from input, if ok use the FORCE option'
          CALL WRITEL(LINE)
        ENDIF
      ENDIF

      RETURN
      END
