C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  RBOND                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Searches equivalent bonds within and between unit  *
C *               cells to be used for band overlap population.      *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 07.05.92/mbs (Martin Braendle c/o       *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: INMODI                                             *
C *                                                                  *
C *  Calls......: WRITEL                                             *
C *                                                                  *
C ********************************************************************

      SUBROUTINE RBOND 

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION   IDTR1(NBONMX,3),DD1(NBONMX),INDEXX(NBONMX),
     .            INEXT2(NBONMX)
      LOGICAL     LO1,LO2,LO3,LO4,LO5,LO6
      CHARACTER*7 IDAT1(NBONMX,2),AC11,AC12,AC21,AC22
      CHARACTER   LINE*80

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /BOND2/IDTR(NBONMX,3),DDX(NBONMX),IDBOND,INEXT(NBONMX),
     .              IFIRST,ITYPE(14)

      COMMON /BOND1/IDAT(NBONMX,2)
      CHARACTER*7 IDAT

      DO 5 I=1,14
        ITYPE(I)=0
    5 CONTINUE

      DO 10 I=1,IDBOND
        INDEXX(I)=1
        INEXT2(I)=0
        I1=IDTR(I,1)
        I2=IDTR(I,2)
        I3=IDTR(I,3)
        IF (I3 .EQ. 0) THEN
          IF (I2 .LT. 0) I2=2
          ISUM=I1+10*I2
        ELSE
          ISUM=100+10*I1+I2
        ENDIF
        INEXT(I)=ISUM
   10 CONTINUE

      IP=1
      IPN=1
   20 IFLAG=0
      DO 30 I=IPN+1,IDBOND
        IF (INDEXX(I) .EQ. 0) GOTO 30
        AC11=IDAT(IP,1)
        AC21=IDAT(I,1)
        AC12=IDAT(IP,2)
        AC22=IDAT(I,2)
        LO1=(AC11(1:5) .EQ. AC21(1:5))
        LO2=(AC12(1:5) .EQ. AC22(1:5))
        LO3=(AC11(1:5) .EQ. AC22(1:5))
        LO4=(AC12(1:5) .EQ. AC21(1:5))
        LO6=((LO1 .AND. LO2) .OR. (LO3 .AND. LO4))
        DDP=DDX(IP)+1.0D-4
        DDM=DDX(IP)-1.0D-4
        LO5=(DDX(I) .GT. DDM) .AND. (DDX(I) .LT. DDP)
        IF (LO6 .AND. LO5) THEN
          INEXT2(IP)=I
          IP=I
          INDEXX(I)=0
        ELSE
          IF (IFLAG .EQ. 0) THEN
            IFLAG=1
            IPN=I
          ENDIF
        ENDIF
   30 CONTINUE
      IF (IFLAG .NE. 0) THEN
        INEXT2(IP)=IPN+10000
        IP=IPN
        GOTO 20
      ENDIF
      IP=MAX(IP,IPN)
      INEXT2(IP)=10000+IDBOND+1

      DO 40 I=1,IDBOND
        INDEXX(I)=I
   40 CONTINUE

      IDM1=IDBOND-1
      DO 60 I=1,IDM1
        IP1=I+1
        DO 50 J=IP1,IDBOND
          II=INDEXX(I)
          IJ=INDEXX(J)
          IF (INEXT(II) .LE. INEXT(IJ)) GOTO 50
          ITEMP=INDEXX(I)
          INDEXX(I)=INDEXX(J)
          INDEXX(J)=ITEMP
   50   CONTINUE
   60 CONTINUE

      DO 90 I=1,IDBOND
        IDAT1(I,1)=IDAT(INDEXX(I),1)
        IDAT1(I,2)=IDAT(INDEXX(I),2)
        DD1(I)=DDX(INDEXX(I))
        IDTR1(I,1)=IDTR(INDEXX(I),1)
        IDTR1(I,2)=IDTR(INDEXX(I),2)
        IDTR1(I,3)=IDTR(INDEXX(I),3)
        IF (INDEXX(I) .EQ. 1) IFIRST=I
        IIT=0
        IN=INEXT2(INDEXX(I))
        IF (IN .GE. 10000) THEN
          IIT=10000
          IN=IN-10000
        ENDIF
        IF (IN .EQ. 0) GOTO 80
        DO 70 J=1,IDBOND
          IF (INDEXX(J) .EQ. IN) GOTO 80
   70   CONTINUE
   80   INEXT(I)=J+IIT
   90 CONTINUE

      IT=1
      DO 240 I=1,IDBOND
        IDAT(I,1)=IDAT1(I,1)
        IDAT(I,2)=IDAT1(I,2)
        DDX(I)=DD1(I)
        IDTR(I,1)=IDTR1(I,1)
        IDTR(I,2)=IDTR1(I,2)
        IDTR(I,3)=IDTR1(I,3)
        GOTO (100,110,120,130,140,150,160,170,180,190,200,210,220,230)
     .       ,IT
  100   IF ((IDTR(I,1) .NE. 0) .OR. (IDTR(I,2) .NE. 0) .OR. 
     .     (IDTR(I,3) .NE. 0)) THEN
          IT=IT+1
          GOTO 110
        ENDIF
        ITYPE(1)=ITYPE(1)+1
        GOTO 240
  110   IF ((IDTR(I,1) .NE. 1) .OR. (IDTR(I,2) .NE. 0) .OR. 
     .     (IDTR(I,3) .NE. 0)) THEN
          IT=IT+1
          GOTO 120
        ENDIF
        ITYPE(2)=ITYPE(2)+1
        GOTO 240
  120   IF ((IDTR(I,1) .NE. 0) .OR. (IDTR(I,2) .NE. 1) .OR. 
     .     (IDTR(I,3) .NE. 0)) THEN
          IT=IT+1
          GOTO 130
        ENDIF
        ITYPE(3)=ITYPE(3)+1
        GOTO 240
  130   IF ((IDTR(I,1) .NE. 1) .OR. (IDTR(I,2) .NE. 1) .OR. 
     .     (IDTR(I,3) .NE. 0)) THEN
          IT=IT+1
          GOTO 140
        ENDIF
        ITYPE(4)=ITYPE(4)+1
        GOTO 240
  140   IF ((IDTR(I,1) .NE. 1) .OR. (IDTR(I,2) .NE. -1) .OR. 
     .     (IDTR(I,3) .NE. 0)) THEN
          IT=IT+1
          GOTO 150
        ENDIF
        ITYPE(5)=ITYPE(5)+1
        GOTO 240
  150   IF ((IDTR(I,1) .NE. -1) .OR. (IDTR(I,2) .NE. -1) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 160
        ENDIF
        ITYPE(6)=ITYPE(6)+1
        GOTO 240
  160   IF ((IDTR(I,1) .NE. -1) .OR. (IDTR(I,2) .NE. 0) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 170
        ENDIF
        ITYPE(7)=ITYPE(7)+1
        GOTO 240
  170   IF ((IDTR(I,1) .NE. -1) .OR. (IDTR(I,2) .NE. 1) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 180
        ENDIF
        ITYPE(8)=ITYPE(8)+1
        GOTO 240
  180   IF ((IDTR(I,1) .NE. 0) .OR. (IDTR(I,2) .NE. -1) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 190
        ENDIF
        ITYPE(9)=ITYPE(9)+1
        GOTO 240
  190   IF ((IDTR(I,1) .NE. 0) .OR. (IDTR(I,2) .NE. 0) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 200
        ENDIF
        ITYPE(10)=ITYPE(10)+1
        GOTO 240
  200   IF ((IDTR(I,1) .NE. 0) .OR. (IDTR(I,2) .NE. 1) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 210
        ENDIF
        ITYPE(11)=ITYPE(11)+1
        GOTO 240
  210   IF ((IDTR(I,1) .NE. 1) .OR. (IDTR(I,2) .NE. -1) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 220
        ENDIF
        ITYPE(12)=ITYPE(12)+1
        GOTO 240
  220   IF ((IDTR(I,1) .NE. 1) .OR. (IDTR(I,2) .NE. 0) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          IT=IT+1
          GOTO 230
        ENDIF
        ITYPE(13)=ITYPE(13)+1
        GOTO 240
  230   IF ((IDTR(I,1) .NE. 1) .OR. (IDTR(I,2) .NE. 1) .OR. 
     .     (IDTR(I,3) .NE. 1)) THEN
          LINE='overlap population can be calculated with'//
     .         ' first nearest neighbors only'
          CALL WRITEL(LINE)
          RETURN
        ENDIF
        ITYPE(14)=ITYPE(14)+1
  240 CONTINUE

      WRITE(IOUT,250)
  250 FORMAT(/,2X,'cumulated overlap population for the bonds : '/)
      N=1
      M=0
      I=IFIRST
  260 M=M+1
      WRITE(IOUT,270) N,M,IDAT(I,1),IDAT(I,2),DDX(I),IDTR(I,1),
     .                IDTR(I,2),IDTR(I,3)
  270 FORMAT(1X,I4,2X,I3,2X,A7,2X,'<--->',2X,A7,5X,F10.6,3X,3I3)
      N=N+1
      IF (N .GT. IDBOND) GOTO 290
      I=INEXT(I)
      IF (I .GE. 10000) THEN
        M=0
        I=I-10000
        WRITE(IOUT,280)
  280 FORMAT(' ')
      ENDIF
      GOTO 260
  290 CONTINUE

      RETURN
      END
