C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  FUNCTION REPUL1                                                 *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates integral over (electron density divided *
C *               difference of core vector and electron vector).    *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *               Calzaferri group                                   *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 11.06.92                                           *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: REPUL                                              *
C *                                                                  *
C *  Calls......: REPUL2                                             *
C *                                                                  *
C ********************************************************************

      DOUBLE PRECISION FUNCTION REPUL1(R,IATOM,KEYI)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      COMMON /AELCON/AOOCC(NATMMX,4),USECON(NATMMX)
      LOGICAL USECON

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),EXPD(NSYMMX),
     .              EXPD2(NSYMMX),EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),
     .              CD2(NSYMMX),CF1(NSYMMX),CF2(NSYMMX),
     .              COULS(NSYMMX,NAEQMX),COULP(NSYMMX),
     .              COULD(NSYMMX),COULF(NSYMMX),C(NATMMX,3)

      PARAMETER (A0=0.52917706,TWOBA0=2.0D0/A0)

      EREP=0.0D0

      N=NF(KEYI)
      IF ((N .GT. 0) .AND. (AOOCC(IATOM,4) .GT. 0.0D0)) THEN
        IF ((NS(KEYI,1)-N) .EQ. 2) THEN
          COEF2=CF2(KEYI)
          IF (COEF2 .GT. 0.0D0) THEN
            E1=REPUL2(N,R,EXPF(KEYI)*TWOBA0)
            E2=REPUL2(N,R,EXPF2(KEYI)*TWOBA0)
            E3=REPUL2(N,R,(EXPF(KEYI)+EXPF2(KEYI))/A0)
            COEF1=CF1(KEYI)
            E=COEF1*COEF1*E1+COEF2*COEF2*E2+2.0D0*COEF1*COEF2*E3
          ELSE
            E=REPUL2(N,R,EXPF(KEYI)*TWOBA0)
          ENDIF
          E=1.0D0/R-E
          EREP=EREP+AOOCC(IATOM,4)*E
        ENDIF
      ENDIF

      N=ND(KEYI)
      IF ((N .GT. 0) .AND. (AOOCC(IATOM,3) .GT. 0.0D0)) THEN
        IF ((NS(KEYI,1)-N) .EQ. 1) THEN
          COEF2=CD2(KEYI)
          IF (COEF2 .GT. 0.0D0) THEN
            E1=REPUL2(N,R,EXPD(KEYI)*TWOBA0)
            E2=REPUL2(N,R,EXPD2(KEYI)*TWOBA0)
            E3=REPUL2(N,R,(EXPD(KEYI)+EXPD2(KEYI))/A0)
            COEF1=CD1(KEYI)
            E=COEF1*COEF1*E1+COEF2*COEF2*E2+2.0D0*COEF1*COEF2*E3
          ELSE
            E=REPUL2(N,R,EXPD(KEYI)*TWOBA0)
          ENDIF
          E=1.0D0/R-E
          EREP=EREP+AOOCC(IATOM,3)*E
        ENDIF
      ENDIF

      N=NP(KEYI)
      IF ((N .GT. 0) .AND. (AOOCC(IATOM,2) .GT. 0.0D0)) THEN
        COEF2=CP2(KEYI)
        IF (COEF2 .GT. 0.0D0) THEN
          E1=REPUL2(N,R,EXPP(KEYI)*TWOBA0)
          E2=REPUL2(N,R,EXPP2(KEYI)*TWOBA0)
          E3=REPUL2(N,R,(EXPP(KEYI)+EXPP2(KEYI))/A0)
          COEF1=CP1(KEYI)
          E=COEF1*COEF1*E1+COEF2*COEF2*E2+2.0D0*COEF1*COEF2*E3
        ELSE
          E=REPUL2(N,R,EXPP(KEYI)*TWOBA0)
        ENDIF
        E=1.0D0/R-E
        EREP=EREP+AOOCC(IATOM,2)*E
      ENDIF

      N=NS(KEYI,1)
      IF ((N .GT. 0) .AND. (AOOCC(IATOM,1) .GT. 0.0D0)) THEN 
        COEF2=CS2(KEYI,1)
        IF (COEF2 .GT. 0.0D0) THEN
          E1=REPUL2(N,R,EXPS(KEYI,1)*TWOBA0)
          E2=REPUL2(N,R,EXPS2(KEYI,1)*TWOBA0)
          E3=REPUL2(N,R,(EXPS(KEYI,1)+EXPS2(KEYI,1))/A0)
          COEF1=CS1(KEYI,1)
          E=COEF1*COEF1*E1+COEF2*COEF2*E2+2.0D0*COEF1*COEF2*E3
        ELSE
          E=REPUL2(N,R,EXPS(KEYI,1)*TWOBA0)
        ENDIF
        E=1.0D0/R-E
        EREP=EREP+AOOCC(IATOM,1)*E
      ENDIF

      REPUL1=EREP
      RETURN
      END
