C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SYMMAN                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Analyses symmetry of crystal orbitals.             *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Institute for Inorganic and Phy- *
C *               sical Chemistry, University of Berne, Freiestr. 3, *
C *               CH-3012 Berne                                      *
C *  Date.......: 20.04.93                                           *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: CRYST                                              *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE SYMMAN(CREAL,CIMAG,NDIM)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION CREAL(NDIM,NDIM),CIMAG(NDIM,NDIM),CREALR(NDIMMX),
     .          CIMAGR(NDIMMX),CBUFR(NDIMMX),CBUFI(NDIMMX)
      INTEGER   SORB(NATMMX)

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM

      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,IREST,IRESTV

      COMMON/SYMANA/TRANSV(MXSYME,3),SYMOP(MXSYME,3,3),
     .              IPERMU(MXSYME,NATMMX,2),ISYMRE(MXSYME,NDIMMX),
     .              NLINOP(3),IPOINT(3,MXSYME),ILINE(3),NSYMEL,
     .              CSYMEL(MXSYME),FSYMOP(MXSYME),FSYMAN(MXSYME),FSYM
      LOGICAL   FSYMOP,FSYM,FSYMAN
      CHARACTER CSYMEL*2

      COMMON /TEMP1/NORBEF(NATMMX)

      PARAMETER (DELTA=1.0D-3)

C
C Clear symmetry analysis results
C
      DO 20 I=1,NSYMEL
        DO 10 J=1,NDIM
          ISYMRE(I,J)=0
   10   CONTINUE
   20 CONTINUE
C
C Read atom to orbital pointer
C
      REWIND 21
      READ(21) (SORB(I),I=1,INATOM)
C
C For each CO in print range, copy coefficients to buffer ...
C
      DO 110 I=1,NDIM
        DO 30 J=1,NDIM
          CBUFR(J)=CREAL(J,I)
          CBUFI(J)=CIMAG(J,I)
   30   CONTINUE
C
C .., apply symmetry operations ..
C
        DO 100 J=1,NSYMEL
          IF (FSYMAN(J)) THEN
            R11=SYMOP(J,1,1)
            R12=SYMOP(J,1,2)
            R13=SYMOP(J,1,3)
            R21=SYMOP(J,2,1)
            R22=SYMOP(J,2,2)
            R23=SYMOP(J,2,3)
            R31=SYMOP(J,3,1)
            R32=SYMOP(J,3,2)
            R33=SYMOP(J,3,3)
C
C ... by going through the permutation list ...
C
            DO 40 IP=1,INATOM
              IATOM1=IPERMU(J,IP,1)
              IATOM2=IPERMU(J,IP,2)
              IORB1=SORB(IATOM1)
              IORB2=SORB(IATOM2)
              NORB1=NORBEF(IATOM1)
C
C s orbitals
C
              CREALR(IORB2)=CBUFR(IORB1)
              CIMAGR(IORB2)=CBUFI(IORB1)
C
C p orbitals
C
              IF (NORB1 .GT. 1) THEN
C px
                CREALR(IORB2+1)=R11*CBUFR(IORB1+1)+
     .                          R12*CBUFR(IORB1+2)+
     .                          R13*CBUFR(IORB1+3)
C py
                CREALR(IORB2+2)=R21*CBUFR(IORB1+1)+
     .                          R22*CBUFR(IORB1+2)+
     .                          R23*CBUFR(IORB1+3)
C pz
                CREALR(IORB2+3)=R31*CBUFR(IORB1+1)+
     .                          R32*CBUFR(IORB1+2)+
     .                          R33*CBUFR(IORB1+3)
C px
                CIMAGR(IORB2+1)=R11*CBUFI(IORB1+1)+
     .                          R12*CBUFI(IORB1+2)+
     .                          R13*CBUFI(IORB1+3)
C py
                CIMAGR(IORB2+2)=R21*CBUFI(IORB1+1)+
     .                          R22*CBUFI(IORB1+2)+
     .                          R23*CBUFI(IORB1+3)
C pz
                CIMAGR(IORB2+3)=R31*CBUFI(IORB1+1)+
     .                          R32*CBUFI(IORB1+2)+
     .                          R33*CBUFI(IORB1+3)
              ENDIF
C
C d orbitals
C
              IF (NORB1 .GT. 4) THEN
C dx2-y2
                CREALR(IORB2+4)=(R11*R11+R22*R22-R12*R12-R21*R21)*
     .                          CBUFR(IORB1+4)+(R13*R13-R23*R23)*
     .                          CBUFR(IORB1+5)+(R11*R12-R21*R22)*
     .                          CBUFR(IORB1+6)+(R11*R13-R21*R23)*
     .                          CBUFR(IORB1+7)+(R12*R13-R22*R23)*
     .                          CBUFR(IORB1+8)
                CREALR(IORB2+4)=0.5D0*CREALR(IORB2+4)
C dz2
                CREALR(IORB2+5)=(R31*R31-R32*R32)*CBUFR(IORB1+4)+
     .                          R33*R33*CBUFR(IORB1+5)+R31*R32*
     .                          CBUFR(IORB1+6)+R13*R33*
     .                          CBUFR(IORB1+7)+R32*R33*
     .                          CBUFR(IORB1+8)
C dxy
                CREALR(IORB2+6)=2.0D0*(R11*R21-R12*R22)*
     .                          CBUFR(IORB1+4)+2.0D0*R13*R23*
     .                          CBUFR(IORB1+5)+(R11*R22+R21*R12)*
     .                          CBUFR(IORB1+6)+(R11*R23+R21*R13)*
     .                          CBUFR(IORB1+7)+(R12*R23+R13*R22)*
     .                          CBUFR(IORB1+8)
C dxz
                CREALR(IORB2+7)=2.0D0*(R11*R31-R12*R32)*
     .                          CBUFR(IORB1+4)+2.0D0*R13*R33*
     .                          CBUFR(IORB1+5)+(R11*R32+R31*R12)*
     .                          CBUFR(IORB1+6)+(R11*R33+R31*R13)*
     .                          CBUFR(IORB1+7)+(R12*R33+R13*R32)*
     .                          CBUFR(IORB1+8)
C dyz
                CREALR(IORB2+8)=2.0D0*(R21*R31-R22*R32)*
     .                          CBUFR(IORB1+4)+2.0D0*R23*R33*
     .                          CBUFR(IORB1+5)+(R21*R32+R31*R22)*
     .                          CBUFR(IORB1+6)+(R21*R33+R31*R23)*
     .                          CBUFR(IORB1+7)+(R22*R33+R23*R32)*
     .                          CBUFR(IORB1+8)
C dx2-y2
                CIMAGR(IORB2+4)=(R11*R11+R22*R22-R12*R12-R21*R21)*
     .                          CBUFI(IORB1+4)+(R13*R13-R23*R23)*
     .                          CBUFI(IORB1+5)+(R11*R12-R21*R22)*
     .                          CBUFI(IORB1+6)+(R11*R13-R21*R23)*
     .                          CBUFI(IORB1+7)+(R12*R13-R22*R23)*
     .                          CBUFI(IORB1+8)
                CIMAGR(IORB2+4)=0.5D0*CIMAGR(IORB2+4)
C dz2
                CIMAGR(IORB2+5)=(R31*R31-R32*R32)*CBUFI(IORB1+4)+
     .                          R33*R33*CBUFI(IORB1+5)+R31*R32*
     .                          CBUFI(IORB1+6)+R13*R33*
     .                          CBUFI(IORB1+7)+R32*R33*
     .                          CBUFI(IORB1+8)
C dxy
                CIMAGR(IORB2+6)=2.0D0*(R11*R21-R12*R22)*
     .                          CBUFI(IORB1+4)+2.0D0*R13*R23*
     .                          CBUFI(IORB1+5)+(R11*R22+R21*R12)*
     .                          CBUFI(IORB1+6)+(R11*R23+R21*R13)*
     .                          CBUFI(IORB1+7)+(R12*R23+R13*R22)*
     .                          CBUFI(IORB1+8)
C dxz
                CIMAGR(IORB2+7)=2.0D0*(R11*R31-R12*R32)*
     .                          CBUFI(IORB1+4)+2.0D0*R13*R33*
     .                          CBUFI(IORB1+5)+(R11*R32+R31*R12)*
     .                          CBUFI(IORB1+6)+(R11*R33+R31*R13)*
     .                          CBUFI(IORB1+7)+(R12*R33+R13*R32)*
     .                          CBUFI(IORB1+8)
C dyz
                CIMAGR(IORB2+8)=2.0D0*(R21*R31-R22*R32)*
     .                          CBUFI(IORB1+4)+2.0D0*R23*R33*
     .                          CBUFI(IORB1+5)+(R21*R32+R31*R22)*
     .                          CBUFI(IORB1+6)+(R21*R33+R31*R23)*
     .                          CBUFI(IORB1+7)+(R22*R33+R23*R32)*
     .                          CBUFI(IORB1+8)
              ENDIF
C
C f orbitals
C
              IF (NORB1 .GT. 9) THEN
                R1111=R11*R11
                R1212=R12*R12
                R1313=R13*R13
                R2121=R21*R21
                R2222=R22*R22
                R2323=R23*R23
                R3333=R33*R33
                R3232=R32*R32
                R3131=R31*R31
C fz3
                CREALR(IORB2+9)=R33*R3333*CBUFR(IORB1+9)+
     .                          R31*R3333*CBUFR(IORB1+10)+
     .                          R32*R3333*CBUFR(IORB1+11)+
     .                          R31*R32*R33*CBUFR(IORB1+12)+
     .                          R33*(R3131-R3232)*CBUFR(IORB1+13)+
     .                          R31*(R3131-3.D0*R3232)*
     .                          CBUFR(IORB1+14)+
     .                          R32*(3.D0*R3131-R3232)*
     .                          CBUFR(IORB1+15)
C fxz2
                CREALR(IORB2+10)=3.0D0*R3333*R13*CBUFR(IORB1+9)+
     .                           (2.D0*R31*R33*R13+R11*R3333)*
     .                           CBUFR(IORB1+10)+
     .                           (2.D0*R32*R33*R13+R12*R3333)*
     .                           CBUFR(IORB1+11)+
     .                           (R31*R32*R13+R31*R12*R33+R11*R32*
     .                           R33)*
     .                           CBUFR(IORB1+12)+
     .                           (2.D0*R33*R31*R11-2.0D0*R33*R32*
     .                           R12+R13*(R3131-R3232))*
     .                           CBUFR(IORB1+13)+
     .                           (2.D0*R3131*R11-6.D0*R31*R32*R12+
     .                           R11*(R3131-3.D0*R3232))*
     .                           CBUFR(IORB1+14)+
     .                           (6.D0*R32*R31*R11-2.D0*R3232*R12+
     .                           R12*(3.D0*R3131-R3232))*
     .                           CBUFR(IORB1+15)
C fyz2
                CREALR(IORB2+11)=3.D0*R3333*R23*CBUFR(IORB1+9)+
     .                           (2.D0*R31*R33*R23+R21*R3333)*
     .                           CBUFR(IORB1+10)+
     .                           (2.D0*R32*R33*R23+R22*R3333)*
     .                           CBUFR(IORB1+11)+
     .                           (R31*R32*R23+R31*R22*R33+R21*R32*
     .                           R33)*CBUFR(IORB1+12)+
     .                           (2.D0*R33*(R31*R21-R32*R22)+R23*
     .                           (R3131-R3232))*CBUFR(IORB1+13)+
     .                           (2.D0*R3131*R21-6.D0*R31*R32*R22+
     .                           R21*(R3131-3.D0*R3232))*
     .                           CBUFR(IORB1+14)+
     .                           (6.D0*R32*R31*R21-2.D0*R3232*R22+
     .                           R22*(3.D0*R3131-R3232))*
     .                           CBUFR(IORB1+15)
C fxyz
                CREALR(IORB2+12)=6.D0*R33*R23*R13*CBUFR(IORB1+9)+
     .                           2.D0*(R13*R31*R23+R13*R21*R33+
     .                           R11*R33*R23)*CBUFR(IORB1+10)+
     .                           2.D0*(R13*R32*R23+R13*R22*R33+
     .                           R12*R33*R23)*CBUFR(IORB1+11)+
     .                           (R13*R31*R22+R13*R21*R32+R23*R31*
     .                           R12+R23*R11*R32+R33*R21*R12+
     .                           R33*R11*R22)*CBUFR(IORB1+12)+
     .                           2.D0*(R33*R21*R11-R33*R22*R12+
     .                           R23*R31*R11-R23*R32*R12+R13*R31*
     .                           R21-R13*R32*R22)*CBUFR(IORB1+13)+
     .                           2.D0*(R31*R21*R11-3.D0*R31*R22*R12+
     .                           R21*R31*R11-3.D0*R21*R32*R12+R11*
     .                           R31*R21-3.D0*R11*R32*R22)*
     .                           CBUFR(IORB1+14)+
     .                           2.D0*(3.D0*R32*R21*R11-R32*R22*R12+
     .                           3.D0*R22*R31*R11-R22*R32*R12+3.D0*
     .                           R12*R31*R21-R12*R32*R22)*
     .                           CBUFR(IORB1+15)
C fz(x2-y2)
                CREALR(IORB2+13)=3.D0*R33*(R1313-R2323)*
     .                           CBUFR(IORB1+9)+
     .                           (R31*R1313+2.D0*R33*(R11*R13-R21*
     .                           R23)-R31*R2323)*CBUFR(IORB1+10)+
     .                           (R32*R1313+2.D0*R33*(R12*R13-R22*
     .                           R23)-R32*R2323)*CBUFR(IORB1+11)+
     .                           (R13*R31*R12+R13*R11*R32+R11*R12*
     .                           R33-R23*R31*R22-R23*R21*R32-R21*
     .                           R22*R33)*CBUFR(IORB1+12)+
     .                           (R33*(R1111-R1212+R2222-R2121)+
     .                           2.D0*R13*(R31*R11-R32*R12)-2.D0*
     .                           R23*(R31*R21-R32*R22))*
     .                           CBUFR(IORB1+13)+
     .                           (R31*(R1111-3.D0*R1212+3.D0*R2222-
     .                           R2121)+2.D0*R11*(R31*R11-3.D0*R32*
     .                           R12)-2.D0*R21*(R31*R21-3.D0*R32*
     .                           R22))*CBUFR(IORB1+14)+
     .                           (R32*(3.D0*R1111-R1212+R2222-3.D0*
     .                           R2121)+2.D0*R12*(3.D0*R31*R11-R32*
     .                           R12)-2.D0*R22*(3.D0*R31*R21-R32*
     .                           R22))*CBUFR(IORB1+15)
                CREALR(IORB2+13)=0.5D0*CREALR(IORB2+13)
C fx(x2-3y2)
                CREALR(IORB2+14)=R13*(R1313-R2323)*CBUFR(IORB1+9)+
     .                           (R11*R1313-(2.D0*R21*R23*R13+R11*
     .                           R2323)/3.D0)*CBUFR(IORB1+10)+
     .                           (R12*R1313-(2.D0*R22*R23*R13+R12*
     .                           R2323)/3.D0)*CBUFR(IORB1+11)+
     .                           (R11*R12*R13-(R21*R22*R13+R21*R12*
     .                           R23+R11*R22*R23)/3.D0)*
     .                           CBUFR(IORB1+12)+
     .                           (R13*(R1111-R1212)-(2.D0*R23*R21*
     .                           R11-2.D0*R22*R12*R23+R13*R2121-R13*
     .                           R2222)/3.D0)*CBUFR(IORB1+13)+
     .                           (R11*(R1111-3.D0*R1212-R2121+
     .                           R2222)+2.D0*R21*R22*R12)*
     .                           CBUFR(IORB1+14)+
     .                           (R12*(3.D0*R1111-R1212+R2222-
     .                           R2121)-2.D0*R22*R21*R11)*
     .                           CBUFR(IORB1+15)
                CREALR(IORB2+14)=0.5D0*CREALR(IORB2+14)
C fy(3x2-y2)
                CREALR(IORB2+15)=R23*(R1313-R2323)*CBUFR(IORB1+9)+
     .                           (-R21*R2323+(2.D0*R11*R23*R13+R21*
     .                           R1313)/3.D0)*CBUFR(IORB1+10)+
     .                           (-R22*R2323+(2.D0*R12*R23*R13+R22*
     .                           R1313)/3.D0)*CBUFR(IORB1+11)+
     .                           (-R21*R22*R23+(R21*R12*R13+R11*R22*
     .                           R13+R11*R12*R23)/3.D0)*
     .                           CBUFR(IORB1+12)+
     .                           (-R23*(R2121-R2222)+(2.D0*R13*R21*
     .                           R11-2.D0*R13*R22*R12+R23*R1111-R23*
     .                           R1212)/3.D0)*CBUFR(IORB1+13)+
     .                           (-R21*(R2121-3.D0*R2222-R1111+
     .                           R1212)-2.D0*R11*R22*R12)*
     .                           CBUFR(IORB1+14)+
     .                           (-R22*(3.D0*R2121-R2222+R1212-
     .                           R1111)+2.D0*R12*R21*R11)*
     .                           CBUFR(IORB1+15)
                CREALR(IORB2+15)=0.5D0*CREALR(IORB2+15)
C fz3
                CIMAGR(IORB2+9)=R33*R3333*CBUFI(IORB1+9)+
     .                          R31*R3333*CBUFI(IORB1+10)+
     .                          R32*R3333*CBUFI(IORB1+11)+
     .                          R31*R32*R33*CBUFI(IORB1+12)+
     .                          R33*(R3131-R3232)*CBUFI(IORB1+13)+
     .                          R31*(R3131-3.D0*R3232)*
     .                          CBUFI(IORB1+14)+
     .                          R32*(3.D0*R3131-R3232)*
     .                          CBUFI(IORB1+15)
C fxz2
                CIMAGR(IORB2+10)=3.0D0*R3333*R13*CBUFI(IORB1+9)+
     .                           (2.D0*R31*R33*R13+R11*R3333)*
     .                           CBUFI(IORB1+10)+
     .                           (2.D0*R32*R33*R13+R12*R3333)*
     .                           CBUFI(IORB1+11)+
     .                           (R31*R32*R13+R31*R12*R33+R11*R32*
     .                           R33)*
     .                           CBUFI(IORB1+12)+
     .                           (2.D0*R33*R31*R11-2.0D0*R33*R32*
     .                           R12+R13*(R3131-R3232))*
     .                           CBUFI(IORB1+13)+
     .                           (2.D0*R3131*R11-6.D0*R31*R32*R12+
     .                           R11*(R3131-3.D0*R3232))*
     .                           CBUFI(IORB1+14)+
     .                           (6.D0*R32*R31*R11-2.D0*R3232*R12+
     .                           R12*(3.D0*R3131-R3232))*
     .                           CBUFI(IORB1+15)
C fyz2
                CIMAGR(IORB2+11)=3.D0*R3333*R23*CBUFI(IORB1+9)+
     .                           (2.D0*R31*R33*R23+R21*R3333)*
     .                           CBUFI(IORB1+10)+
     .                           (2.D0*R32*R33*R23+R22*R3333)*
     .                           CBUFI(IORB1+11)+
     .                           (R31*R32*R23+R31*R22*R33+R21*R32*
     .                           R33)*CBUFI(IORB1+12)+
     .                           (2.D0*R33*(R31*R21-R32*R22)+R23*
     .                           (R3131-R3232))*CBUFI(IORB1+13)+
     .                           (2.D0*R3131*R21-6.D0*R31*R32*R22+
     .                           R21*(R3131-3.D0*R3232))*
     .                           CBUFI(IORB1+14)+
     .                           (6.D0*R32*R31*R21-2.D0*R3232*R22+
     .                           R22*(3.D0*R3131-R3232))*
     .                           CBUFI(IORB1+15)
C fxyz
                CIMAGR(IORB2+12)=6.D0*R33*R23*R13*CBUFI(IORB1+9)+
     .                           2.D0*(R13*R31*R23+R13*R21*R33+
     .                           R11*R33*R23)*CBUFI(IORB1+10)+
     .                           2.D0*(R13*R32*R23+R13*R22*R33+
     .                           R12*R33*R23)*CBUFI(IORB1+11)+
     .                           (R13*R31*R22+R13*R21*R32+R23*R31*
     .                           R12+R23*R11*R32+R33*R21*R12+
     .                           R33*R11*R22)*CBUFI(IORB1+12)+
     .                           2.D0*(R33*R21*R11-R33*R22*R12+
     .                           R23*R31*R11-R23*R32*R12+R13*R31*
     .                           R21-R13*R32*R22)*CBUFI(IORB1+13)+
     .                           2.D0*(R31*R21*R11-3.D0*R31*R22*R12+
     .                           R21*R31*R11-3.D0*R21*R32*R12+R11*
     .                           R31*R21-3.D0*R11*R32*R22)*
     .                           CBUFI(IORB1+14)+
     .                           2.D0*(3.D0*R32*R21*R11-R32*R22*R12+
     .                           3.D0*R22*R31*R11-R22*R32*R12+3.D0*
     .                           R12*R31*R21-R12*R32*R22)*
     .                           CBUFI(IORB1+15)
C fz(x2-y2)
                CIMAGR(IORB2+13)=3.D0*R33*(R1313-R2323)*
     .                           CBUFI(IORB1+9)+
     .                           (R31*R1313+2.D0*R33*(R11*R13-R21*
     .                           R23)-R31*R2323)*CBUFI(IORB1+10)+
     .                           (R32*R1313+2.D0*R33*(R12*R13-R22*
     .                           R23)-R32*R2323)*CBUFI(IORB1+11)+
     .                           (R13*R31*R12+R13*R11*R32+R11*R12*
     .                           R33-R23*R31*R22-R23*R21*R32-R21*
     .                           R22*R33)*CBUFI(IORB1+12)+
     .                           (R33*(R1111-R1212+R2222-R2121)+
     .                           2.D0*R13*(R31*R11-R32*R12)-2.D0*
     .                           R23*(R31*R21-R32*R22))*
     .                           CBUFI(IORB1+13)+
     .                           (R31*(R1111-3.D0*R1212+3.D0*R2222-
     .                           R2121)+2.D0*R11*(R31*R11-3.D0*R32*
     .                           R12)-2.D0*R21*(R31*R21-3.D0*R32*
     .                           R22))*CBUFI(IORB1+14)+
     .                           (R32*(3.D0*R1111-R1212+R2222-3.D0*
     .                           R2121)+2.D0*R12*(3.D0*R31*R11-R32*
     .                           R12)-2.D0*R22*(3.D0*R31*R21-R32*
     .                           R22))*CBUFI(IORB1+15)
                CIMAGR(IORB2+13)=0.5D0*CIMAGR(IORB2+13)
C fx(x2-3y2)
                CIMAGR(IORB2+14)=R13*(R1313-R2323)*CBUFI(IORB1+9)+
     .                           (R11*R1313-(2.D0*R21*R23*R13+R11*
     .                           R2323)/3.D0)*CBUFI(IORB1+10)+
     .                           (R12*R1313-(2.D0*R22*R23*R13+R12*
     .                           R2323)/3.D0)*CBUFI(IORB1+11)+
     .                           (R11*R12*R13-(R21*R22*R13+R21*R12*
     .                           R23+R11*R22*R23)/3.D0)*
     .                           CBUFI(IORB1+12)+
     .                           (R13*(R1111-R1212)-(2.D0*R23*R21*
     .                           R11-2.D0*R22*R12*R23+R13*R2121-R13*
     .                           R2222)/3.D0)*CBUFI(IORB1+13)+
     .                           (R11*(R1111-3.D0*R1212-R2121+
     .                           R2222)+2.D0*R21*R22*R12)*
     .                           CBUFI(IORB1+14)+
     .                           (R12*(3.D0*R1111-R1212+R2222-
     .                           R2121)-2.D0*R22*R21*R11)*
     .                           CBUFI(IORB1+15)
                CIMAGR(IORB2+14)=0.5D0*CIMAGR(IORB2+14)
C fy(3x2-y2)
                CIMAGR(IORB2+15)=R23*(R1313-R2323)*CBUFI(IORB1+9)+
     .                           (-R21*R2323+(2.D0*R11*R23*R13+R21*
     .                           R1313)/3.D0)*CBUFI(IORB1+10)+
     .                           (-R22*R2323+(2.D0*R12*R23*R13+R22*
     .                           R1313)/3.D0)*CBUFI(IORB1+11)+
     .                           (-R21*R22*R23+(R21*R12*R13+R11*R22*
     .                           R13+R11*R12*R23)/3.D0)*
     .                           CBUFI(IORB1+12)+
     .                           (-R23*(R2121-R2222)+(2.D0*R13*R21*
     .                           R11-2.D0*R13*R22*R12+R23*R1111-R23*
     .                           R1212)/3.D0)*CBUFI(IORB1+13)+
     .                           (-R21*(R2121-3.D0*R2222-R1111+
     .                           R1212)-2.D0*R11*R22*R12)*
     .                           CBUFI(IORB1+14)+
     .                           (-R22*(3.D0*R2121-R2222+R1212-
     .                           R1111)+2.D0*R12*R21*R11)*
     .                           CBUFI(IORB1+15)
                CIMAGR(IORB2+15)=0.5D0*CIMAGR(IORB2+15)
              ENDIF
   40       CONTINUE
C
C ... then compare if sign of coefficients has changed. I numbers COs.
C
            ISYMRE(J,I)=1
            DO 50 L=1,NDIM
              IF (ABS(CREALR(L)-CBUFR(L)) .GT. DELTA) GOTO 60
              IF (ABS(CIMAGR(L)-CBUFI(L)) .GT. DELTA) GOTO 60
   50       CONTINUE
            GOTO 90
   60       ISYMRE(J,I)=-1
            DO 70 L=1,NDIM
              IF (ABS(-CREALR(L)-CBUFR(L)) .GT. DELTA) GOTO 80
              IF (ABS(-CIMAGR(L)-CBUFI(L)) .GT. DELTA) GOTO 80
   70       CONTINUE
            GOTO 90
C
C No match (C3, C6 axis: E symmetry).
C
   80       ISYMRE(J,I)=0
   90       CONTINUE
          ENDIF
C
C End of loop over symmetry operations (J).
C
  100   CONTINUE
C
C End of loop over COs (I).
C
  110 CONTINUE
      RETURN
      END
