C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  WRTKEY                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Collects all the key word options and prints them. *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 12.05.92/mbs, 22.05.92/mbs,             *
C *               25.09.92/mbs, 17.02.93/mbs                         *
C *               (Martin Braendle, c/o                              *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Inclusion of keyword for distance-dependent Wolfs- *
C *               berg-Helmholz approximation (s. G. Calzaferri et   *
C *               al, J. Phys. Chem. 93(1989),5366). 12.05.92/mbs    *
C *               Inclusion of keyword for twobody-electrostatic-    *
C *               core-core-repulsion (s. A.B. Anderson, R. Hoffmann,*
C *               J. Chem. Phys. 60(1974), 4271, and paper cited     *
C *               above.) 22.05.92/mbs                               *
C *               Inclusion of keywords for output of H(k) and S(k). *
C *               25.09.92/mbs                                       *
C *               Individual (kappa,delta)-parameter for atom pairs. *
C *               17.02.93/mbs                                       *
C *               Symmetry analysis of COs. 09.03.93/mbs             *
C *                                                                  *
C *  Called from: GETSEL                                             *
C *                                                                  *
C *  Calls......: ADDK                                               *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE WRTKEY
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      CHARACTER KEYTMP*320,MOT*12
 
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,IREST,IRESTV
      COMMON /OPTV2/IHKB,IHKE,ISKB,ISKE
 
      COMMON /DISTK/CKAPPA,CDELTA,AKAPPA(NATMMX,NATMMX),
     .              ADELTA(NATMMX,NATMMX)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      COMMON/SYMANA/TRANSV(MXSYME,3),SYMOP(MXSYME,3,3),
     .              IPERMU(MXSYME,NATMMX,2),ISYMRE(MXSYME,NDIMMX),
     .              NLINOP(3),IPOINT(3,MXSYME),ILINE(3),NSYMEL,
     .              CSYMEL(MXSYME),FSYMOP(MXSYME),FSYMAN(MXSYME),FSYM
      LOGICAL   FSYMOP,FSYM,FSYMAN
      CHARACTER CSYMEL*2
 
      COMMON /VARGEO/VARINC(NVARMX),IVRLIN(NVARMX),ISTPMX(NVARMX),
     .               IVARTP(NVARMX),ISTEPS(NVARMX),IVARY,NVARY,
     .               NVARLN,NNCPL,LCOUPL(NVARMX+1)
      LOGICAL LCOUPL
C
C RHODEF holds the default RHO value.
C RHOMAX holds the maximum cutoff value RHO can have.
C
      PARAMETER (RHOMAX=30.0D0,RHODEF=10.0D0)
C
C band
C
      WRITE(IOUT,10)
   10 FORMAT('    Options : ',/)
      IF (INDEX(KEYWRD,'BAND') .NE. 0) THEN
        WRITE(IOUT,20)
   20   FORMAT(' *  BAND      ',11X,'- Extended-Huckel Band ',
     .         'calculations')
      ELSE
        WRITE(IOUT,30)
   30   FORMAT(' *  EH        ',11X,'- Extended-Huckel calculations')
      ENDIF
C
C checkgeo
C
      IF (INDEX(KEYWRD,'CHECKGEO') .NE. 0) WRITE(IOUT,40)
   40 FORMAT(' *  CHECKGEO  ',11X,'- Check geometry only')
C
C xyz,zmat
C
      IF (INDEX(KEYWRD,'XYZ') .NE. 0) THEN
        WRITE(IOUT,50)
   50   FORMAT(' *  XYZ       ',11X,'- Cartesian coodinates as input ',
     .         'geometry')
      ELSE IF (INDEX(KEYWRD,'FRACT') .NE. 0) THEN
        WRITE(IOUT,55)
   55   FORMAT(' *  FRACT     ',11X,'- Fractional coodinates as input ',
     .         'geometry')
      ELSE
        IF (INDEX(KEYWRD,'ZMAT') .EQ. 0) THEN
          MOT='ZMAT'
          CALL ADDK(MOT)
        ENDIF
        WRITE(IOUT,60)
   60   FORMAT(' *  ZMAT      ',11X,'- Zmatrix as input geometry')
      ENDIF
C
C dist,distl,distn
C
      IF (INDEX(KEYWRD,'DIST') .NE. 0) WRITE(IOUT,70)
   70 FORMAT(' *  DIST      ',11X,'- Calculate interatomic distances')
 
      IF (INDEX(KEYWRD,'DISTM=') .NE. 0) THEN
        IP1=INDEX(KEYWRD,'DISTM=')+6
        DISTL=READA(KEYWRD,IP1,320)
        WRITE(IOUT,80) DISTL
   80 FORMAT(' *  DISTM = ',F10.3,'   - Limitation of the distance ',
     .         'maximum (dist. in A)')
      ENDIF
 
      IF (INDEX(KEYWRD,'DISTN') .NE. 0) THEN
        IP1=INDEX(KEYWRD,'DISTN=')+6
        IDISTN=INT(READA(KEYWRD,IP1,320))
        WRITE(IOUT,90) IDISTN
   90   FORMAT(' *  DISTN = ',I10,'   - Limitation of the number of ',
     .         'distances')
      ENDIF
C
C rho
C
      RHO=0.0D0
      IF (INDEX(KEYWRD,'RHO=') .NE. 0) THEN
        IP1=INDEX(KEYWRD,'RHO=')+4
        RHO=READA(KEYWRD,IP1,320)
        RHO=MIN(RHO,RHOMAX)
        WRITE(IOUT,100) RHO
  100   FORMAT(' *  RHO   = ',F10.3,
     .         '   - Restrict the range of overlap')
      ELSE
        RHO=RHODEF
      ENDIF
C
C force
C
      IF (INDEX(KEYWRD,'FORCE') .NE. 0) WRITE(IOUT,110)
  110 FORMAT(' *  FORCE     ',11X,'- Ignore test on the EH parameters')
C
C wHij, uwHij, distance dependent wHij, individual
C 12.05.92, 22.05.92/mbs
C
      IF (INDEX(KEYWRD,'KAPPA=') .EQ. 0) THEN
        CKAPPA=1.0D0
        IF (INDEX(KEYWRD,'DISDEPWH') .EQ. 0) CKAPPA =0.75D0
      ELSE
        IP1=INDEX(KEYWRD,'KAPPA=')+6
        CKAPPA=READA(KEYWRD,IP1,320)
      ENDIF
 
      IF (INDEX(KEYWRD,'KAPDEL') .NE. 0) THEN
        IF (INDEX(KEYWRD,'DISDEPWH') .EQ. 0) THEN
          DO 112 I=320,1,-1
            IF (KEYWRD(I:I) .NE. ' ') GOTO 115
  112     CONTINUE
  115     IP2=I
          IP1=INDEX(KEYWRD,'UWHIJ')
          IF (IP1 .GT. 0) THEN
            KEYTMP=KEYWRD(1:IP1-1)//KEYWRD(IP1+6:IP2)
            KEYWRD=KEYTMP
            IP2=IP2-6
          ENDIF
          IP1=INDEX(KEYWRD,'WHIJ')
          IF (IP1 .GT. 0) THEN
            KEYTMP=KEYWRD(1:IP1-1)//KEYWRD(IP1+5:IP2)
            KEYWRD=KEYTMP
          ENDIF
          MOT='DISDEPWH'
          CALL ADDK(MOT)
        ENDIF
      ENDIF
 
      IF ((INDEX(KEYWRD,'WHIJ') .NE. 0) .AND.
     .    (INDEX(KEYWRD,'UWHIJ') .EQ. 0)) THEN
        WRITE(IOUT,120) 1.0D0+CKAPPA
  120   FORMAT(' *  WHIJ      ',11X,'- Weighted H(i,j)',/,
     .         '      HUCKELCONSTANT =',F5.2)
      ELSE IF (INDEX(KEYWRD,'DISDEPWH') .NE. 0) THEN
        WRITE(IOUT,130)
  130   FORMAT(' *  DISDEPWH  ',11X,'- Distance dependent H(i,j)')
        CDELTA=0.35D0
        IP1=INDEX(KEYWRD,'DELTA=')+6
        IF (IP1 .GT. 6) CDELTA=READA(KEYWRD,IP1,320)
        WRITE(IOUT,131) 1.0D0+CKAPPA,CDELTA
  131   FORMAT('      1+KAPPA =',F5.2,/,'      DELTA   =',F5.2)

        IF (INDEX(KEYWRD,'KAPDEL1') .GT. 0) THEN
          WRITE(IOUT,133)
  133     FORMAT(' *  KAPDEL    ',11X,'- Individual (kappa,delta)')
        ENDIF
        IF (INDEX(KEYWRD,'KAPDEL2') .GT. 0) THEN
          WRITE(IOUT,134)
  134     FORMAT(' *  KAPDEL    ',11X,'- Individual atom specific ',
     .           '(kappa,delta)')
        ENDIF
      ELSE
        WRITE(IOUT,135) 1.0D0+CKAPPA
  135   FORMAT(' *  UWHIJ     ',11X,'- Unweighted H(i,j)',/,
     .         '      HUCKELCONSTANT =',F5.2)
      ENDIF
C
C REPUL
C 22.05.92/mbs
C
      IF (INDEX(KEYWRD,'REPUL') .NE. 0) THEN
        WRITE(IOUT,137)
  137   FORMAT(' *  REPUL     ',11X,'- Calculate twobody repulsion ',
     .         'energy')
        WRITE(IOUT,140)
        IF (INDEX(KEYWRD,'AUFBAU') .NE. 0) THEN
          WRITE(IOUT,138)
  138     FORMAT(27X,'use Aufbau principle for calculation of AO',/,
     .           27X,'occupation numbers')
        ELSE
          WRITE(IOUT,139)
  139     FORMAT(27X,'use atomic electron configurations (default)')
        ENDIF
      ENDIF
C
C EAOORB
C 05.06.92/mbs
C
      IF ((INDEX(KEYWRD,'EAOORB') .GT. 0) .AND.
     .    (INDEX(KEYWRD,'REPUL') .EQ. 0)) THEN
        WRITE(IOUT,140)
  140   FORMAT(' *  EAOORB    ',11X,'- Calculate sum over atomic ',
     .         'energy levels')
        IF (INDEX(KEYWRD,'AUFBAU') .NE. 0) THEN
          WRITE(IOUT,138)
        ELSE
          WRITE(IOUT,139)
        ENDIF
      ENDIF
C
C nelec
C
      IF (INDEX(KEYWRD,'NELEC=') .EQ. 0) THEN
        NELEC=0
        WRITE(IOUT,145)
  145   FORMAT(' *  NELEC     ',11X,'- Calculate number of electrons')
      ELSE
        IP1=INDEX(KEYWRD,'NELEC=')+6
        NELEC=INT(READA(KEYWRD,IP1,320))
        WRITE(IOUT,150) NELEC
  150   FORMAT(' *  NELEC = ',I10,'  - Number of electrons')
      ENDIF
C
C charge
C
      ICHARG=0
      IF (INDEX(KEYWRD,'CHARGE=') .NE. 0) THEN
        IP1=INDEX(KEYWRD,'CHARGE=')+7
        ICHARG=INT(READA(KEYWRD,IP1,320))
        WRITE(IOUT,160) ICHARG
  160   FORMAT(' *  CHARGE= ',I10,'  - Charge on the system')
      ENDIF
C
C state
C
      IF (INDEX(KEYWRD,'SINGLET') .NE. 0) WRITE(IOUT,170)
      IF (INDEX(KEYWRD,'DOUBLET') .NE. 0) WRITE(IOUT,180)
      IF (INDEX(KEYWRD,'TRIPLET') .NE. 0) WRITE(IOUT,190)
      IF (INDEX(KEYWRD,'QUARTET') .NE. 0) WRITE(IOUT,200)
      IF (INDEX(KEYWRD,'QUINTET') .NE. 0) WRITE(IOUT,210)
      IF (INDEX(KEYWRD,'SEXTET') .NE. 0) WRITE(IOUT,220)
  170 FORMAT(' *  SINGLET   ',11X,'- Singlet state required')
  180 FORMAT(' *  DOUBLET   ',11X,'- Doublet state required')
  190 FORMAT(' *  TRIPLET   ',11X,'- Triplet state required')
  200 FORMAT(' *  QUARTET   ',11X,'- Quartet state required')
  210 FORMAT(' *  QUINTET   ',11X,'- Quintet state required')
  220 FORMAT(' *  SEXTET    ',11X,'- Sextet state required')
C
C Ei limitation output
C
      IEB=0
      IEE=0
      IP1=INDEX(KEYWRD,'EOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+5
        IEB=INT(READA(KEYWRD,IP1,320))
  230   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 240
        IF (IP1 .GT. 319) GOTO 240
        GOTO 230
  240   IP1=IP1+2
        IEE=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'EOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,250) IEB,IEE
  250 FORMAT(' *  EOUT  = ',I3,' to ',I3,'   - Limitation of energy ',
     .       'output')
C
C Cij matrix limitation output
C
      LB=0
      LE=0
      IP1=INDEX(KEYWRD,'MOOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+6
        LB=INT(READA(KEYWRD,IP1,320))
  260   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 270
        IF (IP1 .GT. 319) GOTO 270
        GOTO 260
  270   IP1=IP1+2
        LE=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'MOOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,280) LB,LE
  280 FORMAT(' *  MOOUT = ',I3,' to ',I3,'  - Limitation of MO/FMO ',
     .       'output')
 
      IP1=INDEX(KEYWRD,'COOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+6
        LB=INT(READA(KEYWRD,IP1,320))
  290   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 300
        IF (IP1 .GT. 319) GOTO 300
        GOTO 290
  300   IP1=IP1+2
        LE=INT(READA(KEYWRD,IP1,320))
        IF (LE .LT. LB) LE=LB
      ELSE
        IF (INDEX(KEYWRD,'COOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,310) LB, LE
  310 FORMAT(' *  COOUT = ',I3,' to ',I3,'   - Limitation of CO/FMO ',
     .       'output')
 
      LBR=0
      LER=0
      IP1=INDEX(KEYWRD,'AOOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+6
        LBR=INT(READA(KEYWRD,IP1,320))
  320   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 330
        IF (IP1 .GT. 319) GOTO 330
        GOTO 320
  330   IP1=IP1+2
        LER=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'AOOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,340) LBR,LER
  340 FORMAT(' *  AOOUT = ',I3,' to ',I3,'  - Limitation of AO ',
     .       'output')
C
C Sij limitation output
C
      ISB=0
      ISE=0
      IP1=INDEX(KEYWRD,'SOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+5
        ISB=INT(READA(KEYWRD,IP1,320))
  350   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 360
        IF (IP1 .GT. 319) GOTO 360
        GOTO 350
  360   IP1=IP1+2
        ISE=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'SOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,370) ISB,ISE
  370 FORMAT(' *  SOUT  = ',I3,' to ',I3,'  - Limitation of overlap ',
     .       'output')
C
C Hij limitation output
C
      IHB=0
      IHE=0
      IP1=INDEX(KEYWRD,'HOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+5
        IHB=INT(READA(KEYWRD,IP1,320))
  380   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 390
        IF (IP1 .GT. 319) GOTO 390
        GOTO 380
  390   IP1=IP1+2
        IHE=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'HOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,400) IHB,IHE
  400 FORMAT(' *  HOUT  = ',I3,' to ',I3,'  - Limitation of Huckel ',
     .       'matrix output')
C
C grad
C
      IF (INDEX(KEYWRD,'GRAD') .NE. 0) WRITE(IOUT,410)
  410 FORMAT(' *  GRAD      ',11X,'- Calculate the orbital energy ',
     .       'gradients')
C
C FMO
C
      IF (INDEX(KEYWRD,'FMO') .NE. 0) WRITE(IOUT,420)
  420 FORMAT(' *  FMO       ',11X,'- Fragment molecular orbital ',
     .       'analysis')
C
C aopop
C
      IPOPE=0
      LIP1=INDEX(KEYWRD,'AOPOP=')
      IF (LIP1 .NE. 0) THEN
        LIP1=LIP1+6
        IPOPE=INT(READA(KEYWRD,LIP1,320))
      ENDIF
C
C fmopop
C
      JPOPE=0
      KIP1=INDEX(KEYWRD,'FMOPOP=')
      IF (KIP1 .NE. 0) THEN
        KIP1=KIP1+7
        JPOPE=INT(READA(KEYWRD,KIP1,320))
      ENDIF
C
C ovpop
C
      IOVLB=0
      IOVLE=0
      IP1=INDEX(KEYWRD,'OVLPOP=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+7
        IOVLB=INT(READA(KEYWRD,IP1,320))
  430   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 440
        IF (IP1 .GT. 319) GOTO 440
        GOTO 430
  440   IP1=IP1+2
        IOVLE=INT(READA(KEYWRD,IP1,320))
      ENDIF
C
C energy window for overlap calculation
C
      WSTART=0
      WEND=0
      IIP1=INDEX(KEYWRD,'WINDOW=')
      IF (IIP1 .NE. 0) THEN
        IIP1=IIP1+7
        WSTART=READA(KEYWRD,IIP1,320)
  450   IIP1=IIP1+1
        IF (KEYWRD(IIP1:IIP1+1) .EQ. 'TO') GOTO 460
        IF (IIP1 .GT. 319) GOTO 460
        GOTO 450
  460   IIP1=IIP1+2
        WEND=READA(KEYWRD,IIP1,320)
      ENDIF
      IF (WEND .LT. WSTART) WEND=WSTART+0.1
 
      IF (IIP1 .NE. 0) THEN
        IP1=0
        LIP1=0
        KIP1=0
      ENDIF
 
      IF (LIP1 .NE. 0) THEN
        WRITE(IOUT,470) IPOPE
  470   FORMAT(' *  AOPOP = ',I9,'  - Atomic orbital population ',
     .         'analysis (with its limitation)')
      ELSE
        IF (INDEX(KEYWRD,'AOPOP') .NE. 0) WRITE(IOUT,480)
  480   FORMAT(' *  AOPOP     ',11X,'- Atomic orbital population ',
     .         'analysis')
      ENDIF
 
      IF (KIP1 .NE. 0) THEN
        WRITE(IOUT,490) JPOPE
  490   FORMAT(' *  FMOPOP= ',I9,'  - Fragment molecular orbital ',
     .       'population analysis (with its limitation)')
      ELSE
        IF (INDEX(KEYWRD,'FMOPOP') .NE. 0) WRITE(IOUT,500)
  500   FORMAT(' *  FMOPOP    ',11X,'- Fragment molecular orbital ',
     .         'population analysis')
      ENDIF
 
      IF (IP1 .NE. 0) THEN
        WRITE(IOUT,510) IOVLB,IOVLE
  510   FORMAT(' *  OVLPOP=',I3,' to ',I3,'  - Overlap population ',
     .         'analysis in between band orbitals m and n')
      ELSE
        IF (INDEX(KEYWRD,'OVLPOP') .NE. 0) WRITE(IOUT,520)
  520   FORMAT(' *  OVLPOP    ',11X,'- Overlap population analysis')
      ENDIF
C
      IF (IIP1 .NE. 0) WRITE(IOUT,530) WSTART, WEND
  530 FORMAT(' *  WINDOW=',F10.6,/,'          to ',F10.6,
     .       '  - Energy window for overlap population analysis')
C
C fermi level
C
      FERMI=0.0
      IF (INDEX(KEYWRD,'FERMI=') .NE. 0) THEN
        IP1=INDEX(KEYWRD,'FERMI=')+6
        FERMI=READA(KEYWRD,IP1,320)
        WRITE(IOUT,540) FERMI
  540   FORMAT(' *  FERMI =',F10.6,'  - Fermi level')
      ENDIF
C
      IF (INDEX(KEYWRD,'CHECKFILE') .NE. 0) WRITE(IOUT,550)
  550 FORMAT(' *  CHECKFILE ',11X,'- Check the file and stop')
C
C restart
C
      IREST=0
      IRESTV=0
      IF (INDEX(KEYWRD,'RESTART=') .NE. 0) THEN
        IP1=INDEX(KEYWRD,'RESTART=')+8
        ITEMP=INT(READA(KEYWRD,IP1,320))
        IF (INDEX(KEYWRD,'VARIATION') .GT. 0) THEN
          IREST=NKMAX+1
          IRESTV=ITEMP
          WRITE(IOUT,560) IREST
  560     FORMAT(' *  RESTART =',I10,'  - Restarting geometry ',
     .           'variation #')
        ELSE
          IREST=ITEMP
          IRESTV=0
          WRITE(IOUT,570) IREST
  570     FORMAT(' *  RESTART =',I10,'  - Restarting k point #')
        ENDIF
      ENDIF
C
C bonds
C
      IF (INDEX(KEYWRD,'BOND') .NE. 0) WRITE(IOUT,580)
  580 FORMAT(' *  BOND      ',11X,'- File name.ehb created')
C
C polar
C
      IF (INDEX(KEYWRD,'POLAR') .NE. 0) WRITE(IOUT,590)
  590 FORMAT(' *  POLAR     ',11X,'- Polar orbital components')
C
C H(k) output (and limitation)
C
      IHKB=0
      IHKE=0
      IP1=INDEX(KEYWRD,'HKOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+5
        IHKB=INT(READA(KEYWRD,IP1,320))
  600   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 610
        IF (IP1 .GT. 319) GOTO 610
        GOTO 600
  610   IP1=IP1+2
        IHKE=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'HKOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,620) IHKB,IHKE
  620 FORMAT(' *  HKOUT = ',I3,' to ',I3,'  - Limitation of H(k) ',
     .       'matrix output')
C
C S(k) output (and limitation)
C
      ISKB=0
      ISKE=0
      IP1=INDEX(KEYWRD,'SKOUT=')
      IF (IP1 .NE. 0) THEN
        IP1=IP1+5
        ISKB=INT(READA(KEYWRD,IP1,320))
  630   IP1=IP1+1
        IF (KEYWRD(IP1:IP1+1) .EQ. 'TO') GOTO 640
        IF (IP1 .GT. 319) GOTO 640
        GOTO 630
  640   IP1=IP1+2
        ISKE=INT(READA(KEYWRD,IP1,320))
      ELSE
        IF (INDEX(KEYWRD,'SKOUT') .NE. 0) IP1=1
      ENDIF
      IF (IP1 .NE. 0) WRITE(IOUT,650) ISKB,ISKE
  650 FORMAT(' *  SKOUT = ',I3,' to ',I3,'  - Limitation of S(k) ',
     .       'output')
 
C
C Symmetry analysis of crystal orbitals.
C Clear result matrix.
C
      DO 670 I=1,MXSYME
        DO 660 J=1,NDIMMX
          ISYMRE(I,J)=0
  660   CONTINUE
  670 CONTINUE
 
      IF (INDEX(KEYWRD,'SYMMETRY') .GT. 0) THEN
        FSYM=.TRUE.
        WRITE(IOUT,680)
  680   FORMAT(' *  SYMMETRY  ',11X,'- Symmetry analysis of crystal ',
     .         'orbitals')
      ENDIF
C
C Geometry variation.
C
      NVAR=1
      IF (INDEX(KEYWRD,'VARIATION') .GT. 0) THEN
        WRITE(IOUT,690)
  690   FORMAT(' *  VARIATION ',11X,'- Geometry variation')
        NVARY=0
        IF (INDEX(KEYWRD,'REPUL') .EQ. 0) THEN
          MOT='REPUL'
          CALL ADDK(MOT)
        ENDIF
      ENDIF
C
C Output of orbital radii
C
      IF (INDEX(KEYWRD,'ORBRADOUT') .GT. 0) THEN
        WRITE(IOUT,700)
  700   FORMAT(' *  ORBRADOUT ',11X,'- Print orbital radii')
      ENDIF     
C
C Output of a file per k-point containing geometry and crystal orbitals 
C in convenient way for the use along with the EDiT program to calculate
C oscillator strength at different k-points.
C
      IF (INDEX(KEYWRD,'EDITOUT') .GT. 0) THEN
        WRITE(IOUT,710)
  710   FORMAT(' *  EDITOUT   ',11X,'- Print one EDiT file per k-point')
      ENDIF
      RETURN
      END