C ********************************************************************
C * EHPC - Extended Hueckel properties calculations                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GPROJB                                                          *
C *                                                                  *
C *  Routine of EHPC                                                 *
C *                                                                  *
C *  Usage......: Obtains information for OVLPOP projections.        *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 20.09.94/mbs (Martin Braendle c/o       *
C *               Calzaferri group, Institute for Inorganic and Phy- *
C *               sical Chemistry, Freiestr. 3, CH-3012 Berne)       *
C *  Remarks....: Changed heavily for multiple geometry variations.  *
C *               The number of bonds for each overlap projection    *
C *               will now be determined in subroutines PROJTB and   *
C *               SMOOTB, because distances may vary with varying    *
C *               geometry.                                          *  
C *                                                                  *
C *  Called from: PPART                                              *
C *                                                                  *
C *  Calls......: WRITEL,GETNAM,READL,READA                          *
C *                                                                  *
C ********************************************************************

      SUBROUTINE GPROJB

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      CHARACTER*7   ACI1,ACI2,ACC,NAMT1(NATMMX),NAMT2(NATMMX)
      CHARACTER*80  STRING,LINE
      CHARACTER*131 LL

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /ATOM1/NSS(NSYMMX),NPP(NSYMMX),NDD(NSYMMX),NFF(NSYMMX),
     .              IATOM,ISYM(NATMMX),NELEM

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /OPTV/DELTA,EMIN,EMAX,STEP,XLIMIT,LEVB,LEVE,NMESH,IVARYS,
     .             IVARYE,NVARY
      COMMON /OVLP/IPROJB(NBONMX,MXCARD),NDPRB(MXCARD),NPROJB,
     .             NBTOT(MXCARD)

      COMMON /OVLPRO/DTMIN(NBONMX,MXCARD),DTMAX(NBONMX,MXCARD),
     .               IDMAX(MXCARD),NOM1(NBONMX,MXCARD),
     .               NOM2(NBONMX,MXCARD)
      CHARACTER*7 NOM1,NOM2

      COMMON /TIT1/NCARD(MXCARD),NCARDF(MXCARD),NCARDB(MXCARD)

      COMMON /TIT2/TITL(MXCARD,MXTITL),TITLF(MXCARD,MXTITL),
     .             TITLB(MXCARD,MXTITL)
      CHARACTER TITL*80,TITLF*80,TITLB*80

      DO 10 I=1,MXCARD
        IDMAX(I)=0
   10 CONTINUE
      NPROJB=0
      IF (INDEX(SLINE,'OVLPROJ') .EQ. 0) THEN
        LINE='Cards OVLPROJ expected.'
        CALL WRITEL(LINE)
        RETURN
      ENDIF
      WRITE(IOUT,20)
   20 FORMAT(' ')

   30 IF (INDEX(SLINE,'OVLPROJ') .EQ. 0) GOTO 190
      NPROJB=NPROJB+1
      IF (NPROJB .GT. MXCARD) THEN
        LINE='Number of projections limited.'
        CALL WRITEL(LINE)
      ENDIF
   40 IF (INDEX(SLINE,'OVLPROJ') .EQ. 0) GOTO 120
      NUMPRO=INT(READA(SLINE,8,80))
      IF (NUMPRO .NE. NPROJB) GOTO 120
      NCARDB(NPROJB)=NCARDB(NPROJB)+1
      TITLB(NPROJB,NCARDB(NPROJB))=SLINE(9:80)
      NBTOT(NPROJB)=0

      IT=0
      CALL GETNAM(2,STRING,IT)
      ACI1=STRING(1:7)
      ACI2=ACI1
      NAT1=0
      DO 60 I=1,IATOM
        ACC=AC(I)
        IF (STRING(1:2) .EQ. 'XX') ACI1=AC(I)
        DO 50 K=1,7
          IF (K .GT. 2) THEN
            IF (ACC(K:K) .NE. ACI1(K:K) .AND. ACI1(K:K) .NE. ' ')
     .        GOTO 60
          ELSE
            IF (ACC(K:K) .NE. ACI1(K:K)) GOTO 60
          ENDIF
   50   CONTINUE
        NAT1=NAT1+1
        NAMT1(NAT1)=AC(I)
   60 CONTINUE

      IT=0
      CALL GETNAM(4,STRING,IT)
      ACI1=STRING(1:7)
      ACI2=ACI1
      NAT2=0
      DO 80 I=1,IATOM
        ACC=AC(I)
        IF (STRING(1:2) .EQ. 'XX') ACI1=AC(I)
        DO 70 K=1,7
          IF (K .GT. 2) THEN
            IF (ACC(K:K) .NE. ACI1(K:K) .AND. ACI1(K:K) .NE. ' ') 
     .        GOTO 80
          ELSE
            IF (ACC(K:K) .NE. ACI1(K:K)) GOTO 80
          ENDIF
   70   CONTINUE
        NAT2=NAT2+1
        NAMT2(NAT2)=AC(I)
   80 CONTINUE

      ID=0
      DMIN=READA(SLINE,ISTART(6),80)
      DMAX=READA(SLINE,ISTART(8),80)
      DO 90 I=1,NAT1
        DO 85 J=1,NAT2
          ID=ID+1
          NOM1(ID,NPROJB)=NAMT1(I)
          NOM2(ID,NPROJB)=NAMT2(J)
          DTMIN(ID,NPROJB)=DMIN
          DTMAX(ID,NPROJB)=DMAX
   85   CONTINUE
   90 CONTINUE
      IDMAX(NPROJB)=ID

      LL=' OVLPROJ # '
      DO 100 I=8,80
        J=I+4
        LL(J:J)=SLINE(I:I)
  100 CONTINUE
      WRITE(IOUT,110) LL
  110 FORMAT(1X,131A)
      CALL READL
      GOTO 40

  120 WRITE(IOUT,20)
      GOTO 30
  190 CONTINUE
      RETURN
      END
