C ********************************************************************
C * EHPC - Extended Hueckel properties calculations                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SMOOTO                                                          *
C *                                                                  *
C *  Routine of EHPC                                                 *
C *                                                                  *
C *  Usage......: Applies gaussian smoothing to generate the AO and/ *
C *               or FMO contribution to the density of states (i.e. *
C *               projected DOS) as a function of energy (for a set  *
C *               of given mesh points) for plotting purposes.       *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 31.01.92/mbs, 14.10.93/mbs, 21.9.94/mbs *
C *               (Martin Braendle c/o Calzaferri group, Institute   *
C *               for Inorganic and Physical Chemistry, Freiestr. 3, *
C *               CH-3012 Berne)                                     *
C *  Remarks....: Formatted output will be generated.                *
C *               Smoothed integrated PDOS implemented.              *
C *               Geometry variations implemented.                   *
C *                                                                  *
C *  Called from: PPART                                              *
C *                                                                  *
C *  Calls......: WRITEL,OUTPUT                                      *
C *                                                                  *
C ********************************************************************

      SUBROUTINE SMOOTO(E,A,EDX,XND,XNDI,NDIM,NKP,XNORM,IPROJ,NDPR,
     .                  NPROJ,NDK,LUNIT)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION E(NDIM),A(NDIM,NDIM),EDX(NMESH),XND(NMESH),XNDI(NMESH),
     .          IPROJ(NDIMMX,NFMOMX),NDPR(NFMOMX),XK(3)
      CHARACTER CDUMMY*80

      COMMON /ATOM1/NSS(NSYMMX),NPP(NSYMMX),NDD(NSYMMX),NFF(NSYMMX),
     .              IATOM,ISYM(NATMMX),NELEM
      COMMON /ENERG/IENERG
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /OPTV/DELTA,EMIN,EMAX,STEP,XLIMIT,LEVB,LEVE,NMESH,IVARYS,
     .             IVARYE,NVARY

      IF (DELTA .EQ. 0.0D0) DELTA=PDELTA
      IF (XLIMIT .EQ. 0.0D0) XLIMIT=PLIMIT
      ELIM=-LOG(XLIMIT)
      ELIM=SQRT(ELIM)*DELTA

      XSTEP=DABS(EMAX-EMIN)/DBLE(NMESH-1)
      DO 10 I=1,NMESH
        EDX(I)=EMIN+XSTEP*DBLE(I-1)
   10 CONTINUE

      REWIND 1
      REWIND LUNIT
      REWIND 8

      DO 30 I=1,NDIM+NELEM+IATOM+1
        READ(1,20) CDUMMY
   20   FORMAT(A80)
   30 CONTINUE
      IF (IVARYS .GT. 1) THEN
        DO 90 IVARCT=1,IVARYS-1
          DO 40 IKK=1,NKP*(NDIM+1)+NDK+1
            READ(1,20) CDUMMY
   40     CONTINUE
          DO 80 IKK=1,NKP
            READ(LUNIT,50) IPOP
   50       FORMAT(I6)
            DO 70 I=1,NDIM
              READ(LUNIT,60) (A(I,J),J=1,IPOP)
C
C old version files (prior to 29.06.93)/mbs
C
C   60         FORMAT(F10.5)
C
   60         FORMAT(F8.5)
   70       CONTINUE
   80     CONTINUE
   90   CONTINUE
      ENDIF

      DO 140 IKK=1,NKP
        READ(1,100) (XK(I),I=1,3),WEI
  100   FORMAT(4F12.6)
        WRITE(8,110) WEI
  110   FORMAT(F12.6)
        DO 120 I=1,NDIM
          READ(1,110) E(I)
          WRITE(8,110) E(I)
  120   CONTINUE
        READ(LUNIT,50) IPOP
        WRITE(8,50) IPOP
        DO 130 I=1,NDIM
          READ(LUNIT,60) (A(I,J),J=1,IPOP)
          WRITE(8,60) (A(I,J),J=1,IPOP)
  130   CONTINUE
  140 CONTINUE

      DO 470 IPRO=1,NPROJ
C
C Initialization
C
        REWIND 8
        DO 160 I=1,NMESH
          XND(I)=0.0D0
          XNDI(I)=0.0D0
  160   CONTINUE

        XBE=0.0D0
        DO 390 IKK=1,NKP
          READ(8,110) WEI
          DO 170 I=1,NDIM
            READ(8,110) E(I)
  170     CONTINUE

          READ(8,50) IPOP
          DO 180 I=1,NDIM
            READ(8,60) (A(I,J),J=1,IPOP)
  180     CONTINUE

          WEI=WEI*XNORM
C
C Look for the last point
C
          IRUN=NMESH
          N3=NDIM
          N1=1
  190     N2=(N3+N1)/2
          IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 200
          IF (E(N2) .GT. EDX(IRUN)) THEN
            N3=N2
          ELSE
            N1=N2
          ENDIF
          GOTO 190
  200     NEND=N2
          IF (E(N2) .GT. EDX(IRUN)) NEND=NEND-1
C
C Look for the first point
C
          IRUN=1
          N3=NDIM
          N1=1
  210     N2=(N3+N1)/2
          IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 220
          IF (E(N2) .GT. EDX(IRUN)) THEN
            N3=N2
          ELSE
            N1=N2
          ENDIF
          GOTO 210
  220     NSTART=N2
          IF (E(N2) .LT. EDX(IRUN)) NSTART=NSTART+1
          NST=NSTART
C
C XNADDB: Tail area of PDOS gaussian below EMIN
C
          XNADDB=0.0D0
C
C Fit gaussian slopes onto energy mesh.
C IRUN runs through all energy mesh points.
C
          IRUN=0
  230     IRUN=IRUN+1
          IF (IRUN .GT. NMESH) GOTO 320
          CVAL=EDX(IRUN)
          IF (IRUN .GT. 1) THEN
            DO 240 IP=NST,NDIM
              IF (E(IP) .GE. CVAL) GOTO 250
              NST=NST+1
  240       CONTINUE
          ENDIF
  250     NCOU=NST
C
C Count levels NCOU until the height of the gaussian slope lies below
C LIMIT, which is expressed by the energy difference DIF larger than
C ELIM. Right k-space part of energy level CVAL.
C
  260     DIF=ABS(E(NCOU)-CVAL)
          IF (DIF .GT. ELIM) GOTO 280
          IF (NCOU .GT. IPOP) THEN
            IF (LUNIT .EQ. 2) SLINE='The smoothing requires levels'//
     .        'above the limitation AOPOP=n of EHMACC.'
            IF (LUNIT .EQ. 4) SLINE='The smoothing requires levels'//
     .        'above the limitation FMOPOP=n of EHMACC.'
            CALL WRITEL(SLINE)
            CALL CLEXIT
          ENDIF
          Z=DIF/DELTA
          Z=-Z*Z
          Z=DEXP(Z)*WEI
          DO 270 N=1,NDPR(IPRO)
            NN=IPROJ(N,IPRO)
            XND(IRUN)=XND(IRUN)+A(NN,NCOU)*Z
  270     CONTINUE
          NCOU=NCOU+1
          IF (NCOU .LE. NDIM) GOTO 260
C
C Count levels NCOU until the height of the gaussian slope lies below
C LIMIT, which is expressed by the energy difference DIF larger than
C ELIM. Left k-space part of energy level CVAL.
C
  280     NCOU=NST-1
  290     IF (NCOU .LE. 0) GOTO 310
          DIF=ABS(E(NCOU)-CVAL)
          IF (DIF .GT. ELIM) GOTO 310
          Z=DIF/DELTA
          Z=-Z*Z
          Z=DEXP(Z)*WEI
          DO 300 N=1,NDPR(IPRO)
            NN=IPROJ(N,IPRO)
            XND(IRUN)=XND(IRUN)+A(NN,NCOU)*Z
            IF (NCOU .LT. NSTART) THEN
              XNADDB=XNADDB+A(NN,NCOU)*Z
            ENDIF
  300     CONTINUE
          NCOU=NCOU-1
          GOTO 290
  310     GOTO 230
  320     CONTINUE
C
C XNLTB: Projected DOS below EMIN
C
          XNLTB=0.0D0
C
C Determine projected DOS below EMIN by the above procedure.
C
          NCOU=-1
          XRUN=EDX(1)
  330     XRUN=XRUN-XSTEP
          IF (NCOU .EQ. NSTART) GOTO 360
          NCOU=NSTART
  340     DIF=ABS(E(NCOU)-XRUN)
          IF (DIF .GT. ELIM) GOTO 330
          Z=DIF/DELTA
          Z=-Z*Z
          Z=DEXP(Z)*WEI
          DO 350 N=1,NDPR(IPRO)
            NN=IPROJ(N,IPRO)
            XNLTB=XNLTB+A(NN,NCOU)*Z
  350     CONTINUE
          NCOU=NCOU+1
          IF (NCOU .GT. NDIM) GOTO 330
          GOTO 340
  360     CONTINUE
          DO 380 I=1,NSTART-1
            DO 370 N=1,NDPR(IPRO)
              NN=IPROJ(N,IPRO)
              XBE=XBE+WEI/XNORM*A(NN,I)
  370       CONTINUE
  380     CONTINUE
  390   CONTINUE
        XBE=XBE+(XNLTB-XNADDB)*XSTEP
C
C Sum up projected DOS.
C
        DO 400 I=1,NMESH
          XBE=XBE+XSTEP*XND(I)
          XNDI(I)=XBE
  400   CONTINUE
C
C Energy mesh
C
        IF (IENERG .EQ. 0) THEN
          WRITE(IOUT,410)
  410     FORMAT(//////,5X,'Energy mesh points',//)
          CALL OUTPUT (EDX,NMESH)
          IENERG=1
          WRITE(7,420) EDX
  420     FORMAT(F12.5)
        ENDIF
C
C Output of AOPOP and FMOPOP
C
        IF (LUNIT .EQ. 2) WRITE(IOUT,430) IPRO
  430   FORMAT(//////,5X,'Smoothed AO population for the projection #',
     .         I2,//)
        IF (LUNIT .EQ. 4) WRITE(IOUT,440) IPRO
  440   FORMAT(//////,5X,'Smoothed FMO population for the projection #',
     .         I2,//)
        CALL OUTPUT(XND,NMESH)
        WRITE(7,420) XND
        IF (LUNIT .EQ. 2) WRITE(IOUT,450) IPRO
  450   FORMAT(//////,5X,'Smoothed integrated AO population for the ',
     .         'projection #',I2,//)
        IF (LUNIT .EQ. 4) WRITE(IOUT,460) IPRO
  460   FORMAT(//////,5X,'Smoothed integrated FMO population for the ',
     .         'projection #',I2,//)
        CALL OUTPUT(XNDI,NMESH)
        WRITE(7,420) XNDI
  470 CONTINUE
      RETURN
      END
